/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.api.capability.forge;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.common.capabilities.Capability;

import net.dries007.tfc.api.capability.heat.ItemHeatHandler;
import net.dries007.tfc.util.forge.ForgeStep;
import net.dries007.tfc.util.forge.ForgeSteps;

public class ForgeableHandler extends ItemHeatHandler implements IForgeable
{
    protected final ForgeSteps steps;
    protected int work;
    protected ResourceLocation recipeName;

    public ForgeableHandler(@Nullable NBTTagCompound nbt, float heatCapacity, float meltTemp)
    {
        this.heatCapacity = heatCapacity;
        this.meltTemp = meltTemp;

        steps = new ForgeSteps();

        deserializeNBT(nbt);
    }

    public ForgeableHandler()
    {
        // for custom implementations
        steps = new ForgeSteps();
    }

    @Override
    public int getWork()
    {
        return work;
    }

    @Override
    public void setWork(int work)
    {
        this.work = work;
    }

    @Override
    @Nullable
    public ResourceLocation getRecipeName()
    {
        return recipeName;
    }

    @Override
    public void setRecipe(@Nullable ResourceLocation recipeName)
    {
        this.recipeName = recipeName;
    }

    @Override
    @Nonnull
    public ForgeSteps getSteps()
    {
        return steps;
    }

    @Override
    public void addStep(ForgeStep step)
    {
        steps.addStep(step);
        work += step.getStepAmount();
    }

    @Override
    public void reset()
    {
        steps.reset();
        recipeName = null;
        work = 0;
    }

    @Override
    public boolean hasCapability(@Nonnull Capability<?> capability, @Nullable EnumFacing facing)
    {
        return capability == CapabilityForgeable.FORGEABLE_CAPABILITY || super.hasCapability(capability, facing);
    }

    @SuppressWarnings("unchecked")
    @Nullable
    @Override
    public <T> T getCapability(@Nonnull Capability<T> capability, @Nullable EnumFacing facing)
    {
        return hasCapability(capability, facing) ? (T) this : null;
    }

    @Override
    @Nonnull
    public NBTTagCompound serializeNBT()
    {
        NBTTagCompound nbt = super.serializeNBT();

        nbt.func_74768_a("work", work);
        nbt.func_74782_a("steps", steps.serializeNBT());
        if (recipeName != null)
        {
            nbt.func_74778_a("recipe", recipeName.toString());
        }

        return nbt;
    }

    @Override
    public void deserializeNBT(@Nullable NBTTagCompound nbt)
    {
        if (nbt != null)
        {
            work = nbt.func_74762_e("work");
            recipeName = nbt.func_74764_b("recipe") ? new ResourceLocation(nbt.func_74779_i("recipe")) : null; // stops defaulting to empty string
            steps.deserializeNBT(nbt.func_74775_l("steps"));
        }
        super.deserializeNBT(nbt);
    }
}
