/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import net.minecraft.block.Block;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.GuiCreateWorld;
import net.minecraft.client.gui.inventory.GuiInventory;
import net.minecraft.client.gui.recipebook.GuiButtonRecipe;
import net.minecraft.client.resources.I18n;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.WorldType;
import net.minecraft.world.chunk.Chunk;
import net.minecraftforge.client.event.GuiScreenEvent;
import net.minecraftforge.client.event.RenderGameOverlayEvent;
import net.minecraftforge.client.event.TextureStitchEvent;
import net.minecraftforge.event.entity.player.ItemTooltipEvent;
import net.minecraftforge.fml.client.registry.RenderingRegistry;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.eventhandler.EventPriority;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.oredict.OreDictionary;

import net.dries007.tfc.ConfigTFC;
import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.capability.food.CapabilityFood;
import net.dries007.tfc.api.capability.food.IFood;
import net.dries007.tfc.api.capability.heat.CapabilityItemHeat;
import net.dries007.tfc.api.capability.heat.IItemHeat;
import net.dries007.tfc.api.capability.size.CapabilityItemSize;
import net.dries007.tfc.api.capability.size.IItemSize;
import net.dries007.tfc.api.util.IMetalObject;
import net.dries007.tfc.api.util.IRockObject;
import net.dries007.tfc.client.button.GuiButtonPlayerInventoryTab;
import net.dries007.tfc.client.render.RenderFallingBlockTFC;
import net.dries007.tfc.client.render.animal.*;
import net.dries007.tfc.client.render.projectile.RenderThrownJavelin;
import net.dries007.tfc.network.PacketSwitchPlayerInventoryTab;
import net.dries007.tfc.objects.entity.EntityFallingBlockTFC;
import net.dries007.tfc.objects.entity.animal.*;
import net.dries007.tfc.objects.entity.projectile.EntityThrownJavelin;
import net.dries007.tfc.util.calendar.CalendarTFC;
import net.dries007.tfc.util.calendar.Month;
import net.dries007.tfc.world.classic.ClimateRenderHelper;
import net.dries007.tfc.world.classic.ClimateTFC;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataProvider;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataTFC;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;
import static net.minecraft.util.text.TextFormatting.*;

@Mod.EventBusSubscriber(value = Side.CLIENT, modid = MOD_ID)
public class ClientEvents
{
    public static void preInit()
    {
        RenderingRegistry.registerEntityRenderingHandler(EntityFallingBlockTFC.class, RenderFallingBlockTFC::new);
        RenderingRegistry.registerEntityRenderingHandler(EntityThrownJavelin.class, RenderThrownJavelin::new);
        RenderingRegistry.registerEntityRenderingHandler(EntitySheepTFC.class, RenderSheepTFC::new);
        RenderingRegistry.registerEntityRenderingHandler(EntityCowTFC.class, RenderCowTFC::new);
        RenderingRegistry.registerEntityRenderingHandler(EntityBearTFC.class, RenderBearTFC::new);
        RenderingRegistry.registerEntityRenderingHandler(EntityChickenTFC.class, RenderChickenTFC::new);
        RenderingRegistry.registerEntityRenderingHandler(EntityPheasantTFC.class, RenderPheasantTFC::new);
        RenderingRegistry.registerEntityRenderingHandler(EntityDeerTFC.class, RenderDeerTFC::new);
        RenderingRegistry.registerEntityRenderingHandler(EntityPigTFC.class, RenderPigTFC::new);
        RenderingRegistry.registerEntityRenderingHandler(EntityWolfTFC.class, RenderWolfTFC::new);
        RenderingRegistry.registerEntityRenderingHandler(EntityRabbitTFC.class, RenderRabbitTFC::new);
    }

    @SideOnly(Side.CLIENT)
    @SubscribeEvent(priority = EventPriority.HIGH)
    public static void onInitGuiPre(GuiScreenEvent.InitGuiEvent.Pre event)
    {
        if (ConfigTFC.CLIENT.makeWorldTypeClassicDefault && event.getGui() instanceof GuiCreateWorld)
        {
            GuiCreateWorld gui = ((GuiCreateWorld) event.getGui());
            // Only change if default is selected, because coming back from customisation, this will be set already.
            if (gui.field_146331_K == WorldType.field_77137_b.func_82747_f())
            {
                gui.field_146331_K = TerraFirmaCraft.getWorldTypeTFC().func_82747_f();
            }
        }
    }

    @SideOnly(Side.CLIENT)
    @SubscribeEvent
    public static void onInitGuiPost(GuiScreenEvent.InitGuiEvent.Post event)
    {
        if (event.getGui() instanceof GuiInventory)
        {
            int buttonId = event.getButtonList().size();
            int guiLeft = ((GuiInventory) event.getGui()).getGuiLeft();
            int guiTop = ((GuiInventory) event.getGui()).getGuiTop();

            event.getButtonList().add(new GuiButtonPlayerInventoryTab(TFCGuiHandler.Type.INVENTORY, guiLeft, guiTop, ++buttonId, false));
            event.getButtonList().add(new GuiButtonPlayerInventoryTab(TFCGuiHandler.Type.SKILLS, guiLeft, guiTop, ++buttonId, true));
            event.getButtonList().add(new GuiButtonPlayerInventoryTab(TFCGuiHandler.Type.CALENDAR, guiLeft, guiTop, ++buttonId, true));
            event.getButtonList().add(new GuiButtonPlayerInventoryTab(TFCGuiHandler.Type.NUTRITION, guiLeft, guiTop, ++buttonId, true));
        }
    }

    @SideOnly(Side.CLIENT)
    @SubscribeEvent
    public static void onGuiButtonPress(GuiScreenEvent.ActionPerformedEvent.Post event)
    {
        if (event.getGui() instanceof GuiInventory && event.getButton() instanceof GuiButtonPlayerInventoryTab)
        {
            // This should generally be true, but check just in case something has disabled it
            GuiButtonPlayerInventoryTab button = (GuiButtonPlayerInventoryTab) event.getButton();
            if (button.isActive())
            {
                TerraFirmaCraft.getNetwork().sendToServer(new PacketSwitchPlayerInventoryTab(button.getGuiType()));
            }
        }
        else if (event.getGui() instanceof GuiInventory && event.getButton() instanceof GuiButtonRecipe)
        {
            // This is necessary to catch the resizing of the inventory gui when you open the recipe book
            for (GuiButton button : event.getButtonList())
            {
                if (button instanceof GuiButtonPlayerInventoryTab)
                {
                    ((GuiButtonPlayerInventoryTab) button).updateGuiLeft(((GuiInventory) event.getGui()).getGuiLeft());
                }
            }
        }
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public static void onRenderGameOverlayText(RenderGameOverlayEvent.Text event)
    {
        Minecraft mc = Minecraft.func_71410_x();
        List<String> list = event.getRight();
        if (ConfigTFC.GENERAL.debug && mc.field_71474_y.field_74330_P)
        {
            //noinspection ConstantConditions
            BlockPos blockpos = new BlockPos(mc.func_175606_aa().field_70165_t, mc.func_175606_aa().func_174813_aQ().field_72338_b, mc.func_175606_aa().field_70161_v);
            Chunk chunk = mc.field_71441_e.func_175726_f(blockpos);
            if (mc.field_71441_e.func_175667_e(blockpos) && !chunk.func_76621_g())
            {
                final int x = blockpos.func_177958_n() & 15, z = blockpos.func_177952_p() & 15;
                ChunkDataTFC data = chunk.getCapability(ChunkDataProvider.CHUNK_DATA_CAPABILITY, null);

                list.add("");
                list.add(AQUA + "TerraFirmaCraft");

                if (data == null || !data.isInitialized()) list.add("No data ?!");
                else
                {
                    list.add(String.format("%sRegion: %s%.1f\u00b0C%s Avg: %s%.1f\u00b0C%s Min: %s%.1f\u00b0C%s Max: %s%.1f\u00b0C",
                        GRAY, WHITE, data.getRegionalTemp(), GRAY,
                        WHITE, data.getAverageTemp(), GRAY,
                        WHITE, ClimateTFC.monthTemp(data.getRegionalTemp(), Month.JANUARY, blockpos.func_177952_p()), GRAY,
                        WHITE, ClimateTFC.monthTemp(data.getRegionalTemp(), Month.JULY, blockpos.func_177952_p())));
                    list.add(String.format("%sTemperature: %s%.1f\u00b0C Daily: %s%.1f\u00b0C",
                        GRAY, WHITE, ClimateRenderHelper.get(blockpos).getTemperature(),
                        WHITE, ClimateTFC.getHeightAdjustedTemp(mc.field_71441_e, blockpos)));

                    list.add(I18n.func_135052_a("tfc.tooltip.date", CalendarTFC.CALENDAR_TIME.getTimeAndDate()));
                    list.add(I18n.func_135052_a("tfc.tooltip.debug_times", CalendarTFC.TOTAL_TIME.getTicks(), CalendarTFC.PLAYER_TIME.getTicks(), CalendarTFC.CALENDAR_TIME.getTicks()));

                    list.add(GRAY + "Biome: " + WHITE + mc.field_71441_e.func_180494_b(blockpos).func_185359_l());

                    list.add(GRAY + "Rainfall: " + WHITE + data.getRainfall());
                    list.add(GRAY + "Flora Density: " + WHITE + data.getFloraDensity());
                    list.add(GRAY + "Flora Diversity: " + WHITE + data.getFloraDiversity());

                    list.add(GRAY + "Valid Trees: ");
                    data.getValidTrees().forEach(t -> list.add(String.format("%s %s (%.1f)", WHITE, t.getRegistryName(), t.getDominance())));

                    //list.add(GRAY + "Rocks: " + WHITE + data.getRockLayer1(x, z).name + ", " + data.getRockLayer2(x, z).name + ", " + data.getRockLayer3(x, z).name);
                    //list.add(GRAY + "Stability: " + WHITE + data.getStabilityLayer(x, z).name);
                    //list.add(GRAY + "Drainage: " + WHITE + data.getDrainageLayer(x, z).name);
                    list.add(GRAY + "Sea level offset: " + WHITE + data.getSeaLevelOffset(x, z));
                    //list.add(GRAY + "Fish population: " + WHITE + data.getFishPopulation());

                    //list.add("");
                    //list.add(GRAY + "Rock at feet: " + WHITE + data.getRockLayerHeight(x, blockpos.getY(), z).name);

                    // list.add("");
                    //data.getOresSpawned().stream().map(String::valueOf).forEach(list::add);
                }
            }
        }
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public static void onItemTooltip(ItemTooltipEvent event)
    {
        ItemStack stack = event.getItemStack();
        Item item = stack.func_77973_b();
        List<String> tt = event.getToolTip();
        if (!stack.func_190926_b())
        {
            // Stuff that should always be shown as part of the tooltip
            IItemSize size = CapabilityItemSize.getIItemSize(stack);
            if (size != null)
            {
                size.addSizeInfo(stack, tt);
            }
            IItemHeat heat = stack.getCapability(CapabilityItemHeat.ITEM_HEAT_CAPABILITY, null);
            if (heat != null)
            {
                heat.addHeatInfo(stack, tt);
            }
            IFood nutrients = stack.getCapability(CapabilityFood.CAPABILITY, null);
            if (nutrients != null)
            {
                nutrients.addNutrientInfo(stack, tt);
            }

            if (event.getFlags().func_194127_a()) // Only added with advanced tooltip mode
            {
                if (item instanceof IMetalObject)
                {
                    ((IMetalObject) item).addMetalInfo(stack, tt);
                }
                else if (item instanceof ItemBlock)
                {
                    Block block = ((ItemBlock) item).func_179223_d();
                    if (block instanceof IMetalObject)
                    {
                        ((IMetalObject) block).addMetalInfo(stack, tt);
                    }
                }
                if (item instanceof IRockObject)
                {
                    ((IRockObject) item).addRockInfo(stack, tt);
                }
                else if (item instanceof ItemBlock)
                {
                    Block block = ((ItemBlock) item).func_179223_d();
                    if (block instanceof IRockObject)
                    {
                        ((IRockObject) block).addRockInfo(stack, tt);
                    }
                }

                if (ConfigTFC.CLIENT.showToolClassTooltip)
                {
                    Set<String> toolClasses = item.getToolClasses(stack);
                    if (toolClasses.size() == 1)
                    {
                        tt.add(I18n.func_135052_a("tfc.tooltip.toolclass", toolClasses.iterator().next()));
                    }
                    else if (toolClasses.size() > 1)
                    {
                        tt.add(I18n.func_135052_a("tfc.tooltip.toolclasses"));
                        for (String toolClass : toolClasses)
                        {
                            tt.add("+ " + toolClass);
                        }
                    }
                }
                if (ConfigTFC.CLIENT.showOreDictionaryTooltip)
                {
                    int[] ids = OreDictionary.getOreIDs(stack);
                    if (ids.length == 1)
                    {
                        tt.add(I18n.func_135052_a("tfc.tooltip.oredictionaryentry", OreDictionary.getOreName(ids[0])));
                    }
                    else if (ids.length > 1)
                    {
                        tt.add(I18n.func_135052_a("tfc.tooltip.oredictionaryentries"));
                        ArrayList<String> names = new ArrayList<>(ids.length);
                        for (int id : ids)
                        {
                            names.add("+ " + OreDictionary.getOreName(id));
                        }
                        names.sort(null); // Natural order (String.compare)
                        tt.addAll(names);
                    }
                }
                if (ConfigTFC.CLIENT.showNBTTooltip)
                {
                    if (stack.func_77942_o())
                    {
                        tt.add("NBT: " + stack.func_77978_p());
                    }
                }
            }
        }
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public static void textureStitched(TextureStitchEvent.Post event)
    {
        FluidSpriteCache.clear();
    }
}
