/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.gui;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.lwjgl.opengl.GL11;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.client.resources.I18n;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.Slot;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.text.TextFormatting;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.capability.IFluidHandler;
import net.minecraftforge.fluids.capability.IFluidTankProperties;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandler;

import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.recipes.barrel.BarrelRecipe;
import net.dries007.tfc.client.FluidSpriteCache;
import net.dries007.tfc.client.button.GuiButtonBarrelSeal;
import net.dries007.tfc.client.button.IButtonTooltip;
import net.dries007.tfc.network.PacketGuiButton;
import net.dries007.tfc.objects.container.ContainerBarrel;
import net.dries007.tfc.objects.te.TEBarrel;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

public class GuiBarrel extends GuiContainerTE<TEBarrel>
{
    public static final ResourceLocation BARREL_BACKGROUND = new ResourceLocation(MOD_ID, "textures/gui/barrel.png");
    private final String translationKey;

    public GuiBarrel(Container container, InventoryPlayer playerInv, TEBarrel tile, String translationKey)
    {
        super(container, playerInv, tile, BARREL_BACKGROUND);

        this.translationKey = translationKey;
    }

    @Override
    public void func_73866_w_()
    {
        super.func_73866_w_();
        func_189646_b(new GuiButtonBarrelSeal(tile, 0, field_147009_r, field_147003_i));
    }

    @Override
    protected void func_191948_b(int mouseX, int mouseY)
    {
        super.func_191948_b(mouseX, mouseY);

        int relX = mouseX - field_147003_i;
        int relY = mouseY - field_147009_r;

        if (relX >= 7 && relY >= 19 && relX < 25 && relY < 71)
        {
            IFluidHandler tank = ((ContainerBarrel) field_147002_h).getBarrelTank();

            if (tank != null)
            {
                FluidStack fluid = tank.getTankProperties()[0].getContents();
                List<String> tooltip = new ArrayList<>();

                if (fluid == null || fluid.amount == 0)
                {
                    tooltip.add(I18n.func_135052_a(MOD_ID + ".tooltip.barrel_empty"));
                }
                else
                {
                    tooltip.add(fluid.getLocalizedName());
                    tooltip.add(TextFormatting.GRAY.toString() + I18n.func_135052_a(MOD_ID + ".tooltip.barrel_fluid_amount", fluid.amount));
                }

                this.drawHoveringText(tooltip, mouseX, mouseY, field_146289_q);
            }
        }

        // Button Tooltips
        for (GuiButton button : field_146292_n)
        {
            if (button instanceof IButtonTooltip && button.func_146115_a())
            {
                IButtonTooltip tooltip = (IButtonTooltip) button;
                if (tooltip.hasTooltip())
                {
                    func_146279_a(I18n.func_135052_a(tooltip.getTooltip()), mouseX, mouseY);
                }
            }
        }
    }

    @Override
    protected void func_146979_b(int mouseX, int mouseY)
    {
        String name = I18n.func_135052_a(translationKey + ".name");
        field_146289_q.func_78276_b(name, field_146999_f / 2 - field_146289_q.func_78256_a(name) / 2, 6, 0x404040);

        if (tile.isSealed())
        {
            // Draw over the input items, making them look unavailable
            IItemHandler handler = tile.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null);
            if (handler != null)
            {
                GL11.glDisable(GL11.GL_DEPTH_TEST);
                for (int slotId = 0; slotId < handler.getSlots(); slotId++)
                {
                    drawSlotOverlay(field_147002_h.func_75139_a(slotId));
                }
                GL11.glEnable(GL11.GL_DEPTH_TEST);
            }

            // Draw the text displaying both the seal date, and the recipe name
            field_146289_q.func_78276_b(tile.getSealedDate(), field_146999_f / 2 - field_146289_q.func_78256_a(tile.getSealedDate()) / 2, 73, 0x404040);

            BarrelRecipe recipe = tile.getRecipe();
            if (recipe != null)
            {
                String resultName = recipe.getResultName();
                field_146289_q.func_78276_b(resultName, 112 - field_146289_q.func_78256_a(resultName) / 2, 19, 0x404040);
            }
        }
    }

    @Override
    protected void func_146976_a(float partialTicks, int mouseX, int mouseY)
    {
        super.func_146976_a(partialTicks, mouseX, mouseY);

        ContainerBarrel container = (ContainerBarrel) field_147002_h;
        IFluidHandler tank = container.getBarrelTank();

        if (tank != null)
        {
            IFluidTankProperties t = tank.getTankProperties()[0];
            FluidStack fs = t.getContents();

            if (fs != null)
            {
                int fillHeightPixels = (int) (50 * fs.amount / (float) t.getCapacity());

                if (fillHeightPixels > 0)
                {
                    Fluid fluid = fs.getFluid();
                    TextureAtlasSprite sprite = FluidSpriteCache.getSprite(fluid);

                    int positionX = field_147003_i + 8;
                    int positionY = field_147009_r + 54;

                    Minecraft.func_71410_x().field_71446_o.func_110577_a(TextureMap.field_110575_b);
                    BufferBuilder buffer = Tessellator.func_178181_a().func_178180_c();

                    GlStateManager.func_179141_d();
                    GlStateManager.func_179147_l();
                    GlStateManager.func_187428_a(GlStateManager.SourceFactor.SRC_ALPHA, GlStateManager.DestFactor.ONE_MINUS_SRC_ALPHA, GlStateManager.SourceFactor.ONE, GlStateManager.DestFactor.ZERO);

                    int color = fluid.getColor();

                    float r = ((color >> 16) & 0xFF) / 255f;
                    float g = ((color >> 8) & 0xFF) / 255f;
                    float b = (color & 0xFF) / 255f;
                    float a = ((color >> 24) & 0xFF) / 255f;

                    GlStateManager.func_179131_c(r, g, b, a);

                    buffer.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_181707_g);

                    while (fillHeightPixels > 15)
                    {
                        buffer.func_181662_b(positionX, positionY, 0).func_187315_a(sprite.func_94209_e(), sprite.func_94206_g()).func_181675_d();
                        buffer.func_181662_b(positionX, positionY + 16, 0).func_187315_a(sprite.func_94209_e(), sprite.func_94210_h()).func_181675_d();
                        buffer.func_181662_b(positionX + 16, positionY + 16, 0).func_187315_a(sprite.func_94212_f(), sprite.func_94210_h()).func_181675_d();
                        buffer.func_181662_b(positionX + 16, positionY, 0).func_187315_a(sprite.func_94212_f(), sprite.func_94206_g()).func_181675_d();

                        fillHeightPixels -= 16;
                        positionY -= 16;
                    }

                    if (fillHeightPixels > 0)
                    {
                        int blank = 16 - fillHeightPixels;
                        positionY += blank;
                        buffer.func_181662_b(positionX, positionY, 0).func_187315_a(sprite.func_94209_e(), sprite.func_94207_b(blank)).func_181675_d();
                        buffer.func_181662_b(positionX, positionY + fillHeightPixels, 0).func_187315_a(sprite.func_94209_e(), sprite.func_94210_h()).func_181675_d();
                        buffer.func_181662_b(positionX + 16, positionY + fillHeightPixels, 0).func_187315_a(sprite.func_94212_f(), sprite.func_94210_h()).func_181675_d();
                        buffer.func_181662_b(positionX + 16, positionY, 0).func_187315_a(sprite.func_94212_f(), sprite.func_94207_b(blank)).func_181675_d();
                    }

                    Tessellator.func_178181_a().func_78381_a();

                    Minecraft.func_71410_x().field_71446_o.func_110577_a(BARREL_BACKGROUND);
                    GlStateManager.func_179131_c(1, 1, 1, 1);
                }
            }
        }

        func_73729_b(field_147003_i + 7, field_147009_r + 19, 176, 0, 18, 52);
    }

    @Override
    protected void func_146284_a(GuiButton button) throws IOException
    {
        TerraFirmaCraft.getNetwork().sendToServer(new PacketGuiButton(button.field_146127_k));
        super.func_146284_a(button);
    }

    private void drawSlotOverlay(Slot slot)
    {
        int xPos = slot.field_75223_e - 1;
        int yPos = slot.field_75221_f - 1;

        this.func_73733_a(xPos, yPos, xPos + 18, yPos + 18, 0x75FFFFFF, 0x75FFFFFF);
    }
}
