/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.gui.overlay;

import java.awt.*;

import org.lwjgl.opengl.GL11;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.gui.ScaledResolution;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.FoodStats;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.client.GuiIngameForge;
import net.minecraftforge.client.event.RenderGameOverlayEvent;
import net.minecraftforge.client.event.RenderGameOverlayEvent.ElementType;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.api.capability.food.IFoodStatsTFC;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

@SideOnly(Side.CLIENT)
public final class PlayerDataOverlay
{
    private static final ResourceLocation ICONS = new ResourceLocation(MOD_ID, "textures/gui/overlay/icons.png");
    private static final PlayerDataOverlay INSTANCE = new PlayerDataOverlay();

    public static PlayerDataOverlay getInstance() { return INSTANCE; }

    @SubscribeEvent
    public void render(RenderGameOverlayEvent.Pre event)
    {
        Minecraft mc = Minecraft.func_71410_x();
        EntityPlayer player = mc.field_71439_g.field_71071_by.field_70458_d;
        GuiIngameForge.renderFood = false;

        // We check for crosshairs just because it's always drawn and is before air bar
        if (event.getType() != ElementType.CROSSHAIRS)
        {
            return;
        }

        FoodStats foodStats = player.func_71024_bL();
        float baseMaxHealth = 1000;
        float currentThirst = 100;
        if (foodStats instanceof IFoodStatsTFC)
        {
            IFoodStatsTFC foodStatsTFC = (IFoodStatsTFC) foodStats;
            baseMaxHealth = 20 * foodStatsTFC.getHealthModifier() * 50; //20 = 1000 HP in overlay
            currentThirst = foodStatsTFC.getThirst();
        }
        // This is for air to be drawn above our bars
        GuiIngameForge.right_height += 10;

        ScaledResolution sr = event.getResolution();

        FontRenderer fontrenderer = mc.field_71466_p;

        int healthRowHeight = sr.func_78328_b() - 40;
        int armorRowHeight = healthRowHeight - 10;
        int mid = sr.func_78326_a() / 2;

        GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
        mc.field_71446_o.func_110577_a(ICONS);

        if (mc.field_71442_b.func_78763_f())
        {
            //Draw Health
            GL11.glEnable(GL11.GL_BLEND);
            this.drawTexturedModalRect(mid - 91, healthRowHeight, 0, 0, 90, 10);
            float curHealth = player.func_110143_aJ() * baseMaxHealth / (float) 20;
            float percentHealth = curHealth / baseMaxHealth;
            float surplusPercent = Math.max(percentHealth - 1, 0);
            int uSurplus = 90;
            if (percentHealth > 1) percentHealth = 1;

            this.drawTexturedModalRect(mid - 91, healthRowHeight, 0, 10, (int) (90 * percentHealth), 10);
            while (surplusPercent > 0)
            {
                //Draw beyond max health bar(if other mods adds more health)
                float percent = Math.min(surplusPercent, 1);
                this.drawTexturedModalRect(mid - 91, healthRowHeight, uSurplus, 10, (int) (90 * percent), 10);
                surplusPercent -= 1.0f;
                uSurplus = uSurplus == 90 ? 0 : 90; //To alternate between red and yellow bars (if mods adds that much surplus health)
                //To anyone seeing this: feel free to make a colorize(Hue tweaking?) function to get other color bars
                //Or just add more color bars to overlay icons.
            }
            //Draw Food and Water
            float foodLevel = player.func_71024_bL().func_75116_a();
            float percentFood = foodLevel / 20f;
            float percentThirst = currentThirst / 100f;


            GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
            this.drawTexturedModalRect(mid + 1, healthRowHeight, 0, 20, 90, 5);
            GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);

            this.drawTexturedModalRect(mid + 1, healthRowHeight, 0, 25, (int) (90 * percentFood), 5);

            this.drawTexturedModalRect(mid + 1, healthRowHeight + 5, 90, 20, 90, 5);
            this.drawTexturedModalRect(mid + 1, healthRowHeight + 5, 90, 25, (int) (90 * percentThirst), 5);

            //Draw Notifications
            String healthString = ((int) curHealth) + "/" + ((int) (baseMaxHealth));
            fontrenderer.func_78276_b(healthString, mid - 45 - (fontrenderer.func_78256_a(healthString) / 2), healthRowHeight + 2, Color.white.getRGB());

            GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
            mc.field_71446_o.func_110577_a(new ResourceLocation("minecraft:textures/gui/icons.png"));

            //Draw experience bar when not riding anything, riding a non-living entity such as a boat/minecart, or riding a pig.
            if (!(player.func_184187_bx() instanceof EntityLiving))
            {
                int cap = player.func_71050_bK();
                int left = mid - 91;

                if (cap > 0)
                {
                    short barWidth = 182;
                    int filled = (int) (player.field_71106_cc * (barWidth + 1));
                    int top = sr.func_78328_b() - 29;
                    drawTexturedModalRect(left, top, 0, 64, barWidth, 5);
                    if (filled > 0)
                        drawTexturedModalRect(left, top, 0, 69, filled, 5);
                }

                if (player.field_71068_ca > 0)
                {
                    int color = 8453920;
                    String text = Integer.toString(player.field_71068_ca);
                    int x = (sr.func_78326_a() - fontrenderer.func_78256_a(text)) / 2;
                    int y = sr.func_78328_b() - 30;
                    fontrenderer.func_78276_b(text, x + 1, y, 0);
                    fontrenderer.func_78276_b(text, x - 1, y, 0);
                    fontrenderer.func_78276_b(text, x, y + 1, 0);
                    fontrenderer.func_78276_b(text, x, y - 1, 0);
                    fontrenderer.func_78276_b(text, x, y, color);
                }

                // We have to reset the color back to white
                GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
            }

            // Draw mount's health bar
            if (player.func_184187_bx() instanceof EntityLivingBase)
            {
                GuiIngameForge.renderHealthMount = false;
                mc.field_71446_o.func_110577_a(ICONS);
                EntityLivingBase mount = ((EntityLivingBase) player.func_184187_bx());
                this.drawTexturedModalRect(mid + 1, armorRowHeight, 90, 0, 90, 10);
                double mountMaxHealth = mount.func_110148_a(SharedMonsterAttributes.field_111267_a).func_111125_b();
                double mountCurrentHealth = mount.func_110143_aJ();
                float mountPercentHealth = (float) Math.min(mountCurrentHealth / mountMaxHealth, 1.0f);
                this.drawTexturedModalRect(mid + 1, armorRowHeight, 90, 10, (int) (90 * mountPercentHealth), 10);

                String mountHealthString = (int) Math.min(mountCurrentHealth, mountMaxHealth) + "/" + (int) mountMaxHealth;
                fontrenderer.func_78276_b(mountHealthString, mid + 47 - (fontrenderer.func_78256_a(mountHealthString) / 2), armorRowHeight + 2, Color.white.getRGB());
            }

            mc.field_71446_o.func_110577_a(new ResourceLocation("minecraft:textures/gui/icons.png"));
        }
    }

    @SuppressWarnings("PointlessArithmeticExpression")
    public void drawTexturedModalRect(float xCoord, float yCoord, int minU, int minV, int maxU, int maxV)
    {
        float textureScaleU = 0.00390625F;
        float textureScaleV = 0.00390625F;
        Tessellator tessellator = Tessellator.func_178181_a();
        BufferBuilder vb = tessellator.func_178180_c();
        vb.func_181668_a(7, DefaultVertexFormats.field_181707_g);
        vb.func_181662_b(xCoord + 0.0F, yCoord + maxV, 0).func_187315_a((minU + 0) * textureScaleU, (minV + maxV) * textureScaleV).func_181675_d();
        vb.func_181662_b(xCoord + maxU, yCoord + maxV, 0).func_187315_a((minU + maxU) * textureScaleU, (minV + maxV) * textureScaleV).func_181675_d();
        vb.func_181662_b(xCoord + maxU, yCoord + 0.0F, 0).func_187315_a((minU + maxU) * textureScaleU, (minV + 0) * textureScaleV).func_181675_d();
        vb.func_181662_b(xCoord + 0.0F, yCoord + 0.0F, 0).func_187315_a((minU + 0) * textureScaleU, (minV + 0) * textureScaleV).func_181675_d();
        tessellator.func_78381_a();
    }
}
