/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.model.animal;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.client.model.ModelBase;
import net.minecraft.client.model.ModelRenderer;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.entity.Entity;
import net.minecraft.util.math.MathHelper;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC;
import net.dries007.tfc.objects.entity.animal.EntityChickenTFC;

@SideOnly(Side.CLIENT)
@ParametersAreNonnullByDefault
public class ModelChickenTFC extends ModelBase
{
    private ModelRenderer head;
    private ModelRenderer body;
    private ModelRenderer rightLeg;
    private ModelRenderer leftLeg;
    private ModelRenderer rightWing;
    private ModelRenderer leftWing;
    private ModelRenderer bill;
    private ModelRenderer chin;
    private ModelRenderer[] tails;
    private ModelRenderer crown;

    public ModelChickenTFC()
    {
        byte var1 = 16;
        this.head = new ModelRenderer(this, 0, 0);
        this.head.func_78790_a(-2.0F, -6.0F, -2.0F, 4, 6, 3, 0.0F);
        this.head.func_78793_a(0.0F, -1 + var1, -4.0F);
        this.bill = new ModelRenderer(this, 14, 0);
        this.bill.func_78790_a(-2.0F, -4.0F, -4.0F, 4, 2, 2, 0.0F);
        this.bill.func_78793_a(0.0F, -1 + var1, -4.0F);
        this.chin = new ModelRenderer(this, 14, 4);
        this.chin.func_78790_a(-1.0F, -2.0F, -3.0F, 2, 2, 2, 0.0F);
        this.chin.func_78793_a(0.0F, -1 + var1, -4.0F);
        this.body = new ModelRenderer(this, 0, 9);
        this.body.func_78790_a(-3.0F, -4.0F, -3.0F, 6, 8, 6, 0.0F);
        this.body.func_78793_a(0.0F, var1, 0.0F);
        this.rightLeg = new ModelRenderer(this, 26, 0);
        this.rightLeg.func_78789_a(-1.0F, 0.0F, -3.0F, 3, 5, 3);
        this.rightLeg.func_78793_a(-2.0F, 3 + var1, 1.0F);
        this.leftLeg = new ModelRenderer(this, 26, 0);
        this.leftLeg.func_78789_a(-1.0F, 0.0F, -3.0F, 3, 5, 3);
        this.leftLeg.func_78793_a(1.0F, 3 + var1, 1.0F);
        this.rightWing = new ModelRenderer(this, 24, 13);
        this.rightWing.func_78789_a(0.0F, 0.0F, -3.0F, 1, 4, 6);
        this.rightWing.func_78793_a(-4.0F, -3 + var1, 0.0F);
        this.leftWing = new ModelRenderer(this, 24, 13);
        this.leftWing.func_78789_a(-1.0F, 0.0F, -3.0F, 1, 4, 6);
        this.leftWing.func_78793_a(4.0F, -3 + var1, 0.0F);

        crown = new ModelRenderer(this, 0, 23);
        crown.func_78789_a(0, -7, -3, 0, 4, 5);
        crown.func_78793_a(0.0F, -3 + var1, -1.0F);


        tails = new ModelRenderer[32];
        for (int i = 0; i < 32; i++)
        {
            tails[i] = new ModelRenderer(this, 45, 0);
            tails[i].func_78790_a(0, 16, 0, 3, 21, 0, 0);
            tails[i].func_78793_a(0, 32, 2);
        }
    }

    @Override
    public void func_78088_a(@Nonnull Entity entity, float par2, float par3, float par4, float par5, float par6, float par7)
    {
        this.func_78087_a(par2, par3, par4, par5, par6, par7, entity);

        EntityChickenTFC chicken = ((EntityChickenTFC) entity);

        float percent = chicken.getPercentToAdulthood();
        float ageScale = 2.0F - percent;

        GlStateManager.func_179094_E();

        GlStateManager.func_179109_b(0.0F, 0.75f - (0.75f * percent), 0f);
        GlStateManager.func_179152_a(1 / ageScale, 1 / ageScale, 1 / ageScale);

        if (percent >= 0.75 && chicken.getGender() == EntityAnimalTFC.Gender.MALE)
        {
            crown.field_78807_k = false;
            this.body.field_78795_f = (float) Math.PI / 4F;
            this.rightWing.field_78795_f = (float) Math.PI / 4F;
            rightWing.func_78793_a(-4.0F, 13.5f, -2.5F);
            this.leftWing.field_78795_f = (float) Math.PI / 4F;
            leftWing.func_78793_a(4.0F, 13.5f, -2.5F);
            this.head.func_78793_a(0.0F, 13, -1.0F);
            this.bill.func_78793_a(0.0F, 13, -1.0F);
            this.chin.func_78793_a(0.0F, 13, -1.0F);
        }
        this.head.func_78785_a(par7);
        this.bill.func_78785_a(par7);
        this.chin.func_78785_a(par7);

        GlStateManager.func_179094_E();
        GlStateManager.func_179152_a(0.75f, 0.75f, 0.75f);
        this.crown.func_78785_a(par7);
        GlStateManager.func_179121_F();
        GlStateManager.func_179121_F();

        GlStateManager.func_179094_E();
        GlStateManager.func_179109_b(0.0F, 0.75f - (0.75f * percent), 0f);
        GlStateManager.func_179152_a(1 / ageScale, 1 / ageScale, 1 / ageScale);
        this.body.func_78785_a(par7);
        this.rightLeg.func_78785_a(par7);
        this.leftLeg.func_78785_a(par7);
        this.rightWing.func_78785_a(par7);
        this.leftWing.func_78785_a(par7);
        GlStateManager.func_179121_F();
        GlStateManager.func_179094_E();
        GlStateManager.func_179152_a(0.25F / percent, 0.5F / percent, 0.25F / percent);
        for (int i = 0; i < 32; i++)
        {
            tails[i].func_78785_a(par7);
        }
        GlStateManager.func_179121_F();
    }

    @Override
    public void func_78087_a(float par1, float par2, float par3, float par4, float par5, float par6, Entity ent)
    {
        this.head.field_78795_f = -(par5 / (180F / (float) Math.PI));
        this.head.field_78796_g = par4 / (180F / (float) Math.PI);
        this.crown.field_78795_f = -(par5 / (180F / (float) Math.PI));
        this.crown.field_78796_g = par4 / (180F / (float) Math.PI);
        this.bill.field_78795_f = this.head.field_78795_f;
        this.bill.field_78796_g = this.head.field_78796_g;
        this.chin.field_78795_f = this.head.field_78795_f;
        this.chin.field_78796_g = this.head.field_78796_g;
        this.body.field_78795_f = (float) Math.PI / 2F;
        crown.field_78807_k = true;

        for (int i = 0; i < 32; i++)
        {
            tails[i].field_78795_f = (float) Math.PI * 3 / 5F + (float) ((i % 3) * Math.PI / 32) * (i % 2 != 0 ? 1 : -1);
            tails[i].field_78808_h = (float) Math.PI / -2 + (float) Math.PI * i / 31;
        }
        this.rightLeg.field_78795_f = MathHelper.func_76134_b(par1 * 0.6662F) * 1.4F * par2;
        this.leftLeg.field_78795_f = MathHelper.func_76134_b(par1 * 0.6662F + (float) Math.PI) * 1.4F * par2;
        this.rightWing.field_78808_h = par3;
        this.leftWing.field_78808_h = -par3;
        this.rightWing.field_78795_f = 0;
        this.leftWing.field_78795_f = 0;
        this.rightWing.func_78793_a(-4.0F, 13, 0.0F);
        this.leftWing.func_78793_a(4.0F, 13, 0.0F);
    }
}
