/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.model.animal;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.client.model.ModelBase;
import net.minecraft.client.model.ModelRenderer;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.entity.Entity;
import net.minecraft.util.math.MathHelper;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC;
import net.dries007.tfc.objects.entity.animal.EntityDeerTFC;

@SideOnly(Side.CLIENT)
@ParametersAreNonnullByDefault
public class ModelDeerTFC extends ModelBase
{
    //fields
    private ModelRenderer antler24;
    private ModelRenderer antler23;
    private ModelRenderer antler22;
    private ModelRenderer antler21;
    private ModelRenderer antler14;
    private ModelRenderer antler13;
    private ModelRenderer antler12;
    private ModelRenderer antler11;
    private ModelRenderer hoof2;
    private ModelRenderer toes3;
    private ModelRenderer thigh1;
    private ModelRenderer ear2;
    private ModelRenderer ear1;
    private ModelRenderer calf2;
    private ModelRenderer tail;
    private ModelRenderer collar;
    private ModelRenderer upperLeg4;
    private ModelRenderer neck;
    private ModelRenderer rump;
    private ModelRenderer head;
    private ModelRenderer body;
    private ModelRenderer leg1;
    private ModelRenderer leg2;
    private ModelRenderer leg3;
    private ModelRenderer leg4;
    private ModelRenderer snout;
    private ModelRenderer torso;
    private ModelRenderer upperLeg3;
    private ModelRenderer calf1;
    private ModelRenderer lowerleg3;
    private ModelRenderer lowerleg4;
    private ModelRenderer thigh2;
    private ModelRenderer toes4;
    private ModelRenderer toes2;
    private ModelRenderer toes1;
    private ModelRenderer hoof1;
    private ModelRenderer hoof3;
    private ModelRenderer hoof4;
    private boolean running;

    public ModelDeerTFC()
    {
        field_78090_t = 128;
        field_78089_u = 64;

        antler24 = new ModelRenderer(this, 44, 0);
        antler24.func_78789_a(-6.8F, -15.4F, -1.8F, 1, 2, 1);
        antler24.func_78793_a(0F, 0F, 0F);
        antler24.func_78787_b(128, 64);
        antler24.field_78809_i = true;
        setRotation(antler24, 0F, 0F, 0.2792527F);
        antler23 = new ModelRenderer(this, 44, 0);
        antler23.func_78789_a(-2.8F, -11F, 8F, 1, 2, 1);
        //antler23.addBox(0F,0F,0F,1,2,1);
        antler23.func_78793_a(0F, 0F, 0F);
        antler23.func_78787_b(128, 64);
        antler23.field_78809_i = true;
        setRotation(antler23, 0.8726646F, -0.4363323F, 0F);
        antler22 = new ModelRenderer(this, 44, 0);
        antler22.func_78789_a(2.3F, -14.3F, -5.1F, 1, 3, 1);
        antler22.func_78793_a(0F, 0F, 0F);
        antler22.func_78787_b(128, 64);
        antler22.field_78809_i = true;
        setRotation(antler22, -0.2268928F, 0F, -0.3490659F);
        antler21 = new ModelRenderer(this, 44, 0);
        antler21.func_78789_a(-2F, -13F, -2.5F, 1, 3, 1);
        antler21.func_78793_a(0F, -1F, -8F);
        antler21.func_78787_b(128, 64);
        antler21.field_78809_i = true;
        setRotation(antler21, 0F, 0F, 0F);
        antler14 = new ModelRenderer(this, 44, 0);
        antler14.func_78789_a(5.8F, -15.4F, -1.8F, 1, 2, 1);
        antler14.func_78793_a(0F, 0F, 0F);
        antler14.func_78787_b(128, 64);
        antler14.field_78809_i = true;
        setRotation(antler14, 0F, 0F, -0.2792527F);
        antler13 = new ModelRenderer(this, 44, 0);
        antler13.func_78789_a(1.8F, -11F, 8F, 1, 2, 1);
        antler13.func_78793_a(0F, 0F, 0F);
        antler13.func_78787_b(128, 64);
        antler13.field_78809_i = true;
        setRotation(antler13, 0.8726646F, 0.4363323F, 0F);
        antler12 = new ModelRenderer(this, 44, 0);
        antler12.func_78789_a(-3.3F, -14.3F, -5.1F, 1, 3, 1);
        antler12.func_78793_a(0F, 0F, 0F);
        antler12.func_78787_b(128, 64);
        antler12.field_78809_i = true;
        setRotation(antler12, -0.2268928F, 0F, 0.3490659F);
        antler11 = new ModelRenderer(this, 44, 0);
        antler11.func_78789_a(1F, -13F, -2.5F, 1, 3, 1);
        antler11.func_78793_a(0F, -1F, -8F);
        antler11.func_78787_b(128, 64);
        antler11.field_78809_i = true;
        setRotation(antler11, 0F, 0F, 0F);
        toes3 = new ModelRenderer(this, 18, 22);
        toes3.func_78789_a(-0.5F, 0.4F, -3F, 2, 1, 4);
        toes3.func_78793_a(0F, 5F, 0F);
        toes3.func_78787_b(128, 64);
        setRotation(toes3, 1.134464F, 0F, 0F);
        thigh1 = new ModelRenderer(this, 40, 22);
        thigh1.func_78789_a(-1F, -2.3F, -2F, 2, 9, 5);
        thigh1.func_78793_a(-3F, 2F, 7F);
        thigh1.func_78787_b(128, 64);
        setRotation(thigh1, -0.1745329F, 0F, 0.1745329F);
        ear2 = new ModelRenderer(this, 54, 16);
        ear2.field_78809_i = true;
        ear2.func_78789_a(-9F, -10F, -2F, 5, 2, 1);
        ear2.func_78793_a(0F, 0F, 0F);
        ear2.func_78787_b(128, 64);
        setRotation(ear2, 0F, 0.3490659F, 0.34906585F);
        ear1 = new ModelRenderer(this, 54, 16);
        ear1.func_78789_a(4F, -10F, -2F, 5, 2, 1);
        ear1.func_78793_a(0F, 0F, 0F);
        ear1.func_78787_b(128, 64);
        setRotation(ear1, 0F, -0.3490659F, -0.34906585F);
        calf2 = new ModelRenderer(this, 54, 7);
        calf2.field_78809_i = true;
        calf2.func_78789_a(-1F, -1F, 0F, 2, 6, 3);
        calf2.func_78793_a(0F, 6F, 0F);
        calf2.func_78787_b(128, 64);
        setRotation(calf2, 0.5585054F, 0F, 0F);
        tail = new ModelRenderer(this, 24, 52);
        tail.func_78789_a(-1.5F, -0.5F, 0F, 3, 2, 9);
        tail.func_78793_a(0F, -1.5F, 10F);
        tail.func_78787_b(128, 64);
        tail.field_78809_i = true;
        setRotation(tail, -1.308997F, 0F, 0F);
        collar = new ModelRenderer(this, 30, 38);
        collar.func_78789_a(0F, -2F, -4F, 5, 6, 7);
        collar.func_78793_a(-2.5F, -1F, -8F);
        collar.func_78787_b(128, 64);
        collar.field_78809_i = true;
        setRotation(collar, 1.151917F, 0F, 0F);
        upperLeg4 = new ModelRenderer(this, 30, 22);
        upperLeg4.field_78809_i = true;
        upperLeg4.func_78789_a(-1.25F, -1F, -1.5F, 2, 5, 3);
        upperLeg4.func_78793_a(4F, 5F, -7F);
        upperLeg4.func_78787_b(128, 64);

        setRotation(upperLeg4, 0.3490659F, 0F, 0.0349066F);
        neck = new ModelRenderer(this, 57, 22);
        neck.func_78789_a(-2F, -4F, -2F, 4, 5, 8);
        neck.func_78793_a(0F, -1F, -8F);
        neck.func_78787_b(128, 64);
        neck.field_78809_i = true;
        setRotation(neck, 1.815142F, 0F, 0F);
        rump = new ModelRenderer(this, 0, 47);
        rump.func_78789_a(-3F, -4F, 3F, 6, 10, 6);
        rump.func_78793_a(0F, 1.5F, 1F);
        rump.func_78787_b(128, 64);
        rump.field_78809_i = true;
        setRotation(rump, -0.0872665F, 0F, 0F);
        head = new ModelRenderer(this, 54, 35);
        head.func_78789_a(-2.5F, -11F, -5F, 5, 6, 6);
        head.func_78793_a(0F, -1F, -8F);
        head.func_78787_b(128, 64);
        head.field_78809_i = true;
        setRotation(head, 0.1570796F, 0F, 0F);
        head.func_78792_a(ear1);
        head.func_78792_a(ear2);
        body = new ModelRenderer(this, 18, 4);
        body.func_78789_a(-4F, -11F, -8F, 8, 8, 10);
        body.func_78793_a(0F, 1F, 2F);
        body.func_78787_b(128, 64);
        body.field_78809_i = true;
        setRotation(body, 1.43117F, 0F, 0F);
        leg1 = new ModelRenderer(this, 0, 16);
        leg1.func_78789_a(-0F, -1F, 0F, 2, 9, 2);
        leg1.func_78793_a(0F, 5F, 0F);
        leg1.func_78787_b(128, 64);
        setRotation(leg1, 0F, 0F, 0F);
        leg2 = new ModelRenderer(this, 0, 16);
        leg2.field_78809_i = true;
        leg2.func_78789_a(-1F, -1F, 0F, 2, 9, 2);
        leg2.func_78793_a(0F, 5F, 0F);
        leg2.func_78787_b(128, 64);

        setRotation(leg2, 0F, 0F, 0F);
        leg3 = new ModelRenderer(this, 8, 16);
        leg3.func_78789_a(-0.5F, 0F, 0F,/*-0.5F, 3F, 0F,*/ 2, 7, 2);
        leg3.func_78793_a(0F, 3F, -1F);
        leg3.func_78787_b(128, 64);
        setRotation(leg3, 0F, 0F, 0F);
        leg4 = new ModelRenderer(this, 8, 16);
        leg4.field_78809_i = true;
        leg4.func_78789_a(-1.5F, 0F, 0F, 2, 7, 2);
        leg4.func_78793_a(0F, 3F, -1F);
        leg4.func_78787_b(128, 64);

        setRotation(leg4, -0.3490659F, 0F, -0.0349066F);
        snout = new ModelRenderer(this, 54, 0);
        snout.func_78789_a(-1.5F, -9.3F, -9F, 3, 3, 4);
        snout.func_78793_a(0F, 0F, 0F);
        snout.func_78787_b(128, 64);
        snout.field_78809_i = true;
        head.func_78792_a(snout);
        torso = new ModelRenderer(this, 0, 29);
        torso.func_78789_a(-3.5F, -3F, -5F, 7, 10, 8);
        torso.func_78793_a(0F, 1F, 2F);
        torso.func_78787_b(128, 64);
        torso.field_78809_i = true;
        setRotation(torso, 0.122173F, 0F, 0F);
        upperLeg3 = new ModelRenderer(this, 30, 22);
        upperLeg3.func_78789_a(-0.7F, -1F, -1.5F, 2, 5, 3);
        upperLeg3.func_78793_a(-4F, 5F, -7F);
        upperLeg3.func_78787_b(128, 64);
        setRotation(upperLeg3, 0.3490659F, 0F, -0.0349066F);
        calf1 = new ModelRenderer(this, 54, 7);
        calf1.func_78789_a(0F, -1F, 0F, 2, 6, 3);
        calf1.func_78793_a(0F, 7F, 0F);
        calf1.func_78787_b(128, 64);
        setRotation(calf1, 0.5585054F, 0F, 0F);
        lowerleg3 = new ModelRenderer(this, 30, 30);
        lowerleg3.func_78789_a(-0.5F, 0F, 0F, 2, 6, 2);
        lowerleg3.func_78793_a(0F, 7F, 0F);
        lowerleg3.func_78787_b(128, 64);
        setRotation(lowerleg3, 0F, 0F, 0F);
        lowerleg4 = new ModelRenderer(this, 30, 30);
        lowerleg4.field_78809_i = true;
        lowerleg4.func_78789_a(-1.5F, 0F, 0F, 2, 6, 2);
        lowerleg4.func_78793_a(0F, 7F, 0F);
        lowerleg4.func_78787_b(128, 64);

        setRotation(lowerleg4, 0F, 0F, 0F);
        thigh2 = new ModelRenderer(this, 40, 22);
        thigh2.field_78809_i = true;
        thigh2.func_78789_a(-1F, -2.3F, -2F, 2, 9, 5);
        thigh2.func_78793_a(3F, 2F, 7F);
        thigh2.func_78787_b(128, 64);

        setRotation(thigh2, -0.1745329F, 0F, -0.1745329F);
        toes4 = new ModelRenderer(this, 18, 22);
        toes4.field_78809_i = true;
        toes4.func_78789_a(-1.5F, 0.4F, -3F, 2, 1, 4);
        toes4.func_78793_a(0F, 5F, 0F);
        toes4.func_78787_b(128, 64);

        setRotation(toes4, 1.134464F, 0F, 0F);
        toes2 = new ModelRenderer(this, 18, 22);
        toes2.field_78809_i = true;
        toes2.func_78789_a(-1.0F, 0.4F, -3F, 2, 1, 4);
        toes2.func_78793_a(0F, 8F, 0F);
        toes2.func_78787_b(128, 64);

        setRotation(toes2, 1.134464F, 0F, 0F);
        toes1 = new ModelRenderer(this, 18, 22);
        toes1.func_78789_a(-0F, 0.4F, -3F, 2, 1, 4);
        toes1.func_78793_a(0F, 8F, 0F);
        toes1.func_78787_b(128, 64);
        setRotation(toes1, 1.134464F, 0F, 0F);
        hoof1 = new ModelRenderer(this, 30, 0);
        hoof1.func_78789_a(-0F, 0F, -6.3F, 2, 1, 2);
        hoof1.func_78793_a(0F, 5F, 0.5F);
        hoof1.func_78787_b(128, 64);
        setRotation(hoof1, 0F, 0F, 0F);
        hoof2 = new ModelRenderer(this, 30, 0);
        hoof2.field_78809_i = true;
        hoof2.func_78789_a(-0.5F, 0F, -6.3F, 2, 1, 2);
        hoof2.func_78793_a(0F, 5F, -0.5F);
        hoof2.func_78787_b(128, 64);

        setRotation(hoof2, 0F, 0F, 0F);
        hoof3 = new ModelRenderer(this, 30, 0);
        hoof3.func_78789_a(-0.5F, 0F, -6.3F, 2, 1, 2);
        hoof3.func_78793_a(0F, 5F, -0.5F);
        hoof3.func_78787_b(128, 64);
        setRotation(hoof3, 0F, 0F, 0F);
        hoof4 = new ModelRenderer(this, 30, 0);
        hoof4.field_78809_i = true;
        hoof4.func_78789_a(-1.5F, 0F, -6.3F, 2, 1, 2);
        hoof4.func_78793_a(0F, 5F, -0.5F);
        hoof4.func_78787_b(128, 64);

        setRotation(hoof4, 0F, 0F, 0F);

        upperLeg4.func_78792_a(leg4);
        leg4.func_78792_a(lowerleg4);
        lowerleg4.func_78792_a(toes4);
        toes4.func_78792_a(hoof4);

        upperLeg3.func_78792_a(leg3);
        leg3.func_78792_a(lowerleg3);
        lowerleg3.func_78792_a(toes3);
        toes3.func_78792_a(hoof3);

        thigh1.func_78792_a(calf1);
        calf1.func_78792_a(leg1);
        leg1.func_78792_a(toes1);
        toes1.func_78792_a(hoof1);

        thigh2.func_78792_a(calf2);
        calf2.func_78792_a(leg2);
        leg2.func_78792_a(toes2);
        toes2.func_78792_a(hoof2);

        antler11.func_78792_a(antler12);
        antler21.func_78792_a(antler22);
        antler11.func_78792_a(antler13);
        antler21.func_78792_a(antler23);
        antler11.func_78792_a(antler14);
        antler21.func_78792_a(antler24);
    }

    @Override
    public void func_78088_a(@Nonnull Entity entity, float f, float f1, float f2, float f3, float f4, float f5)
    {
        super.func_78088_a(entity, f, f1, f2, f3, f4, f5);
        this.func_78087_a(f, f1, f2, f3, f4, f5, entity);

        EntityDeerTFC deer = ((EntityDeerTFC) entity);

        running = false;
        //running = ((EntityDeer)entity).getRunning();
        float age = 1f - deer.getPercentToAdulthood();

        float aa = 2F - (1.0F - age);
        GlStateManager.func_179109_b(0.0F, -6F * f5 * age / (float) Math.pow(aa, 0.4), 0);
        GlStateManager.func_179094_E();
        float ab = (float) Math.sqrt(1.0F / aa);
        GlStateManager.func_179152_a(ab, ab, ab);
        GlStateManager.func_179109_b(0.0F, 22F * f5 * age / (float) Math.pow(aa, 0.4), 2F * f5 * age / ab);
        if (deer.getGender() == EntityAnimalTFC.Gender.MALE)
        {
            if (aa <= 1.75)
            {
                antler11.field_78807_k = false;
                antler21.field_78807_k = false;
                if (aa <= 1.5)
                {
                    antler12.field_78807_k = false;
                    antler22.field_78807_k = false;
                    if (aa <= 1.3)
                    {
                        antler13.field_78807_k = false;
                        antler23.field_78807_k = false;
                        if (aa <= 1.1)
                        {
                            antler14.field_78807_k = false;
                            antler24.field_78807_k = false;
                        }
                    }
                }
            }
        }
        antler11.func_78785_a(f5);
        antler21.func_78785_a(f5);
        head.func_78785_a(f5);
        GlStateManager.func_179121_F();
        GlStateManager.func_179094_E();
        GlStateManager.func_179152_a(1.0F / aa, ab, 1.0F / aa);
        GlStateManager.func_179109_b(0.0F, 22F * f5 * age / (float) Math.pow(aa, 0.4), 0.0F);
        thigh1.func_78785_a(f5);
        upperLeg4.func_78785_a(f5);
        upperLeg3.func_78785_a(f5);
        thigh2.func_78785_a(f5);
        GlStateManager.func_179121_F();
        GlStateManager.func_179094_E();
        GlStateManager.func_179152_a(1.0F / aa, 1.0F / aa, 1.0F / aa);
        GlStateManager.func_179109_b(0.0F, 22F * f5 * age, 0.0F);
        tail.func_78785_a(f5);
        collar.func_78785_a(f5);
        neck.func_78785_a(f5);
        rump.func_78785_a(f5);
        body.func_78785_a(f5);
        torso.func_78785_a(f5);
        GlStateManager.func_179121_F();
    }

    @Override
    public void func_78087_a(float f, float f1, float f2, float f3, float f4, float f5, Entity entity)
    {
        super.func_78087_a(f, f1, f2, f3, f4, f5, entity);

        f1 = Math.min(f1 * 7.5f, 0.75f);
        f *= 0.95f;

        antler11.field_78807_k = true;
        antler12.field_78807_k = true;
        antler13.field_78807_k = true;
        antler14.field_78807_k = true;
        antler21.field_78807_k = true;
        antler22.field_78807_k = true;
        antler23.field_78807_k = true;
        antler24.field_78807_k = true;
        setRotation(antler21, f4 / (180F / (float) Math.PI), f3 / (180F / (float) Math.PI), 0F);
        setRotation(head, f4 / (180F / (float) Math.PI) + 0.1570796F, f3 / (180F / (float) Math.PI), 0F);
        setRotation(antler11, f4 / (180F / (float) Math.PI), f3 / (180F / (float) Math.PI), 0F);
        setRotation(torso, 0.122173F, 0F, 0F);
        setRotation(collar, f4 / (3 * (180F / (float) Math.PI)) + 1.151917F, f3 / (3 * (180F / (float) Math.PI)), 0F);
        setRotation(neck, f4 / (1.5F * (180F / (float) Math.PI)) + 1.815142F, f3 / (1.5F * (180F / (float) Math.PI)), 0F);
        setRotation(rump, -0.0872665F, 0F, 0F);
        setRotation(body, 1.43117F, 0F, 0F);
        setRotation(calf1, 0.5585054F, 0F, -0.1745329F);
        setRotation(calf2, 0.5585054F, 0F, 0.1745329F);
        setRotation(toes3, 1.134464F, 0, 0);
        setRotation(hoof1, -1.134464F, 0F, 0F);
        setRotation(hoof2, -1.134464F, 0F, 0F);
        setRotation(hoof3, -1.134464F, 0F, 0F);
        setRotation(hoof4, -1.134464F, 0F, 0F);

        setRotation(tail, -1.308997F, 0F, 0F);

        setRotation(leg1, -22F / 180F * (float) Math.PI, 0F, 0F);
        setRotation(leg2, -22F / 180F * (float) Math.PI, 0F, 0F);
        setRotation(leg3, -0.3490659F, 0F, 0.0349066F);
        setRotation(leg4, -0.3490659F, 0F, -0.0349066F);

        setRotation(upperLeg4, 0.3490659F, 0F, 0.0349066F);
        setRotation(upperLeg3, 0.3490659F, 0F, -0.0349066F);
        setRotation(thigh1, -0.1745329F, 0F, 0.1745329F);
        setRotation(thigh2, -0.1745329F, 0F, -0.1745329F);

        if (!running)
        {
            setRotation(upperLeg4, MathHelper.func_76134_b(f / 1.5F + 3F * (float) Math.PI / 2F) * 0.7F * f1 + 0.3490659F, 0F, 0.0349066F);
            setRotation(upperLeg3, MathHelper.func_76134_b(f / 1.5F + (float) Math.PI / 2F) * 0.7F * f1 + 0.3490659F, 0F, -0.0349066F);
            setRotation(thigh1, MathHelper.func_76134_b(f / 1.5F + (float) Math.PI * 7F / 4F) * 0.7F * f1 - 0.1745329F, 0F, 0.1745329F);
            setRotation(thigh2, MathHelper.func_76134_b(f / 1.5F + 3f * (float) Math.PI / 4F) * 0.7F * f1 - 0.1745329F, 0F, -0.1745329F);
            if (MathHelper.func_76126_a(f / 1.5F + (float) Math.PI / 2F) * 0.7F * f1 > 0)
            {
                setRotation(lowerleg3, MathHelper.func_76126_a(f / 1.5F + (float) Math.PI / 2F) * 1.4F * f1, 0F, 0F);
                setRotation(leg3, -MathHelper.func_76126_a(f / 1.5F + (float) Math.PI / 2F) * 0.7F * f1 - 0.3490659F, 0F, 0.0349066F);
                setRotation(toes3, MathHelper.func_76126_a(f / 1.5F + (float) Math.PI / 2F) * 2.1F * f1 + 1.134464F, 0, 0);
            }
            if (MathHelper.func_76126_a(f / 1.5F + 1F * (float) Math.PI / 2F) * 0.7F * f1 < 0)
            {
                setRotation(lowerleg4, MathHelper.func_76126_a(f / 1.5F + 3F * (float) Math.PI / 2F) * 1.4F * f1, 0F, 0F);
                setRotation(leg4, -MathHelper.func_76126_a(f / 1.5F + 3F * (float) Math.PI / 2F) * 0.7F * f1 - 0.3490659F, 0F, -0.0349066F);
                setRotation(toes4, MathHelper.func_76126_a(f / 1.5F + 3F * (float) Math.PI / 2F) * 2.1F * f1 + 1.134464F, 0, 0);
            }
            if (MathHelper.func_76126_a(f / 1.5F + (float) Math.PI * 7F / 4F) * 0.7F * f1 > 0)
            {
                setRotation(calf1, MathHelper.func_76126_a(f / 1.5F + (float) Math.PI * 7F / 4F) * 1.4F * f1 + 0.5585054F, 0F, -0.1745329F);
                setRotation(leg1, -MathHelper.func_76126_a(f / 1.5F + (float) Math.PI * 7F / 4F) * 1.4F * f1 - 22F / 180F * (float) Math.PI, 0F, 0F);
                setRotation(toes1, MathHelper.func_76126_a(f / 1.5F + (float) Math.PI * 7F / 4F) * 2.1F * f1 + 1.134464F, 0F, 0F);
            }
            if (MathHelper.func_76126_a(f / 1.5F + 3f * (float) Math.PI / 4F) * 0.7F * f1 > 0)
            {
                setRotation(calf2, MathHelper.func_76126_a(f / 1.5F + 3f * (float) Math.PI / 4F) * 1.4F * f1 + 0.5585054F, 0F, 0.1745329F);
                setRotation(leg2, -MathHelper.func_76126_a(f / 1.5F + 3f * (float) Math.PI / 4F) * 1.4F * f1 - 22F / 180F * (float) Math.PI, 0F, 0F);
                setRotation(toes2, MathHelper.func_76126_a(f / 1.5F + 3f * (float) Math.PI / 4F) * 2.1F * f1 + 1.134464F, 0F, 0F);
            }

        }
        else
        {
            if (MathHelper.func_76134_b(f / 1.5F + 5 * (float) Math.PI / 4F) > -Math.sqrt(0.5) && MathHelper.func_76134_b(f / 1.5F + 5 * (float) Math.PI / 4F) < Math.sqrt(0.5))
            {
                setRotation(upperLeg4, MathHelper.func_76134_b(f / 1.5F + 5F * (float) Math.PI / 4F) * 2.8F * f1 + 0.3490659F, 0F, 0.0349066F);
            }
            if (MathHelper.func_76126_a(f / 1.5F + 5F * (float) Math.PI / 4F - 3F * (float) Math.PI / 8) > 0)
            {
                setRotation(lowerleg4, MathHelper.func_76126_a(f / 1.5F + 5F * (float) Math.PI / 4F - 3F * (float) Math.PI / 8) * 3.5F * f1, 0F, 0F);
                setRotation(leg4, -MathHelper.func_76126_a(f / 1.5F + 5F * (float) Math.PI / 4F - 3F * (float) Math.PI / 8) * 3.5F * f1 - 0.3490659F, 0F, -0.0349066F);
                setRotation(toes4, MathHelper.func_76126_a(f / 1.5F + 5F * (float) Math.PI / 4F - 3F * (float) Math.PI / 8) * 2.1F * f1 + 1.134464F, 0, 0);
            }


            if (MathHelper.func_76134_b(f / 1.5F + (float) Math.PI / 2F) > -Math.sqrt(0.5) && MathHelper.func_76134_b(f / 1.5F + (float) Math.PI / 2F) < Math.sqrt(0.5))
            {
                setRotation(upperLeg3, MathHelper.func_76134_b(f / 1.5F + (float) Math.PI / 2F) * 2.8F * f1 + 0.3490659F, 0F, -0.0349066F);
            }
            if (MathHelper.func_76126_a(f / 1.5F + (float) Math.PI / 2F - 3F * (float) Math.PI / 8) > 0)
            {
                setRotation(lowerleg3, MathHelper.func_76126_a(f / 1.5F + (float) Math.PI / 2F - 3F * (float) Math.PI / 8) * 3.5F * f1, 0F, 0F);
                setRotation(leg3, -MathHelper.func_76126_a(f / 1.5F + (float) Math.PI / 2F - 3F * (float) Math.PI / 8) * 3.5F * f1 - 0.3490659F, 0F, 0.0349066F);
                setRotation(toes3, MathHelper.func_76126_a(f / 1.5F + (float) Math.PI / 2F - 3F * (float) Math.PI / 8) * 2.1F * f1 + 1.134464F, 0, 0);
            }

            setRotation(thigh1, MathHelper.func_76134_b(f / 1.5F + (float) Math.PI * 7F / 4F) * 2.8F * f1 - 0.1745329F, 0F, 0.1745329F);
            setRotation(thigh2, MathHelper.func_76134_b(f / 1.5F + 3f * (float) Math.PI / 4F) * 2.8F * f1 - 0.1745329F, 0F, -0.1745329F);
        }
    }

    private void setRotation(ModelRenderer model, float x, float y, float z)
    {
        model.field_78795_f = x;
        model.field_78796_g = y;
        model.field_78808_h = z;
    }

}
