/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.agriculture;

import java.util.*;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import com.google.common.collect.ImmutableList;
import net.minecraft.block.Block;
import net.minecraft.block.BlockLeaves;
import net.minecraft.block.BlockPlanks;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.*;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.api.types.IFruitTree;
import net.dries007.tfc.objects.te.TETickCounter;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.OreDictionaryHelper;
import net.dries007.tfc.util.calendar.CalendarTFC;
import net.dries007.tfc.util.calendar.ICalendar;
import net.dries007.tfc.world.classic.ClimateTFC;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataTFC;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class BlockFruitTreeLeaves extends BlockLeaves
{
    public static final PropertyEnum<EnumLeafState> LEAF_STATE = PropertyEnum.func_177709_a("state", BlockFruitTreeLeaves.EnumLeafState.class);
    public static final PropertyBool HARVESTABLE = PropertyBool.func_177716_a("harvestable");
    private static final Map<IFruitTree, BlockFruitTreeLeaves> MAP = new HashMap<>();

    public static BlockFruitTreeLeaves get(IFruitTree tree)
    {
        return MAP.get(tree);
    }

    public final IFruitTree tree;

    public BlockFruitTreeLeaves(IFruitTree tree)
    {
        this.tree = tree;
        if (MAP.put(tree, this) != null) throw new IllegalStateException("There can only be one.");
        func_180632_j(field_176227_L.func_177621_b().func_177226_a(field_176237_a, false).func_177226_a(LEAF_STATE, EnumLeafState.NORMAL).func_177226_a(HARVESTABLE, false));
        field_185686_c = true; // Fast / Fancy graphics works correctly
        OreDictionaryHelper.register(this, "tree", "leaves");
        OreDictionaryHelper.register(this, "tree", "leaves", tree.getName());
        Blocks.field_150480_ab.func_180686_a(this, 30, 60);
        func_149675_a(true);
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public IBlockState func_176203_a(int meta)
    {
        return func_176223_P().func_177226_a(HARVESTABLE, meta > 3).func_177226_a(LEAF_STATE, EnumLeafState.valueOf(meta & 0b11));
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(LEAF_STATE).ordinal() + (state.func_177229_b(HARVESTABLE) ? 4 : 0);
    }

    @SuppressWarnings("deprecation")
    @Override
    public AxisAlignedBB func_180646_a(IBlockState blockState, IBlockAccess worldIn, BlockPos pos)
    {
        return field_185506_k;
    }

    @Override
    public void func_180645_a(World world, BlockPos pos, IBlockState state, Random random)
    {
        if (!world.field_72995_K)
        {
            if (state.func_177229_b(HARVESTABLE) && tree.isHarvestMonth(CalendarTFC.CALENDAR_TIME.getMonthOfYear()))
            {
                TETickCounter te = Helpers.getTE(world, pos, TETickCounter.class);
                if (te != null)
                {
                    float temp = ClimateTFC.getTemp(world, pos);
                    float rainfall = ChunkDataTFC.getRainfall(world, pos);
                    long hours = te.getTicksSinceUpdate() / ICalendar.TICKS_IN_HOUR;
                    if (hours > tree.getGrowthTime() && tree.isValidForGrowth(temp, rainfall))
                    {
                        world.func_175656_a(pos, world.func_180495_p(pos).func_177226_a(LEAF_STATE, EnumLeafState.FRUIT));
                        te.resetCounter();
                    }
                }
            }
            else if (tree.isFlowerMonth(CalendarTFC.CALENDAR_TIME.getMonthOfYear()))
            {
                if (world.func_180495_p(pos).func_177229_b(LEAF_STATE) != EnumLeafState.FLOWERING)
                {
                    world.func_175656_a(pos, world.func_180495_p(pos).func_177226_a(LEAF_STATE, EnumLeafState.FLOWERING));
                }
            }
            else
            {
                if (world.func_180495_p(pos).func_177229_b(LEAF_STATE) != EnumLeafState.NORMAL)
                {
                    world.func_175656_a(pos, world.func_180495_p(pos).func_177226_a(LEAF_STATE, EnumLeafState.NORMAL));
                }
            }
            doLeafDecay(world, pos, state);
        }
    }

    @Override
    public void func_180650_b(World worldIn, BlockPos pos, IBlockState state, Random rand)
    {
        doLeafDecay(worldIn, pos, state);
    }

    @Override
    @SuppressWarnings("deprecation")
    public void func_189540_a(IBlockState state, World world, BlockPos pos, @Nullable Block blockIn, @Nullable BlockPos fromPos)
    {
        doLeafDecay(world, pos, state);
    }

    @Override
    public void func_176213_c(World worldIn, BlockPos pos, IBlockState state)
    {
        TETickCounter tile = Helpers.getTE(worldIn, pos, TETickCounter.class);
        if (tile != null)
        {
            tile.resetCounter();
        }
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        if (!worldIn.field_72995_K)
        {
            if (worldIn.func_180495_p(pos).func_177229_b(LEAF_STATE) == EnumLeafState.FRUIT)
            {
                Helpers.spawnItemStack(worldIn, pos, tree.getFoodDrop());
                worldIn.func_175656_a(pos, worldIn.func_180495_p(pos).func_177226_a(LEAF_STATE, EnumLeafState.NORMAL));
                TETickCounter te = Helpers.getTE(worldIn, pos, TETickCounter.class);
                if (te != null)
                {
                    te.resetCounter();
                }
            }
        }
        return true;
    }

    @Override
    public void func_180634_a(World worldIn, BlockPos pos, IBlockState state, Entity entityIn)
    {
        //Copy-paste from BlockLeavesTFC
        //Player will take damage when falling through leaves if fall is over 9 blocks, fall damage is then set to 0.
        entityIn.func_180430_e((entityIn.field_70143_R - 6), 1.0F); // TODO: 17/4/18 Balance fall distance reduction.
        entityIn.field_70143_R = 0;
        //Entity motion is reduced by leaves.
        entityIn.field_70159_w *= 0.1D;
        if (entityIn.field_70181_x < 0)
        {
            entityIn.field_70181_x *= 0.1D;
        }
        entityIn.field_70179_y *= 0.1D;
    }

    @Override
    @Nonnull
    public BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, field_176237_a, LEAF_STATE, HARVESTABLE);
    }

    @Override
    public boolean hasTileEntity(IBlockState state)
    {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state)
    {
        return new TETickCounter();
    }

    @Override
    public List<ItemStack> onSheared(ItemStack item, IBlockAccess world, BlockPos pos, int fortune)
    {
        return ImmutableList.of(new ItemStack(this));
    }

    @SideOnly(Side.CLIENT)
    @Override
    public BlockRenderLayer func_180664_k()
    {
        /*
         * This is a way to make sure the leave settings are updated.
         * The result of this call is cached somewhere, so it's not that important, but:
         * The alternative would be to use `Minecraft.getMinecraft().gameSettings.fancyGraphics` directly in the 2 relevant methods.
         * It's better to do that than to refer to Blocks.LEAVES, for performance reasons.
         */
        field_185686_c = Minecraft.func_71410_x().field_71474_y.field_74347_j;
        return super.func_180664_k();
    }

    @Override
    public BlockPlanks.EnumType func_176233_b(int meta)
    {
        return BlockPlanks.EnumType.OAK;
    }

    @Override
    public void beginLeavesDecay(IBlockState state, World world, BlockPos pos)
    {
        // Don't do vanilla decay
    }

    @Override
    public void getDrops(NonNullList<ItemStack> drops, IBlockAccess world, BlockPos pos, IBlockState state, int fortune)
    {
        // Stops dropping oak saplings inherited from BlockLeaves
        drops.clear();
    }

    @SuppressWarnings("deprecation")
    @SideOnly(Side.CLIENT)
    @Override
    public boolean func_176225_a(IBlockState blockState, IBlockAccess blockAccess, BlockPos pos, EnumFacing side)
    {
        /*
         * See comment on getRenderLayer()
         */
        field_185686_c = Minecraft.func_71410_x().field_71474_y.field_74347_j;
        return true;// super.shouldSideBeRendered(blockState, blockAccess, pos, side);
    }

    private void doLeafDecay(World world, BlockPos pos, IBlockState state)
    {
        // TFC Leaf Decay, modified for fruit trees
        if (world.field_72995_K)
            return;

        List<BlockPos> paths = new ArrayList<>();
        List<BlockPos> pathsToAdd;
        BlockPos.MutableBlockPos pos1 = new BlockPos.MutableBlockPos(pos);
        IBlockState state1;
        paths.add(pos); // Center block

        // Fruit Tree Leaves need branches or full blocks within 2 blocks
        for (int i = 0; i < 2; i++)
        {
            pathsToAdd = new ArrayList<>();
            for (BlockPos p1 : paths)
            {
                for (EnumFacing face : EnumFacing.values())
                {
                    pos1.func_189533_g(p1).func_189536_c(face);
                    if (paths.contains(pos1.func_185334_h()))
                        continue;
                    state1 = world.func_180495_p(pos1);
                    if (state1.func_177230_c() == BlockFruitTreeTrunk.get(tree) || state1.func_177230_c() == BlockFruitTreeBranch.get(tree))
                        return;
                    if (state1.func_177230_c() == this)
                        pathsToAdd.add(pos1.func_185334_h());

                }
            }
            paths.addAll(pathsToAdd);
        }

        world.func_175698_g(pos);
    }

    /**
     * Enum state for blockstate
     * Used to render the correct texture of this leaf block
     */
    public enum EnumLeafState implements IStringSerializable
    {
        NORMAL, FLOWERING, FRUIT;

        private static final EnumLeafState[] VALUES = values();

        @Nonnull
        public static EnumLeafState valueOf(int index)
        {
            return index < 0 || index > VALUES.length ? NORMAL : VALUES[index];
        }

        @Override
        public String func_176610_l()
        {
            return this.name().toLowerCase();
        }
    }
}
