/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.agriculture;

import java.util.*;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import net.dries007.tfc.api.types.IFruitTree;
import net.dries007.tfc.objects.te.TETickCounter;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.calendar.ICalendar;
import net.dries007.tfc.world.classic.ClimateTFC;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataTFC;

@ParametersAreNonnullByDefault
public class BlockFruitTreeTrunk extends Block
{
    /* Connection sides (used if there's a branch on facing) */
    public static final PropertyBool NORTH = PropertyBool.func_177716_a("north");
    public static final PropertyBool SOUTH = PropertyBool.func_177716_a("south");
    public static final PropertyBool EAST = PropertyBool.func_177716_a("east");
    public static final PropertyBool WEST = PropertyBool.func_177716_a("west");
    private static final Map<IFruitTree, BlockFruitTreeTrunk> MAP = new HashMap<>();
    private static final AxisAlignedBB TRUNK_AABB = new AxisAlignedBB(0.3125D, 0.0D, 0.3125D, 0.6875D, 1.0D, 0.6875D);

    public static BlockFruitTreeTrunk get(IFruitTree tree)
    {
        return MAP.get(tree);
    }

    private final IFruitTree tree;

    public BlockFruitTreeTrunk(IFruitTree tree)
    {
        super(Material.field_151575_d, Material.field_151575_d.func_151565_r());
        if (MAP.put(tree, this) != null) throw new IllegalStateException("There can only be one.");
        func_149711_c(2.0F);
        func_149675_a(true);
        setHarvestLevel("axe", 0);
        func_149672_a(SoundType.field_185848_a);
        this.tree = tree;
        func_180632_j(field_176227_L.func_177621_b().func_177226_a(NORTH, false).func_177226_a(SOUTH, false).func_177226_a(EAST, false).func_177226_a(WEST, false));
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return 0;
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public IBlockState func_176221_a(@Nonnull IBlockState state, IBlockAccess worldIn, BlockPos pos)
    {
        for (EnumFacing face : EnumFacing.field_176754_o)
        {
            if (worldIn.func_180495_p(pos.func_177972_a(face)).func_177230_c() == BlockFruitTreeBranch.get(tree))
            {
                if (face == EnumFacing.NORTH)
                {
                    state = state.func_177226_a(NORTH, true);
                }
                else if (face == EnumFacing.SOUTH)
                {
                    state = state.func_177226_a(SOUTH, true);
                }
                else if (face == EnumFacing.EAST)
                {
                    state = state.func_177226_a(EAST, true);
                }
                else
                {
                    state = state.func_177226_a(WEST, true);
                }
            }
        }
        return state;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @Override
    @SuppressWarnings("deprecation")
    @Nonnull
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return TRUNK_AABB;
    }

    @Override
    @Nonnull
    @SuppressWarnings("deprecation")
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return BlockFaceShape.UNDEFINED;
    }

    @SuppressWarnings("deprecation")
    @Override
    public void func_185477_a(IBlockState state, @Nonnull World worldIn, @Nonnull BlockPos pos, @Nonnull AxisAlignedBB entityBox, @Nonnull List<AxisAlignedBB> collidingBoxes, @Nullable Entity entityIn, boolean isActualState)
    {
        func_185492_a(pos, entityBox, collidingBoxes, TRUNK_AABB);
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @Override
    public void func_180645_a(World worldIn, BlockPos pos, IBlockState state, Random random)
    {
        super.func_180650_b(worldIn, pos, state, random);
        if (!worldIn.field_72995_K)
        {
            // Attempt to grow
            float temp = ClimateTFC.getTemp(worldIn, pos);
            float rainfall = ChunkDataTFC.getRainfall(worldIn, pos);
            TETickCounter te = Helpers.getTE(worldIn, pos, TETickCounter.class);
            if (te != null)
            {
                long hours = te.getTicksSinceUpdate() / ICalendar.TICKS_IN_HOUR;
                if (hours > tree.getGrowthTime() && tree.isValidForGrowth(temp, rainfall))
                {
                    te.resetCounter();
                    if (worldIn.func_180495_p(pos.func_177984_a()).func_177230_c() != this) //If the above block is a trunk, this one don't need to do anything
                    {
                        if (this.getTrunkHeight(worldIn, pos) < 4)
                        {
                            BlockPos missingLeaf = getMissingLeaf(worldIn, pos);
                            if (missingLeaf != null)
                            {
                                //Missing leaf, spawn that first
                                if (worldIn.func_180495_p(missingLeaf).func_185904_a().func_76222_j())
                                {
                                    worldIn.func_175656_a(missingLeaf, BlockFruitTreeLeaves.get(tree).func_176223_P().func_177226_a(BlockFruitTreeLeaves.HARVESTABLE, false));
                                }
                            }
                            else
                            {
                                //Time to grow one trunk
                                worldIn.func_175656_a(pos.func_177984_a(), this.func_176223_P());
                                //Set all remaining leaf blocks to air
                                levelUpLeaves(worldIn, pos, false);
                            }
                        }
                        else
                        {
                            //Starting with top
                            if (worldIn.func_180495_p(pos.func_177984_a()).func_177230_c() != BlockFruitTreeBranch.get(tree))
                            {
                                BlockPos missingLeaf = getMissingLeaf(worldIn, pos);
                                if (missingLeaf != null)
                                {
                                    //There is a missing leaf
                                    if (worldIn.func_180495_p(missingLeaf).func_185904_a().func_76222_j())
                                    {
                                        worldIn.func_175656_a(missingLeaf, BlockFruitTreeLeaves.get(tree).func_176223_P().func_177226_a(BlockFruitTreeLeaves.HARVESTABLE, true));
                                    }
                                    return; //Done what we needed to do on this stage
                                }
                                else
                                {
                                    worldIn.func_175656_a(pos.func_177984_a(), BlockFruitTreeBranch.get(tree).func_176223_P());
                                    levelUpLeaves(worldIn, pos, true);
                                    return; //Done what we needed to do on this stage
                                }
                            }
                            else
                            {
                                BlockPos missingLeaf = getMissingLeaf(worldIn, pos.func_177984_a());
                                if (missingLeaf != null)
                                {
                                    //There is a missing leaf
                                    if (worldIn.func_180495_p(missingLeaf).func_185904_a().func_76222_j())
                                    {
                                        worldIn.func_175656_a(missingLeaf, BlockFruitTreeLeaves.get(tree).func_176223_P().func_177226_a(BlockFruitTreeLeaves.HARVESTABLE, true));
                                    }
                                    return; //Done what we needed to do on this stage
                                }
                                //is there a missing leaf in diagonals?
                                missingLeaf = getMissingLeaf(worldIn, pos.func_177984_a(), EnumFacing.NORTH);
                                if (missingLeaf != null)
                                {
                                    if (worldIn.func_180495_p(missingLeaf).func_185904_a().func_76222_j())
                                    {
                                        worldIn.func_175656_a(missingLeaf, BlockFruitTreeLeaves.get(tree).func_176223_P().func_177226_a(BlockFruitTreeLeaves.HARVESTABLE, true));
                                    }
                                    return; //Done what we needed to do on this stage
                                }
                                missingLeaf = getMissingLeaf(worldIn, pos.func_177984_a(), EnumFacing.SOUTH);
                                if (missingLeaf != null)
                                {
                                    if (worldIn.func_180495_p(missingLeaf).func_185904_a().func_76222_j())
                                    {
                                        worldIn.func_175656_a(missingLeaf, BlockFruitTreeLeaves.get(tree).func_176223_P().func_177226_a(BlockFruitTreeLeaves.HARVESTABLE, true));
                                    }
                                    return; //Done what we needed to do on this stage
                                }
                            }
                            List<EnumFacing> shuffledList = Arrays.asList(EnumFacing.field_176754_o.clone()); //if we don't clone, the original array shuffes!
                            Collections.shuffle(shuffledList, new Random(pos.func_177986_g()));
                            for (EnumFacing branchFacing : shuffledList)
                            {
                                BlockPos branchPos = pos.func_177972_a(branchFacing);
                                if (worldIn.func_180495_p(branchPos).func_177230_c() != BlockFruitTreeBranch.get(tree))
                                {
                                    if (worldIn.func_180495_p(branchPos).func_185904_a().func_76222_j())
                                    {
                                        worldIn.func_175656_a(branchPos, BlockFruitTreeBranch.get(tree).func_176223_P());
                                    }
                                    return; //Done what we needed to do on this stage
                                }
                                BlockPos missingLeaf = getMissingLeaf(worldIn, branchPos, branchFacing);
                                if (missingLeaf != null)
                                {
                                    if (worldIn.func_180495_p(missingLeaf).func_185904_a().func_76222_j())
                                    {
                                        worldIn.func_175656_a(missingLeaf, BlockFruitTreeLeaves.get(tree).func_176223_P().func_177226_a(BlockFruitTreeLeaves.HARVESTABLE, true));
                                        return; //Done what we needed to do on this stage
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    @Override
    public void func_176213_c(World worldIn, BlockPos pos, IBlockState state)
    {
        TETickCounter tile = Helpers.getTE(worldIn, pos, TETickCounter.class);
        if (tile != null)
        {
            tile.resetCounter();
        }
    }

    @Override
    public void func_180663_b(World worldIn, BlockPos pos, IBlockState state)
    {
        super.func_180663_b(worldIn, pos, state);
        if (!worldIn.field_72995_K)
        {
            //Destroy this tree upwards starting from this block
            //Not needed to destroy leaves, they will decay by itself
            Block branch = BlockFruitTreeBranch.get(tree);
            if (worldIn.func_180495_p(pos.func_177984_a()).func_177230_c() == this || worldIn.func_180495_p(pos.func_177984_a()).func_177230_c() == branch)
            {
                worldIn.func_175655_b(pos.func_177984_a(), false);
            }
            if (worldIn.func_180495_p(pos.func_177978_c()).func_177230_c() == branch)
            {
                worldIn.func_175655_b(pos.func_177984_a(), false);
            }
            if (worldIn.func_180495_p(pos.func_177968_d()).func_177230_c() == branch)
            {
                worldIn.func_175655_b(pos.func_177984_a(), false);
            }
            if (worldIn.func_180495_p(pos.func_177976_e()).func_177230_c() == branch)
            {
                worldIn.func_175655_b(pos.func_177984_a(), false);
            }
            if (worldIn.func_180495_p(pos.func_177974_f()).func_177230_c() == branch)
            {
                worldIn.func_175655_b(pos.func_177984_a(), false);
            }
        }
    }

    @Override
    @Nonnull
    public BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, NORTH, SOUTH, EAST, WEST);
    }

    @Override
    public boolean hasTileEntity(IBlockState state)
    {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state)
    {
        return new TETickCounter();
    }

    @Override
    @Nonnull
    public ItemStack getPickBlock(IBlockState state, RayTraceResult target, World world, BlockPos pos, EntityPlayer player)
    {
        return new ItemStack(BlockFruitTreeSapling.get(tree));
    }

    private BlockPos getMissingLeaf(World world, BlockPos branchPos, EnumFacing branchFacing)
    {
        //Helper method, gets a missing leaf block position that connects to this branch

        //the same facing has priority
        if (world.func_180495_p(branchPos.func_177972_a(branchFacing)).func_177230_c() != BlockFruitTreeLeaves.get(tree))
        {
            return branchPos.func_177972_a(branchFacing);
        }

        //The rest is shuffled
        List<BlockPos> positions = Arrays.asList(branchPos.func_177972_a(branchFacing.func_176746_e()),
            branchPos.func_177972_a(branchFacing.func_176746_e().func_176734_d()),
            branchPos.func_177972_a(branchFacing).func_177972_a(branchFacing.func_176746_e()),
            branchPos.func_177972_a(branchFacing).func_177972_a(branchFacing.func_176746_e().func_176734_d())
        );
        Collections.shuffle(positions);
        for (BlockPos pos : positions)
        {
            if (world.func_180495_p(pos).func_177230_c() != BlockFruitTreeLeaves.get(tree))
            {
                return pos;
            }
        }
        return null; //All leaves are in position
    }

    @Nullable
    private BlockPos getMissingLeaf(World world, BlockPos pos)
    {
        //Helper method, checks if all 5 leafs blocks(horizontals + above) has a missing block leaf

        //Above first
        if (world.func_180495_p(pos.func_177972_a(EnumFacing.UP)).func_177230_c() != BlockFruitTreeLeaves.get(tree))
        {
            return pos.func_177972_a(EnumFacing.UP);
        }
        List<EnumFacing> shuffledList = Arrays.asList(EnumFacing.field_176754_o.clone()); //if i don't clone, the original array shuffes!
        Collections.shuffle(shuffledList);
        for (EnumFacing facing : shuffledList)
        {
            if (world.func_180495_p(pos.func_177972_a(facing)).func_177230_c() != BlockFruitTreeLeaves.get(tree))
            {
                return pos.func_177972_a(facing);
            }
        }
        return null;
    }

    private void levelUpLeaves(World world, BlockPos oldCenter, boolean hasvestable)
    {
        //Remove the old ones
        for (EnumFacing facing : EnumFacing.field_176754_o)
        {
            if (world.func_180495_p(oldCenter.func_177972_a(facing)).func_177230_c() == BlockFruitTreeLeaves.get(tree))
            {
                world.func_175698_g(oldCenter.func_177972_a(facing));
            }
        }
        BlockPos missingLeaf = null;
        //noinspection ConstantConditions
        do
        {
            //Respawn leafs in the new height
            missingLeaf = getMissingLeaf(world, oldCenter.func_177984_a());
            if (missingLeaf != null && world.func_180495_p(missingLeaf).func_185904_a().func_76222_j())
            {
                world.func_175656_a(missingLeaf, BlockFruitTreeLeaves.get(tree).func_176223_P().func_177226_a(BlockFruitTreeLeaves.HARVESTABLE, hasvestable));
            }
            else
            {
                //Found a block(ceiling?) that prevents this tree to grow leafs, aborting
                break;
            }
        } while (missingLeaf != null);
    }

    private int getTrunkHeight(World world, BlockPos pos)
    {
        for (int i = 1; i < 4; i++)
        {
            if (world.func_180495_p(pos.func_177979_c(i)).func_177230_c() != this) return i;
        }
        return 4;
    }
}
