/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.devices;

import java.util.List;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandler;

import net.dries007.tfc.api.capability.size.IItemSize;
import net.dries007.tfc.api.capability.size.Size;
import net.dries007.tfc.api.capability.size.Weight;
import net.dries007.tfc.client.TFCGuiHandler;
import net.dries007.tfc.client.TFCSounds;
import net.dries007.tfc.objects.items.ItemHandstone;
import net.dries007.tfc.objects.te.TEQuern;
import net.dries007.tfc.util.Helpers;

@ParametersAreNonnullByDefault
public class BlockQuern extends Block implements IItemSize
{
    private static final AxisAlignedBB BASE_AABB = new AxisAlignedBB(0D, 0D, 0D, 1D, 0.625D, 1D);
    private static final AxisAlignedBB QUERN_AABB = new AxisAlignedBB(0D, 0D, 0D, 1D, 0.875D, 1D);
    private static final AxisAlignedBB HANDSTONE_AABB = new AxisAlignedBB(0.375D, 0.625D, 0.375D, 0.625D, 0.875D, 0.625D);

    public BlockQuern()
    {
        super(Material.field_151576_e);
        func_149711_c(3.0f);
        func_149672_a(SoundType.field_185851_d);
    }

    @Override
    @Nonnull
    public Size getSize(ItemStack stack)
    {
        return Size.HUGE;
    }

    @Override
    @Nonnull
    public Weight getWeight(ItemStack stack)
    {
        return Weight.HEAVY;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_185481_k(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149730_j(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149637_q(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149721_r(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        TEQuern teQuern = Helpers.getTE(source, pos, TEQuern.class);
        if (teQuern != null && teQuern.hasHandstone())
        {
            return QUERN_AABB;
        }
        else
        {
            return BASE_AABB;
        }
    }

    @Override
    @Nonnull
    @SuppressWarnings("deprecation")
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        if (face == EnumFacing.DOWN)
        {
            return BlockFaceShape.SOLID;
        }
        return BlockFaceShape.UNDEFINED;
    }

    @SuppressWarnings("deprecation")
    @Override
    public void func_185477_a(IBlockState state, World world, BlockPos pos, AxisAlignedBB entityBox, List<AxisAlignedBB> collidingBoxes, @Nullable Entity entityIn, boolean isActualState)
    {
        func_185492_a(pos, entityBox, collidingBoxes, BASE_AABB);
        TEQuern teQuern = Helpers.getTE(world, pos, TEQuern.class);
        if (teQuern != null && teQuern.hasHandstone())
        {
            func_185492_a(pos, entityBox, collidingBoxes, HANDSTONE_AABB);
        }
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @Override
    public void func_180663_b(World world, BlockPos pos, IBlockState state)
    {
        TEQuern teQuern = Helpers.getTE(world, pos, TEQuern.class);
        if (teQuern != null)
        {
            teQuern.onBreakBlock(world, pos);
        }
        super.func_180663_b(world, pos, state);
    }

    @Override
    public boolean func_180639_a(World world, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        if (hand.equals(EnumHand.MAIN_HAND))
        {
            ItemStack playerStack = playerIn.func_184586_b(EnumHand.MAIN_HAND);

            TEQuern teQuern = Helpers.getTE(world, pos, TEQuern.class);
            if (teQuern != null)
            {
                boolean isGrinding = teQuern.isGrinding();
                if (!isGrinding && !playerIn.func_70093_af())
                {
                    if (playerStack.func_190926_b() && facing == EnumFacing.UP && hitX > 0.2f && hitX < 0.4f && hitZ > 0.2f && hitZ < 0.4f && hitY >= 0.875)
                    {
                        teQuern.grind();
                        world.func_184133_a(null, pos, TFCSounds.QUERN_USE, SoundCategory.BLOCKS, 1, 1 + ((world.field_73012_v.nextFloat() - world.field_73012_v.nextFloat()) / 16));
                    }
                    else if (!world.field_72995_K)
                    {
                        TFCGuiHandler.openGui(world, pos, playerIn, TFCGuiHandler.Type.QUERN);
                    }
                    return true;
                }
                else if (!world.field_72995_K && playerIn.func_70093_af())
                {
                    IItemHandler inventory = teQuern.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null);
                    if (inventory != null)
                    {
                        ItemStack inputStack = inventory.getStackInSlot(TEQuern.SLOT_INPUT);
                        ItemStack outputStack = inventory.getStackInSlot(TEQuern.SLOT_OUTPUT);

                        if (hitX > 0.2f && hitX < 0.8f && hitZ > 0.2f && hitZ < 0.8f)
                        {
                            if (hitX > 0.4f && hitX < 0.6f && hitZ > 0.4f && hitZ < 0.6f && playerStack.func_190926_b() && !inputStack.func_190926_b())
                            {
                                playerIn.func_184611_a(EnumHand.MAIN_HAND, inventory.extractItem(TEQuern.SLOT_INPUT, inputStack.func_190916_E(), false));
                                teQuern.setAndUpdateSlots(TEQuern.SLOT_INPUT);
                                world.func_184133_a(null, pos, SoundEvents.field_187550_ag, SoundCategory.BLOCKS, 1, 1);
                            }
                            else if (!isGrinding && playerStack.func_190926_b() && teQuern.hasHandstone())
                            {
                                playerIn.func_184611_a(EnumHand.MAIN_HAND, inventory.extractItem(TEQuern.SLOT_HANDSTONE, inventory.getStackInSlot(TEQuern.SLOT_HANDSTONE).func_190916_E(), false));
                                teQuern.setAndUpdateSlots(TEQuern.SLOT_HANDSTONE);
                                world.func_184133_a(null, pos, SoundEvents.field_187843_fX, SoundCategory.BLOCKS, 1, 1);
                            }
                            else if (!isGrinding && !playerStack.func_190926_b() && teQuern.isItemValid(TEQuern.SLOT_HANDSTONE, playerStack))
                            {
                                if (playerStack.func_190916_E() == 1)
                                    playerIn.func_184611_a(EnumHand.MAIN_HAND, teQuern.insertOrSwapItem(TEQuern.SLOT_HANDSTONE, playerStack));
                                else
                                    playerIn.func_191521_c(teQuern.insertOrSwapItem(TEQuern.SLOT_HANDSTONE, playerStack.func_77979_a(1)));
                                teQuern.setAndUpdateSlots(TEQuern.SLOT_HANDSTONE);
                                world.func_184133_a(null, pos, SoundEvents.field_187845_fY, SoundCategory.BLOCKS, 1, 1);
                            }
                        }
                        else if ((hitX <= 0.2f || hitX >= 0.8f || hitZ <= 0.2f || hitZ >= 0.8f) && hitY >= 0.625)
                        {
                            if (playerStack.func_190926_b() && !outputStack.func_190926_b())
                            {
                                playerIn.func_184611_a(EnumHand.MAIN_HAND, teQuern.takeCraftingResult(outputStack));
                                teQuern.setAndUpdateSlots(TEQuern.SLOT_OUTPUT);
                                world.func_184133_a(null, pos, SoundEvents.field_187550_ag, SoundCategory.BLOCKS, 1, 1);
                            }
                            else if (!playerStack.func_190926_b() && !outputStack.func_190926_b() && !(playerStack.func_77973_b() instanceof ItemHandstone))
                            {
                                playerIn.func_191521_c(teQuern.takeCraftingResult(outputStack));
                                teQuern.setAndUpdateSlots(TEQuern.SLOT_OUTPUT);
                                world.func_184133_a(null, pos, SoundEvents.field_187552_ah, SoundCategory.BLOCKS, 1, 1);
                            }
                            else if (!isGrinding && playerStack.func_190926_b() && teQuern.hasHandstone())
                            {
                                playerIn.func_184611_a(EnumHand.MAIN_HAND, inventory.extractItem(TEQuern.SLOT_HANDSTONE, inventory.getStackInSlot(TEQuern.SLOT_HANDSTONE).func_190916_E(), false));
                                teQuern.setAndUpdateSlots(TEQuern.SLOT_HANDSTONE);
                                world.func_184133_a(null, pos, SoundEvents.field_187843_fX, SoundCategory.BLOCKS, 1, 1);
                            }
                            else if (!isGrinding && !playerStack.func_190926_b() && teQuern.isItemValid(TEQuern.SLOT_HANDSTONE, playerStack))
                            {
                                if (playerStack.func_190916_E() == 1)
                                {
                                    playerIn.func_184611_a(EnumHand.MAIN_HAND, teQuern.insertOrSwapItem(TEQuern.SLOT_HANDSTONE, playerStack));
                                }
                                else
                                {
                                    playerIn.func_191521_c(teQuern.insertOrSwapItem(TEQuern.SLOT_HANDSTONE, playerStack.func_77979_a(1)));
                                }
                                teQuern.setAndUpdateSlots(TEQuern.SLOT_HANDSTONE);
                                world.func_184133_a(null, pos, SoundEvents.field_187845_fY, SoundCategory.BLOCKS, 1, 1);
                            }
                        }
                        return true;
                    }
                }
            }
        }
        return false;
    }

    @Override
    public boolean isNormalCube(IBlockState state, IBlockAccess world, BlockPos pos)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean isSideSolid(IBlockState baseState, IBlockAccess world, BlockPos pos, EnumFacing side)
    {
        return side == EnumFacing.DOWN;
    }

    @Override
    public boolean hasTileEntity(IBlockState state)
    {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state)
    {
        return new TEQuern();
    }
}
