/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.metal;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.objects.items.metal.ItemMetalSheet;
import net.dries007.tfc.objects.te.TEMetalSheet;
import net.dries007.tfc.util.Helpers;

@ParametersAreNonnullByDefault
public class BlockMetalSheet extends Block
{
    public static final PropertyBool[] FACE_PROPERTIES = new PropertyBool[] {
        PropertyBool.func_177716_a("down"),
        PropertyBool.func_177716_a("up"),
        PropertyBool.func_177716_a("north"),
        PropertyBool.func_177716_a("south"),
        PropertyBool.func_177716_a("west"),
        PropertyBool.func_177716_a("east")
    };
    private static final Map<Metal, BlockMetalSheet> MAP = new HashMap<>();
    private static final AxisAlignedBB[] SHEET_AABB = new AxisAlignedBB[] {
        new AxisAlignedBB(0d, 0.9375d, 0d, 1d, 1d, 1d),
        new AxisAlignedBB(0d, 0d, 0d, 1d, 0.0625d, 1d),
        new AxisAlignedBB(0d, 0d, 0.9375d, 1d, 1d, 1d),
        new AxisAlignedBB(0d, 0d, 0d, 1d, 1d, 0.0625d),
        new AxisAlignedBB(0.9375d, 0d, 0d, 1d, 1d, 1d),
        new AxisAlignedBB(0d, 0d, 0d, 0.0625d, 1d, 1d)
    };

    public static BlockMetalSheet get(Metal metal)
    {
        return MAP.get(metal);
    }

    public static ItemStack get(Metal metal, int amount)
    {
        return new ItemStack(MAP.get(metal), amount);
    }

    private final Metal metal;

    public BlockMetalSheet(Metal metal)
    {
        super(Material.field_151573_f);

        this.metal = metal;
        if (MAP.put(metal, this) != null) throw new IllegalStateException("There can only be one.");

        func_149711_c(40F);
        func_149752_b(25F);
        setHarvestLevel("pickaxe", 0);

        func_180632_j(this.field_176227_L.func_177621_b().func_177226_a(FACE_PROPERTIES[0], false).func_177226_a(FACE_PROPERTIES[1], false).func_177226_a(FACE_PROPERTIES[2], false).func_177226_a(FACE_PROPERTIES[3], false).func_177226_a(FACE_PROPERTIES[4], false).func_177226_a(FACE_PROPERTIES[5], false));
    }

    @Nonnull
    public Metal getMetal()
    {
        return metal;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_185481_k(IBlockState state)
    {
        return false;
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return 0;
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public IBlockState func_176221_a(IBlockState state, IBlockAccess worldIn, BlockPos pos)
    {
        TEMetalSheet tile = Helpers.getTE(worldIn, pos, TEMetalSheet.class);
        if (tile != null)
        {
            for (EnumFacing face : EnumFacing.values())
            {
                state = state.func_177226_a(FACE_PROPERTIES[face.func_176745_a()], tile.getFace(face));
            }
        }
        return state;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149637_q(IBlockState state)
    {
        return false;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149721_r(IBlockState state)
    {
        return false;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @Override
    @Nonnull
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        TEMetalSheet tile = Helpers.getTE(source, pos, TEMetalSheet.class);
        int sheets = 0;
        AxisAlignedBB boundingBox = field_185505_j;
        if (tile != null)
        {
            for (EnumFacing face : EnumFacing.values())
            {
                if (tile.getFace(face))
                {
                    if (sheets == 1)
                    {
                        return field_185505_j;
                    }
                    else
                    {
                        boundingBox = SHEET_AABB[face.func_176745_a()];
                        sheets++;
                    }
                }
            }
        }
        // This should't ever return null, since it will return FULL_BLOCK_AABB before that
        return boundingBox;
    }

    @Override
    @Nonnull
    @SuppressWarnings("deprecation")
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return BlockFaceShape.UNDEFINED;
    }


    @SuppressWarnings("deprecation")
    @Override
    public void func_185477_a(IBlockState state, World worldIn, BlockPos pos, AxisAlignedBB entityBox, List<AxisAlignedBB> collidingBoxes, @Nullable Entity entityIn, boolean isActualState)
    {
        TEMetalSheet tile = Helpers.getTE(worldIn, pos, TEMetalSheet.class);
        if (tile != null)
        {
            for (EnumFacing face : EnumFacing.values())
            {
                if (tile.getFace(face))
                {
                    func_185492_a(pos, entityBox, collidingBoxes, SHEET_AABB[face.func_176745_a()]);
                }
            }
        }
    }

    @Nullable
    @Override
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_180646_a(IBlockState blockState, IBlockAccess worldIn, BlockPos pos)
    {
        return field_185506_k;
    }

    @SideOnly(Side.CLIENT)
    @Override
    @Nonnull
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_180640_a(IBlockState state, World worldIn, BlockPos pos)
    {
        return func_185496_a(state, worldIn, pos);
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }


    @Override
    @SuppressWarnings("deprecation")
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos)
    {
        TEMetalSheet tile = Helpers.getTE(worldIn, pos, TEMetalSheet.class);
        if (tile != null)
        {
            for (EnumFacing face : EnumFacing.values())
            {
                if (tile.getFace(face) && !worldIn.isSideSolid(pos.func_177972_a(face.func_176734_d()), face))
                {
                    InventoryHelper.func_180173_a(worldIn, pos.func_177958_n() + 0.5, pos.func_177956_o() + 0.5, pos.func_177952_p() + 0.5, new ItemStack(ItemMetalSheet.get(metal, Metal.ItemType.SHEET)));
                    tile.setFace(face, false);
                }
            }
            if (tile.getFaceCount() == 0)
            {
                // Remove the block
                worldIn.func_175698_g(pos);
            }
        }
    }

    @Override
    public void func_180663_b(World worldIn, BlockPos pos, IBlockState state)
    {
        TEMetalSheet te = Helpers.getTE(worldIn, pos, TEMetalSheet.class);
        if (te != null) te.onBreakBlock(this.metal);
        super.func_180663_b(worldIn, pos, state);
    }

    @Nullable
    @Override
    @SuppressWarnings("deprecation")
    public RayTraceResult func_180636_a(IBlockState blockState, World worldIn, BlockPos pos, Vec3d start, Vec3d end)
    {
        TEMetalSheet tile = Helpers.getTE(worldIn, pos, TEMetalSheet.class);
        if (tile != null)
        {
            for (EnumFacing face : EnumFacing.values())
            {
                if (tile.getFace(face))
                {
                    RayTraceResult result = func_185503_a(pos, start, end, SHEET_AABB[face.func_176745_a()]);
                    if (result != null)
                    {
                        return result;
                    }
                }
            }
        }
        return null;
    }

    @Override
    @Nonnull
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, FACE_PROPERTIES);
    }

    @Override
    public boolean isNormalCube(IBlockState state, IBlockAccess world, BlockPos pos)
    {
        return false;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean isSideSolid(IBlockState base_state, IBlockAccess world, BlockPos pos, EnumFacing side)
    {
        return false;
    }

    @Override
    public boolean hasTileEntity(IBlockState state)
    {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state)
    {
        return new TEMetalSheet();
    }
}
