/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.plants;

import java.util.HashMap;
import java.util.Map;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.BlockFence;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.Mirror;
import net.minecraft.util.Rotation;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.api.types.Plant;
import net.dries007.tfc.world.classic.ClimateTFC;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataTFC;

import net.minecraft.block.Block.EnumOffsetType;

@ParametersAreNonnullByDefault
public class BlockCreepingPlantTFC extends BlockPlantTFC
{
    static final PropertyBool DOWN = PropertyBool.func_177716_a("down");
    static final PropertyBool UP = PropertyBool.func_177716_a("up");
    static final PropertyBool NORTH = PropertyBool.func_177716_a("north");
    static final PropertyBool EAST = PropertyBool.func_177716_a("east");
    static final PropertyBool SOUTH = PropertyBool.func_177716_a("south");
    static final PropertyBool WEST = PropertyBool.func_177716_a("west");

    private static final PropertyBool[] ALL_FACES = new PropertyBool[] {DOWN, UP, NORTH, SOUTH, WEST, EAST};

    private static final AxisAlignedBB DOWN_AABB = new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 0.125D, 1.0D);
    private static final AxisAlignedBB UP_AABB = new AxisAlignedBB(0.0D, 0.875D, 0.0D, 1.0D, 1.0D, 1.0D);
    private static final AxisAlignedBB WEST_AABB = new AxisAlignedBB(0.0D, 0.0D, 0.0D, 0.125D, 1.0D, 1.0D);
    private static final AxisAlignedBB EAST_AABB = new AxisAlignedBB(0.875D, 0.0D, 0.0D, 1.0D, 1.0D, 1.0D);
    private static final AxisAlignedBB NORTH_AABB = new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 1.0D, 0.125D);
    private static final AxisAlignedBB SOUTH_AABB = new AxisAlignedBB(0.0D, 0.0D, 0.875D, 1.0D, 1.0D, 1.0D);
    private static final Map<Plant, BlockCreepingPlantTFC> MAP = new HashMap<>();

    public static BlockCreepingPlantTFC get(Plant plant)
    {
        return BlockCreepingPlantTFC.MAP.get(plant);
    }

    public BlockCreepingPlantTFC(Plant plant)
    {
        super(plant);
        if (MAP.put(plant, this) != null) throw new IllegalStateException("There can only be one.");
    }

    @Override
    @Nonnull
    public IBlockState func_176221_a(IBlockState state, IBlockAccess worldIn, BlockPos pos)
    {
        return super.func_176221_a(state, worldIn, pos)
            .func_177226_a(DOWN, canPlantConnectTo(worldIn, pos, EnumFacing.DOWN))
            .func_177226_a(UP, canPlantConnectTo(worldIn, pos, EnumFacing.UP))
            .func_177226_a(NORTH, canPlantConnectTo(worldIn, pos, EnumFacing.NORTH))
            .func_177226_a(EAST, canPlantConnectTo(worldIn, pos, EnumFacing.EAST))
            .func_177226_a(SOUTH, canPlantConnectTo(worldIn, pos, EnumFacing.SOUTH))
            .func_177226_a(WEST, canPlantConnectTo(worldIn, pos, EnumFacing.WEST));
    }

    @Override
    @Nonnull
    public Block.EnumOffsetType func_176218_Q()
    {
        return EnumOffsetType.NONE;
    }

    @Override
    protected boolean func_185514_i(IBlockState state)
    {
        return true;
    }

    @Override
    public boolean func_180671_f(World worldIn, BlockPos pos, IBlockState state)
    {
        for (EnumFacing face : EnumFacing.values())
        {
            IBlockState blockState = worldIn.func_180495_p(pos.func_177972_a(face));
            if (blockState.func_193401_d(worldIn, pos.func_177972_a(face), face.func_176734_d()) == BlockFaceShape.SOLID || blockState.func_177230_c() instanceof BlockFence)
            {
                return plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, pos)) && plant.isValidRain(ChunkDataTFC.getRainfall(worldIn, pos));
            }
        }
        return false;
    }

    @Override
    @Nonnull
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        state = state.func_185899_b(source, pos);

        int i = 0;
        AxisAlignedBB axisalignedbb = field_185505_j;

        for (PropertyBool propertybool : ALL_FACES)
        {
            if ((state.func_177229_b(propertybool)))
            {
                switch (propertybool.func_177701_a())
                {
                    case "down":
                        axisalignedbb = DOWN_AABB;
                        ++i;
                        break;
                    case "up":
                        axisalignedbb = UP_AABB;
                        ++i;
                        break;
                    case "north":
                        axisalignedbb = NORTH_AABB;
                        ++i;
                        break;
                    case "south":
                        axisalignedbb = SOUTH_AABB;
                        ++i;
                        break;
                    case "west":
                        axisalignedbb = WEST_AABB;
                        ++i;
                        break;
                    case "east":
                        axisalignedbb = EAST_AABB;
                        ++i;
                        break;
                    default:
                        axisalignedbb = field_185505_j;
                }
            }
        }

        return i == 1 ? axisalignedbb : field_185505_j;
    }

    @Override
    @Nullable
    public AxisAlignedBB func_180646_a(IBlockState blockState, IBlockAccess worldIn, BlockPos pos)
    {
        return field_185506_k;
    }

    @Override
    @Nonnull
    protected BlockStateContainer createPlantBlockState()
    {
        return new BlockStateContainer(this, DOWN, UP, NORTH, EAST, WEST, SOUTH, growthStageProperty, DAYPERIOD, AGE);
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public IBlockState func_185499_a(IBlockState state, Rotation rot)
    {
        switch (rot)
        {
            case CLOCKWISE_180:
                return state.func_177226_a(growthStageProperty, plant.getStageForMonth()).func_177226_a(NORTH, state.func_177229_b(SOUTH)).func_177226_a(EAST, state.func_177229_b(WEST)).func_177226_a(SOUTH, state.func_177229_b(NORTH)).func_177226_a(WEST, state.func_177229_b(EAST));
            case COUNTERCLOCKWISE_90:
                return state.func_177226_a(growthStageProperty, plant.getStageForMonth()).func_177226_a(NORTH, state.func_177229_b(EAST)).func_177226_a(EAST, state.func_177229_b(SOUTH)).func_177226_a(SOUTH, state.func_177229_b(WEST)).func_177226_a(WEST, state.func_177229_b(NORTH));
            case CLOCKWISE_90:
                return state.func_177226_a(growthStageProperty, plant.getStageForMonth()).func_177226_a(NORTH, state.func_177229_b(WEST)).func_177226_a(EAST, state.func_177229_b(NORTH)).func_177226_a(SOUTH, state.func_177229_b(EAST)).func_177226_a(WEST, state.func_177229_b(SOUTH));
            default:
                return state.func_177226_a(growthStageProperty, plant.getStageForMonth());
        }
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public IBlockState func_185471_a(IBlockState state, Mirror mirrorIn)
    {
        switch (mirrorIn)
        {
            case LEFT_RIGHT:
                return state.func_177226_a(growthStageProperty, plant.getStageForMonth()).func_177226_a(NORTH, state.func_177229_b(SOUTH)).func_177226_a(SOUTH, state.func_177229_b(NORTH));
            case FRONT_BACK:
                return state.func_177226_a(growthStageProperty, plant.getStageForMonth()).func_177226_a(EAST, state.func_177229_b(WEST)).func_177226_a(WEST, state.func_177229_b(EAST));
            default:
                return super.func_185471_a(state, mirrorIn);
        }
    }

    @Override
    public boolean func_176205_b(IBlockAccess worldIn, BlockPos pos)
    {
        return true;
    }

    @SuppressWarnings("deprecation")
    @Override
    @SideOnly(Side.CLIENT)
    public boolean func_176225_a(IBlockState blockState, IBlockAccess blockAccess, BlockPos pos, EnumFacing side)
    {
        return true;
    }

    @Override
    public boolean canBeConnectedTo(IBlockAccess world, BlockPos pos, EnumFacing facing)
    {
        return canConnectTo(world, pos.func_177972_a(facing), facing.func_176734_d()) && !(world.func_180495_p(pos.func_177972_a(facing)).func_177230_c() instanceof BlockFence);
    }

    @Override
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos)
    {
        if (!worldIn.field_72995_K)
        {
            if (!func_180671_f(worldIn, pos, state))
            {
                worldIn.func_175655_b(pos, true);
            }
        }
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return BlockFaceShape.UNDEFINED;
    }

    protected boolean canConnectTo(IBlockAccess worldIn, BlockPos pos, EnumFacing facing)
    {
        IBlockState iblockstate = worldIn.func_180495_p(pos);
        BlockFaceShape blockfaceshape = iblockstate.func_193401_d(worldIn, pos, facing);
        Block block = iblockstate.func_177230_c();
        return blockfaceshape == BlockFaceShape.SOLID || block instanceof BlockFence;
    }

    protected boolean canPlantConnectTo(IBlockAccess world, BlockPos pos, EnumFacing facing)
    {
        BlockPos other = pos.func_177972_a(facing);
        Block block = world.func_180495_p(other).func_177230_c();
        return block.canBeConnectedTo(world, other, facing.func_176734_d()) || canConnectTo(world, other, facing.func_176734_d());
    }
}
