/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.plants;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.IGrowable;
import net.minecraft.block.state.IBlockState;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.EnumSkyBlock;
import net.minecraft.world.World;

import net.dries007.tfc.api.types.Plant;
import net.dries007.tfc.world.classic.ClimateTFC;

@ParametersAreNonnullByDefault
public class BlockMushroomTFC extends BlockPlantTFC implements IGrowable
{
    private static final Map<Plant, BlockMushroomTFC> MAP = new HashMap<>();

    public static BlockMushroomTFC get(Plant plant)
    {
        return MAP.get(plant);
    }

    public BlockMushroomTFC(Plant plant)
    {
        super(plant);
        if (MAP.put(plant, this) != null) throw new IllegalStateException("There can only be one.");
    }

    @Override
    public boolean func_176196_c(World worldIn, BlockPos pos)
    {
        return super.func_176196_c(worldIn, pos) && this.func_180671_f(worldIn, pos, this.func_176223_P());
    }

    @Override
    protected boolean func_185514_i(IBlockState state)
    {
        return state.func_185913_b();
    }

    @Override
    public void func_180650_b(World worldIn, BlockPos pos, IBlockState state, Random rand)
    {
        if (!worldIn.func_175697_a(pos, 1)) return;

        if (plant.isValidGrowthTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, pos)) && plant.isValidSunlight(Math.subtractExact(worldIn.func_175642_b(EnumSkyBlock.SKY, pos), worldIn.func_175657_ab())))
        {
            int j = state.func_177229_b(AGE);

            if (rand.nextDouble() < getGrowthRate(worldIn, pos) && net.minecraftforge.common.ForgeHooks.onCropsGrowPre(worldIn, pos.func_177984_a(), state, true))
            {
                if (j == 3 && func_176473_a(worldIn, pos, state, worldIn.field_72995_K))
                {
                    func_176474_b(worldIn, rand, pos, state);
                }
                else if (j < 3)
                {
                    worldIn.func_175656_a(pos, state.func_177226_a(AGE, j + 1));
                }
                net.minecraftforge.common.ForgeHooks.onCropsGrowPost(worldIn, pos, state, worldIn.func_180495_p(pos));
            }
        }
        else if (!plant.isValidGrowthTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, pos)) || !plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, pos)))
        {
            int j = state.func_177229_b(AGE);

            if (rand.nextDouble() < getGrowthRate(worldIn, pos) && net.minecraftforge.common.ForgeHooks.onCropsGrowPre(worldIn, pos, state, true))
            {
                if (j == 0 && canShrink(worldIn, pos))
                {
                    shrink(worldIn, pos);
                }
                else if (j > 0)
                {
                    worldIn.func_175656_a(pos, state.func_177226_a(AGE, j - 1));
                }
                net.minecraftforge.common.ForgeHooks.onCropsGrowPost(worldIn, pos, state, worldIn.func_180495_p(pos));
            }
        }

        func_176475_e(worldIn, pos, state);
    }

    @Override
    public boolean func_180671_f(World worldIn, BlockPos pos, IBlockState state)
    {
        if (!worldIn.func_189509_E(pos))
        {
            IBlockState soil = worldIn.func_180495_p(pos.func_177977_b());
            return plant.isValidSunlight(worldIn.func_175699_k(pos)) && soil.func_177230_c().canSustainPlant(soil, worldIn, pos.func_177977_b(), net.minecraft.util.EnumFacing.UP, this);
        }
        else
        {
            return false;
        }
    }

    @Override
    public boolean func_176473_a(World worldIn, BlockPos pos, IBlockState state, boolean isClient)
    {
        int i = 5;

        for (BlockPos blockpos : BlockPos.func_177975_b(pos.func_177982_a(-4, -1, -4), pos.func_177982_a(4, 1, 4)))
        {
            if (worldIn.func_180495_p(blockpos).func_177230_c() == this)
            {
                --i;

                if (i <= 0)
                {
                    return false;
                }
            }
        }
        return true;
    }

    @Override
    public boolean func_180670_a(World worldIn, Random rand, BlockPos pos, IBlockState state)
    {
        return true;
    }

    @Override
    public void func_176474_b(World worldIn, Random rand, BlockPos pos, IBlockState state)
    {
        BlockPos blockpos1 = pos.func_177982_a(rand.nextInt(3) - 1, rand.nextInt(2) - rand.nextInt(2), rand.nextInt(3) - 1);

        for (int k = 0; k < 4; ++k)
        {
            if (worldIn.func_175623_d(blockpos1) && this.func_180671_f(worldIn, blockpos1, this.func_176223_P()))
            {
                pos = blockpos1;
            }

            blockpos1 = pos.func_177982_a(rand.nextInt(3) - 1, rand.nextInt(2) - rand.nextInt(2), rand.nextInt(3) - 1);
        }

        if (worldIn.func_175623_d(blockpos1) && this.func_180671_f(worldIn, blockpos1, this.func_176223_P()))
        {
            worldIn.func_180501_a(blockpos1, this.func_176223_P(), 2);
        }
    }

    private boolean canShrink(World worldIn, BlockPos pos)
    {
        for (BlockPos blockpos : BlockPos.func_177975_b(pos.func_177982_a(-4, -1, -4), pos.func_177982_a(4, 1, 4)))
        {
            if (worldIn.func_180495_p(blockpos).func_177230_c() == this)
            {
                return true;
            }
        }
        return false;
    }

    private void shrink(World worldIn, BlockPos pos)
    {
        worldIn.func_175698_g(pos);
    }
}
