/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.stone;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.api.types.Ore;
import net.dries007.tfc.api.types.Rock;
import net.dries007.tfc.objects.items.metal.ItemOreTFC;

@ParametersAreNonnullByDefault
public class BlockOreTFC extends Block
{
    public static final PropertyEnum<Ore.Grade> GRADE = PropertyEnum.func_177709_a("grade", Ore.Grade.class);
    private static final Map<Ore, Map<Rock, BlockOreTFC>> TABLE = new HashMap<>();

    public static BlockOreTFC get(Ore ore, Rock rock)
    {
        return TABLE.get(ore).get(rock);
    }

    public static IBlockState get(Ore ore, Rock rock, Ore.Grade grade)
    {
        IBlockState state = TABLE.get(ore).get(rock).func_176223_P();
        if (!ore.isGraded()) return state;
        return state.func_177226_a(GRADE, grade);
    }

    public final Ore ore;
    public final Rock rock;

    public BlockOreTFC(Ore ore, Rock rock)
    {
        super(Rock.Type.RAW.material);

        if (!TABLE.containsKey(ore))
            TABLE.put(ore, new HashMap<>());
        TABLE.get(ore).put(rock, this);

        this.ore = ore;
        this.rock = rock;
        func_180632_j(field_176227_L.func_177621_b().func_177226_a(GRADE, Ore.Grade.NORMAL));
        func_149672_a(SoundType.field_185851_d);
        func_149711_c(2.0F).func_149752_b(10.0F);
        setHarvestLevel("pickaxe", 0);
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public IBlockState func_176203_a(int meta)
    {
        return func_176223_P().func_177226_a(GRADE, Ore.Grade.byMetadata(meta));
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(GRADE).getMeta();
    }

    @Override
    @Nonnull
    public Item func_180660_a(IBlockState state, Random rand, int fortune)
    {
        // todo: handle coal
        // todo: handle kimberlite (diamond)
        // todo: handle saltpeter
        return ItemOreTFC.get(ore);
    }

    @Override
    public int func_180651_a(IBlockState state)
    {
        return func_176201_c(state);
    }

    @Override
    @Nonnull
    @SideOnly(Side.CLIENT)
    public BlockRenderLayer func_180664_k()
    {
        return BlockRenderLayer.CUTOUT;
    }

    @Override
    @Nonnull
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, GRADE);
    }

    @Override
    public int quantityDropped(IBlockState state, int fortune, Random random)
    {
        return super.quantityDropped(state, fortune, random); // todo: see how 1710 handles this
    }

    @Override
    @Nonnull
    public ItemStack getPickBlock(IBlockState state, RayTraceResult target, World world, BlockPos pos, EntityPlayer player)
    {
        return new ItemStack(state.func_177230_c());
    }
}
