/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.container;

import javax.annotation.Nullable;

import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.text.TextComponentString;
import net.minecraft.util.text.TextComponentTranslation;
import net.minecraft.util.text.TextFormatting;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandler;

import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.capability.forge.CapabilityForgeable;
import net.dries007.tfc.api.capability.forge.IForgeable;
import net.dries007.tfc.api.recipes.anvil.AnvilRecipe;
import net.dries007.tfc.api.registries.TFCRegistries;
import net.dries007.tfc.client.TFCGuiHandler;
import net.dries007.tfc.objects.inventory.slot.SlotCallback;
import net.dries007.tfc.objects.te.TEAnvilTFC;
import net.dries007.tfc.util.OreDictionaryHelper;
import net.dries007.tfc.util.forge.ForgeStep;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;
import static net.dries007.tfc.client.gui.GuiAnvilTFC.*;
import static net.dries007.tfc.objects.te.TEAnvilTFC.*;

public class ContainerAnvilTFC extends ContainerTE<TEAnvilTFC> implements IButtonHandler
{
    private static final int[] SLOT_SHIFT_ORDER = {SLOT_FLUX, SLOT_HAMMER, SLOT_INPUT_1, SLOT_INPUT_2};

    public ContainerAnvilTFC(InventoryPlayer playerInv, TEAnvilTFC te)
    {
        super(playerInv, te, true, 26);
    }

    @Override
    public void onButtonPress(int buttonID, @Nullable NBTTagCompound extraNBT)
    {
        if (buttonID >= BUTTON_ID_STEP_MIN && buttonID <= BUTTON_ID_STEP_MAX)
        {
            // Add a step to the anvil
            if (attemptWork())
            {
                tile.addStep(ForgeStep.valueOf(buttonID - BUTTON_ID_STEP_MIN));
            }
        }
        else if (buttonID == BUTTON_ID_PLAN)
        {
            // Switch to anvil plan gui
            Slot slotInput = field_75151_b.get(SLOT_INPUT_1);
            if (slotInput != null)
            {
                ItemStack stack = slotInput.func_75211_c();
                if (!stack.func_190926_b() && !AnvilRecipe.getAllFor(stack).isEmpty())
                {
                    TFCGuiHandler.openGui(player.field_70170_p, tile.func_174877_v(), player, TFCGuiHandler.Type.ANVIL_PLAN);
                }
            }
        }
    }

    @Override
    protected void addContainerSlots()
    {
        IItemHandler inventory = tile.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null);
        if (inventory != null)
        {
            func_75146_a(new SlotCallback(inventory, SLOT_INPUT_1, 31, 68, tile));
            func_75146_a(new SlotCallback(inventory, SLOT_INPUT_2, 13, 68, tile));
            func_75146_a(new SlotCallback(inventory, SLOT_HAMMER, 129, 68, tile));
            func_75146_a(new SlotCallback(inventory, SLOT_FLUX, 147, 68, tile));
        }
    }

    @Override
    protected int[] getSlotShiftOrder(int containerSlots)
    {
        return SLOT_SHIFT_ORDER;
    }

    private boolean attemptWork()
    {
        // This only runs on server

        // Get the slot for input
        Slot slotInput = field_75151_b.get(SLOT_INPUT_1);
        if (slotInput == null)
            return false;

        ItemStack stack = slotInput.func_75211_c();
        IForgeable cap = stack.getCapability(CapabilityForgeable.FORGEABLE_CAPABILITY, null);

        // The input must have the forge item capability
        if (cap == null)
            return false;

        // A recipe must exist
        AnvilRecipe recipe = TFCRegistries.ANVIL.getValue(cap.getRecipeName());
        if (recipe == null)
        {
            return false;
        }
        if (!tile.getTier().isAtLeast(recipe.getTier()))
        {
            TerraFirmaCraft.getLog().info("Anvil Tier: {} + Recipe Tier: {}", tile.getTier(), recipe.getTier());
            player.func_145747_a(new TextComponentTranslation(MOD_ID + ".tooltip.anvil_tier_too_low"));
            return false;
        }

        if (!cap.isWorkable())
        {
            player.func_145747_a(new TextComponentTranslation(MOD_ID + ".tooltip.anvil_too_cold"));
            return false;
        }

        Slot slot = field_75151_b.get(SLOT_HAMMER);
        if (slot == null)
            return false;

        stack = slot.func_75211_c();
        if (!stack.func_190926_b())
        {
            stack.func_77972_a(1, player);
            if (stack.func_190916_E() <= 0)
            {
                slot.func_75215_d(ItemStack.field_190927_a);
            }
            else
            {
                slot.func_75215_d(stack);
            }
            return true;
        }
        else
        {
            // Fallback to the held item if it is a hammer
            stack = player.field_71071_by.field_70462_a.get(player.field_71071_by.field_70461_c);
            if (!stack.func_190926_b() && OreDictionaryHelper.doesStackMatchOre(stack, "hammer"))
            {
                stack.func_77972_a(1, player);
                return true;
            }
            else
            {
                player.func_145747_a(new TextComponentString("" + TextFormatting.RED).func_150257_a(new TextComponentTranslation(MOD_ID + ".tooltip.anvil_no_hammer")));
                return false;
            }
        }
    }

}
