/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.entity.animal;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.BlockCarrot;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.IEntityLivingData;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.ai.*;
import net.minecraft.entity.monster.EntityMob;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.datasync.DataParameter;
import net.minecraft.network.datasync.DataSerializers;
import net.minecraft.network.datasync.EntityDataManager;
import net.minecraft.pathfinding.Path;
import net.minecraft.util.*;
import net.minecraft.util.datafix.DataFixer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.MathHelper;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.DifficultyInstance;
import net.minecraft.world.World;
import net.minecraft.world.biome.Biome;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.oredict.OreDictionary;

import net.dries007.tfc.Constants;
import net.dries007.tfc.objects.LootTablesTFC;
import net.dries007.tfc.util.OreDictionaryHelper;
import net.dries007.tfc.util.calendar.CalendarTFC;

import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC.Age;
import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC.Gender;

@ParametersAreNonnullByDefault
public class EntityRabbitTFC extends EntityAnimalMammal implements IAnimalTFC
{
    private static final int DAYS_TO_ADULTHOOD = 240;
    private static final int DAYS_TO_FULL_GESTATION = 30;

    private static final DataParameter<Integer> RABBIT_TYPE = EntityDataManager.func_187226_a(EntityRabbitTFC.class, DataSerializers.field_187192_b);

    public static void registerFixesRabbit(DataFixer fixer)
    {
        EntityLiving.func_189752_a(fixer, EntityRabbitTFC.class);
    }

    private static int getRandomGrowth()
    {
        int lifeTimeDays = Constants.RNG.nextInt(DAYS_TO_ADULTHOOD * 4);
        return (int) (CalendarTFC.PLAYER_TIME.getTotalDays() - lifeTimeDays);
    }

    private int jumpTicks;
    private int jumpDuration;
    private boolean wasOnGround;
    private int currentMoveTypeDuration;

    @SuppressWarnings("unused")
    public EntityRabbitTFC(World worldIn)
    {
        this(worldIn, Gender.fromBool(Constants.RNG.nextBoolean()), getRandomGrowth());
    }

    public EntityRabbitTFC(World worldIn, Gender gender, int birthDay)
    {
        super(worldIn, gender, birthDay);
        this.func_70105_a(0.4F, 0.5F);
        this.field_70767_i = new EntityRabbitTFC.RabbitJumpHelper(this);
        this.field_70765_h = new EntityRabbitTFC.RabbitMoveHelper(this);
        this.setMovementSpeed(0.0D);
    }

    @Override
    public boolean isValidSpawnConditions(Biome biome, float temperature, float rainfall)
    {
        return temperature > -10 && rainfall > 150;
    }

    @Override
    public void func_70619_bc()
    {
        super.func_70619_bc();

        if (this.currentMoveTypeDuration > 0)
        {
            --this.currentMoveTypeDuration;
        }

        if (this.field_70122_E)
        {
            if (!this.wasOnGround)
            {
                this.func_70637_d(false);
                this.checkLandingDelay();
            }

            EntityRabbitTFC.RabbitJumpHelper entityrabbit$rabbitjumphelper = (EntityRabbitTFC.RabbitJumpHelper) this.field_70767_i;

            if (!entityrabbit$rabbitjumphelper.getIsJumping())
            {
                if (this.field_70765_h.func_75640_a() && this.currentMoveTypeDuration == 0)
                {
                    Path path = this.field_70699_by.func_75505_d();
                    Vec3d vec3d = new Vec3d(this.field_70765_h.func_179917_d(), this.field_70765_h.func_179919_e(), this.field_70765_h.func_179918_f());

                    if (path != null && path.func_75873_e() < path.func_75874_d())
                    {
                        vec3d = path.func_75878_a(this);
                    }

                    this.calculateRotationYaw(vec3d.field_72450_a, vec3d.field_72449_c);
                    this.startJumping();
                }
            }
            else if (!entityrabbit$rabbitjumphelper.canJump())
            {
                this.enableJumpControl();
            }
        }

        this.wasOnGround = this.field_70122_E;
    }

    @SideOnly(Side.CLIENT)
    public void func_70103_a(byte id)
    {
        if (id == 1)
        {
            this.func_174808_Z();
            this.jumpDuration = 10;
            this.jumpTicks = 0;
        }
        else
        {
            super.func_70103_a(id);
        }
    }

    public int getRabbitType()
    {
        return this.field_70180_af.func_187225_a(RABBIT_TYPE).intValue();
    }

    public void setRabbitType(int rabbitTypeId)
    {
        this.field_70180_af.func_187227_b(RABBIT_TYPE, Integer.valueOf(rabbitTypeId));
    }

    @SideOnly(Side.CLIENT)
    public float getJumpCompletion(float p_175521_1_)
    {
        return this.jumpDuration == 0 ? 0.0F : ((float) this.jumpTicks + p_175521_1_) / (float) this.jumpDuration;
    }

    public void setMovementSpeed(double newSpeed)
    {
        this.func_70661_as().func_75489_a(newSpeed);
        this.field_70765_h.func_75642_a(this.field_70765_h.func_179917_d(), this.field_70765_h.func_179919_e(), this.field_70765_h.func_179918_f(), newSpeed);
    }

    public void startJumping()
    {
        this.func_70637_d(true);
        this.jumpDuration = 10;
        this.jumpTicks = 0;
    }

    public void func_70636_d()
    {
        super.func_70636_d();

        if (this.jumpTicks != this.jumpDuration)
        {
            ++this.jumpTicks;
        }
        else if (this.jumpDuration != 0)
        {
            this.jumpTicks = 0;
            this.jumpDuration = 0;
            this.func_70637_d(false);
        }
    }

    public void func_70014_b(NBTTagCompound compound)
    {
        super.func_70014_b(compound);
        compound.func_74768_a("RabbitType", this.getRabbitType());
    }

    public void func_70037_a(NBTTagCompound compound)
    {
        super.func_70037_a(compound);
        this.setRabbitType(compound.func_74762_e("RabbitType"));
    }

    @Override
    public void birthChildren()
    {
        int numberOfChilds = 5 + field_70146_Z.nextInt(5); // 5-10
        for (int i = 0; i < numberOfChilds; i++)
        {
            EntityRabbitTFC baby = new EntityRabbitTFC(this.field_70170_p, Gender.fromBool(Constants.RNG.nextBoolean()), (int) CalendarTFC.PLAYER_TIME.getTotalDays());
            baby.func_70012_b(this.field_70165_t, this.field_70163_u, this.field_70161_v, 0.0F, 0.0F);
            this.field_70170_p.func_72838_d(baby);
        }
    }

    @Override
    public long gestationDays()
    {
        return DAYS_TO_FULL_GESTATION;
    }

    public SoundCategory func_184176_by()
    {
        return SoundCategory.NEUTRAL;
    }

    @Override
    public boolean func_70877_b(ItemStack stack)
    {
        return OreDictionaryHelper.doesStackMatchOre(stack, "carrot");
    }

    @Override
    protected void func_70088_a()
    {
        super.func_70088_a();
        this.field_70180_af.func_187214_a(RABBIT_TYPE, Integer.valueOf(0));
    }

    @Override
    public float getPercentToAdulthood()
    {
        if (this.getAge() != Age.CHILD)
            return 1;
        double value = (CalendarTFC.PLAYER_TIME.getTotalDays() - this.getBirthDay()) / (double) DAYS_TO_ADULTHOOD;
        if (value > 1f)
            value = 1f;
        if (value < 0f)
            value = 0;
        return (float) value;
    }

    @Override
    public Age getAge()
    {
        return CalendarTFC.PLAYER_TIME.getTotalDays() >= this.getBirthDay() + DAYS_TO_ADULTHOOD ? Age.ADULT : Age.CHILD;
    }

    @Override
    @SuppressWarnings("unchecked")
    protected void func_184651_r()
    {
        this.field_70714_bg.func_75776_a(1, new EntityAISwimming(this));
        this.field_70714_bg.func_75776_a(1, new EntityRabbitTFC.AIPanic(this, 2.2D));
        this.field_70714_bg.func_75776_a(2, new EntityAIMate(this, 1.2D));
        for (ItemStack is : OreDictionary.getOres("carrot"))
        {
            Item item = is.func_77973_b();
            this.field_70714_bg.func_75776_a(3, new EntityAITempt(this, 1.4D, item, false));
        }
        this.field_70714_bg.func_75776_a(4, new EntityAIAvoidEntity(this, EntityPlayer.class, 8.0F, 2.0D, 2.0D));
        this.field_70714_bg.func_75776_a(4, new EntityAIAvoidEntity(this, EntityMob.class, 4.0F, 2.0D, 2.0D));
        this.field_70714_bg.func_75776_a(6, new EntityAIWanderAvoidWater(this, 0.6D));
        this.field_70714_bg.func_75776_a(11, new EntityAIWatchClosest(this, EntityPlayer.class, 10.0F));
    }

    @Override
    protected void func_110147_ax()
    {
        super.func_110147_ax();
        this.func_110148_a(SharedMonsterAttributes.field_111267_a).func_111128_a(3.0D);
        this.func_110148_a(SharedMonsterAttributes.field_111263_d).func_111128_a(0.30000001192092896D);
    }

    @Override
    protected SoundEvent func_184639_G()
    {
        return SoundEvents.field_187816_ej;
    }

    @Nullable
    protected ResourceLocation func_184647_J()
    {
        return LootTablesTFC.ANIMALS_RABBIT;
    }

    @Nullable
    public IEntityLivingData func_180482_a(DifficultyInstance difficulty, @Nullable IEntityLivingData livingdata)
    {
        livingdata = super.func_180482_a(difficulty, livingdata);
        int i = this.getRandomRabbitType();

        if (livingdata instanceof EntityRabbitTFC.RabbitTypeData)
        {
            i = ((EntityRabbitTFC.RabbitTypeData) livingdata).typeData;
        }
        else
        {
            livingdata = new EntityRabbitTFC.RabbitTypeData(i);
        }

        this.setRabbitType(i);

        return livingdata;
    }

    protected SoundEvent getJumpSound()
    {
        return SoundEvents.field_187824_en;
    }

    @Override
    protected SoundEvent func_184601_bQ(DamageSource damageSourceIn)
    {
        return SoundEvents.field_187822_em;
    }

    @Override
    protected SoundEvent func_184615_bR()
    {
        return SoundEvents.field_187820_el;
    }

    protected float func_175134_bD()
    {
        if (!this.field_70123_F && (!this.field_70765_h.func_75640_a() || this.field_70765_h.func_179919_e() <= this.field_70163_u + 0.5D))
        {
            Path path = this.field_70699_by.func_75505_d();

            if (path != null && path.func_75873_e() < path.func_75874_d())
            {
                Vec3d vec3d = path.func_75878_a(this);

                if (vec3d.field_72448_b > this.field_70163_u + 0.5D)
                {
                    return 0.5F;
                }
            }

            return this.field_70765_h.func_75638_b() <= 0.6D ? 0.2F : 0.3F;
        }
        else
        {
            return 0.5F;
        }
    }

    protected void func_70664_aZ()
    {
        super.func_70664_aZ();
        double d0 = this.field_70765_h.func_75638_b();

        if (d0 > 0.0D)
        {
            double d1 = this.field_70159_w * this.field_70159_w + this.field_70179_y * this.field_70179_y;

            if (d1 < 0.010000000000000002D)
            {
                this.func_191958_b(0.0F, 0.0F, 1.0F, 0.1F);
            }
        }

        if (!this.field_70170_p.field_72995_K)
        {
            this.field_70170_p.func_72960_a(this, (byte) 1);
        }
    }

    public void func_70637_d(boolean jumping)
    {
        super.func_70637_d(jumping);

        if (jumping)
        {
            this.func_184185_a(this.getJumpSound(), this.func_70599_aP(), ((this.field_70146_Z.nextFloat() - this.field_70146_Z.nextFloat()) * 0.2F + 1.0F) * 0.8F);
        }
    }

    protected void createEatingParticles()
    {
        BlockCarrot blockcarrot = (BlockCarrot) Blocks.field_150459_bM;
        IBlockState iblockstate = blockcarrot.func_185528_e(blockcarrot.func_185526_g());
        this.field_70170_p.func_175688_a(EnumParticleTypes.BLOCK_DUST,
                this.field_70165_t + (double) (this.field_70146_Z.nextFloat() * this.field_70130_N * 2.0F) - (double) this.field_70130_N,
                this.field_70163_u + 0.5D + (double) (this.field_70146_Z.nextFloat() * this.field_70131_O),
                this.field_70161_v + (double) (this.field_70146_Z.nextFloat() * this.field_70130_N * 2.0F) - (double) this.field_70130_N, 0.0D, 0.0D,
                0.0D, Block.func_176210_f(iblockstate));
    }

    private void calculateRotationYaw(double x, double z)
    {
        this.field_70177_z = (float) (MathHelper.func_181159_b(z - this.field_70161_v, x - this.field_70165_t) * (180D / Math.PI)) - 90.0F;
    }

    private void enableJumpControl()
    {
        ((EntityRabbitTFC.RabbitJumpHelper) this.field_70767_i).setCanJump(true);
    }

    private void disableJumpControl()
    {
        ((EntityRabbitTFC.RabbitJumpHelper) this.field_70767_i).setCanJump(false);
    }

    private void updateMoveTypeDuration()
    {
        if (this.field_70765_h.func_75638_b() < 2.2D)
        {
            this.currentMoveTypeDuration = 10;
        }
        else
        {
            this.currentMoveTypeDuration = 1;
        }
    }

    private void checkLandingDelay()
    {
        this.updateMoveTypeDuration();
        this.disableJumpControl();
    }

    private int getRandomRabbitType()
    {
        Biome biome = this.field_70170_p.func_180494_b(new BlockPos(this));
        int i = this.field_70146_Z.nextInt(100);
        if (biome.func_150559_j())
        {
            return i < 5 ? 7 : (i < 10 ? 6 : (i < 80 ? 1 : 3));
        }
        else
        {
            return i < 50 ? 0 : (i < 90 ? 5 : (i < 95 ? 2 : 4));
        }
    }

    static class RabbitMoveHelper extends EntityMoveHelper
    {
        private final EntityRabbitTFC rabbit;
        private double nextJumpSpeed;

        public RabbitMoveHelper(EntityRabbitTFC rabbit)
        {
            super(rabbit);
            this.rabbit = rabbit;
        }

        public void func_75642_a(double x, double y, double z, double speedIn)
        {
            if (this.rabbit.func_70090_H())
            {
                speedIn = 1.5D;
            }

            super.func_75642_a(x, y, z, speedIn);

            if (speedIn > 0.0D)
            {
                this.nextJumpSpeed = speedIn;
            }
        }

        public void func_75641_c()
        {
            if (this.rabbit.field_70122_E && !this.rabbit.field_70703_bu && !((EntityRabbitTFC.RabbitJumpHelper) this.rabbit.field_70767_i).getIsJumping())
            {
                this.rabbit.setMovementSpeed(0.0D);
            }
            else if (this.func_75640_a())
            {
                this.rabbit.setMovementSpeed(this.nextJumpSpeed);
            }

            super.func_75641_c();
        }
    }

    public static class RabbitTypeData implements IEntityLivingData
    {
        public int typeData;

        public RabbitTypeData(int type)
        {
            this.typeData = type;
        }
    }

    static class AIPanic extends EntityAIPanic
    {
        private final EntityRabbitTFC rabbit;

        public AIPanic(EntityRabbitTFC rabbit, double speedIn)
        {
            super(rabbit, speedIn);
            this.rabbit = rabbit;
        }

        public void func_75246_d()
        {
            super.func_75246_d();
            this.rabbit.setMovementSpeed(this.field_75265_b);
        }
    }

    public class RabbitJumpHelper extends EntityJumpHelper
    {
        private final EntityRabbitTFC rabbit;
        private boolean canJump;

        public RabbitJumpHelper(EntityRabbitTFC rabbit)
        {
            super(rabbit);
            this.rabbit = rabbit;
        }

        public boolean getIsJumping()
        {
            return this.field_75662_b;
        }

        public boolean canJump()
        {
            return this.canJump;
        }

        public void setCanJump(boolean canJumpIn)
        {
            this.canJump = canJumpIn;
        }

        public void func_75661_b()
        {
            if (this.field_75662_b)
            {
                this.rabbit.startJumping();
                this.field_75662_b = false;
            }
        }
    }
}
