package net.dries007.tfc.objects.entity.animal;

import java.util.UUID;
import javax.annotation.Nullable;

import com.google.common.base.Optional;
import net.minecraft.advancements.CriteriaTriggers;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.datasync.DataParameter;
import net.minecraft.network.datasync.DataSerializers;
import net.minecraft.network.datasync.EntityDataManager;
import net.minecraft.scoreboard.Team;
import net.minecraft.server.management.PreYggdrasilConverter;
import net.minecraft.util.DamageSource;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.objects.entity.IEntityOwnableTFC;
import net.dries007.tfc.objects.entity.ai.EntityAISitTFC;

import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC.Gender;

public abstract class EntityTameableTFC extends EntityAnimalMammal implements IEntityOwnableTFC
{
    protected static final DataParameter<Byte> TAMED = EntityDataManager.func_187226_a(EntityTameableTFC.class, DataSerializers.field_187191_a);
    protected static final DataParameter<Optional<UUID>> OWNER_UNIQUE_ID = EntityDataManager.func_187226_a(EntityTameableTFC.class, DataSerializers.field_187203_m);
    protected EntityAISitTFC aiSit;

    public EntityTameableTFC(World worldIn, Gender gender, int birthDay)
    {
        super(worldIn, gender, birthDay);
        this.setupTamedAI();
    }

    public void func_70014_b(NBTTagCompound compound)
    {
        super.func_70014_b(compound);
        if (this.getOwnerId() == null)
        {
            compound.func_74778_a("OwnerUUID", "");
        }
        else
        {
            compound.func_74778_a("OwnerUUID", this.getOwnerId().toString());
        }

        compound.func_74757_a("Sitting", this.isSitting());
    }

    public void func_70037_a(NBTTagCompound compound)
    {
        super.func_70037_a(compound);
        String s;
        if (compound.func_150297_b("OwnerUUID", 8))
        {
            s = compound.func_74779_i("OwnerUUID");
        }
        else
        {
            String s1 = compound.func_74779_i("Owner");
            s = PreYggdrasilConverter.func_187473_a(this.func_184102_h(), s1);
        }
        if (!s.isEmpty())
        {
            try
            {
                this.setOwnerId(UUID.fromString(s));
                this.setTamed(true);
            }
            catch (Throwable var4)
            {
                this.setTamed(false);
            }
        }
        if (this.aiSit != null)
        {
            this.aiSit.setSitting(compound.func_74767_n("Sitting"));
        }
        this.setSitting(compound.func_74767_n("Sitting"));
    }

    public boolean func_184652_a(EntityPlayer player)
    {
        return !this.func_110167_bD();
    }

    @SideOnly(Side.CLIENT)
    public void func_70103_a(byte id)
    {
        if (id == 7)
        {
            this.playTameEffect(true);
        }
        else if (id == 6)
        {
            this.playTameEffect(false);
        }
        else
        {
            super.func_70103_a(id);
        }
    }

    public boolean isTamed()
    {
        return (this.field_70180_af.func_187225_a(TAMED) & 4) != 0;
    }

    public void setTamed(boolean tamed)
    {
        byte b0 = this.field_70180_af.func_187225_a(TAMED);
        if (tamed)
        {
            this.field_70180_af.func_187227_b(TAMED, (byte) (b0 | 4));
        }
        else
        {
            this.field_70180_af.func_187227_b(TAMED, (byte) (b0 & -5));
        }
        this.setupTamedAI();
    }

    public boolean isSitting()
    {
        return (this.field_70180_af.func_187225_a(TAMED) & 1) != 0;
    }

    public void setSitting(boolean sitting)
    {
        byte b0 = this.field_70180_af.func_187225_a(TAMED);
        if (sitting)
        {
            this.field_70180_af.func_187227_b(TAMED, (byte) (b0 | 1));
        }
        else
        {
            this.field_70180_af.func_187227_b(TAMED, (byte) (b0 & -2));
        }

    }

    @Nullable
    public UUID getOwnerId()
    {
        return (UUID) ((Optional) this.field_70180_af.func_187225_a(OWNER_UNIQUE_ID)).orNull();
    }

    public void setOwnerId(@Nullable UUID p_184754_1_)
    {
        this.field_70180_af.func_187227_b(OWNER_UNIQUE_ID, Optional.fromNullable(p_184754_1_));
    }

    @Nullable
    public EntityLivingBase getOwner()
    {
        try
        {
            UUID uuid = this.getOwnerId();
            return uuid == null ? null : this.field_70170_p.func_152378_a(uuid);
        }
        catch (IllegalArgumentException var2)
        {
            return null;
        }
    }

    public void setTamedBy(EntityPlayer player)
    {
        this.setTamed(true);
        this.setOwnerId(player.func_110124_au());
        if (player instanceof EntityPlayerMP)
        {
            CriteriaTriggers.field_193136_w.func_193178_a((EntityPlayerMP) player, this);
        }
    }

    public boolean isOwner(EntityLivingBase entityIn)
    {
        return entityIn == this.getOwner();
    }

    public EntityAISitTFC getAISit()
    {
        return this.aiSit;
    }

    public boolean shouldAttackEntity(EntityLivingBase target, EntityLivingBase owner)
    {
        return true;
    }

    public Team func_96124_cp()
    {
        if (this.isTamed())
        {
            EntityLivingBase entitylivingbase = this.getOwner();
            if (entitylivingbase != null)
            {
                return entitylivingbase.func_96124_cp();
            }
        }
        return super.func_96124_cp();
    }

    public boolean func_184191_r(Entity entityIn)
    {
        if (this.isTamed())
        {
            EntityLivingBase entitylivingbase = this.getOwner();
            if (entityIn == entitylivingbase)
            {
                return true;
            }
            if (entitylivingbase != null)
            {
                return entitylivingbase.func_184191_r(entityIn);
            }
        }
        return super.func_184191_r(entityIn);
    }

    public void func_70645_a(DamageSource cause)
    {
        if (!this.field_70170_p.field_72995_K && this.field_70170_p.func_82736_K().func_82766_b("showDeathMessages") && this.getOwner() instanceof EntityPlayerMP)
        {
            this.getOwner().func_145747_a(this.func_110142_aN().func_151521_b());
        }
        super.func_70645_a(cause);
    }

    protected void func_70088_a()
    {
        super.func_70088_a();
        this.field_70180_af.func_187214_a(TAMED, (byte) 0);
        this.field_70180_af.func_187214_a(OWNER_UNIQUE_ID, Optional.absent());
    }

    protected void playTameEffect(boolean play)
    {
        EnumParticleTypes enumparticletypes = EnumParticleTypes.HEART;
        if (!play)
        {
            enumparticletypes = EnumParticleTypes.SMOKE_NORMAL;
        }
        for (int i = 0; i < 7; ++i)
        {
            double d0 = this.field_70146_Z.nextGaussian() * 0.02D;
            double d1 = this.field_70146_Z.nextGaussian() * 0.02D;
            double d2 = this.field_70146_Z.nextGaussian() * 0.02D;
            this.field_70170_p.func_175688_a(enumparticletypes, this.field_70165_t + (double) (this.field_70146_Z.nextFloat() * this.field_70130_N * 2.0F) - (double) this.field_70130_N, this.field_70163_u + 0.5D + (double) (this.field_70146_Z.nextFloat() * this.field_70131_O), this.field_70161_v + (double) (this.field_70146_Z.nextFloat() * this.field_70130_N * 2.0F) - (double) this.field_70130_N, d0, d1, d2);
        }

    }

    protected void setupTamedAI() {}
}
