/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.entity.animal;

import java.util.UUID;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import com.google.common.base.Predicate;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.ai.*;
import net.minecraft.entity.monster.AbstractSkeleton;
import net.minecraft.entity.monster.EntityCreeper;
import net.minecraft.entity.monster.EntityGhast;
import net.minecraft.entity.passive.AbstractHorse;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.projectile.EntityArrow;
import net.minecraft.init.Items;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.EnumDyeColor;
import net.minecraft.item.ItemFood;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.datasync.DataParameter;
import net.minecraft.network.datasync.DataSerializers;
import net.minecraft.network.datasync.EntityDataManager;
import net.minecraft.util.*;
import net.minecraft.util.datafix.DataFixer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.MathHelper;
import net.minecraft.world.World;
import net.minecraft.world.biome.Biome;
import net.minecraftforge.event.ForgeEventFactory;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import jdk.nashorn.internal.ir.Block;
import net.dries007.tfc.Constants;
import net.dries007.tfc.objects.LootTablesTFC;
import net.dries007.tfc.objects.entity.ai.*;
import net.dries007.tfc.objects.items.food.ItemFoodTFC;
import net.dries007.tfc.util.calendar.CalendarTFC;

import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC.Age;
import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC.Gender;

@ParametersAreNonnullByDefault
public class EntityWolfTFC extends EntityTameableTFC implements IAnimalTFC
{
    private static final int DAYS_TO_ADULTHOOD = 360;
    private static final int DAYS_TO_FULL_GESTATION = 70;

    private static final DataParameter<Float> DATA_HEALTH_ID = EntityDataManager.func_187226_a(EntityWolfTFC.class, DataSerializers.field_187193_c);
    private static final DataParameter<Boolean> BEGGING = EntityDataManager.func_187226_a(EntityWolfTFC.class, DataSerializers.field_187198_h);
    private static final DataParameter<Integer> COLLAR_COLOR = EntityDataManager.func_187226_a(EntityWolfTFC.class, DataSerializers.field_187192_b);

    public static void registerFixesWolf(DataFixer fixer)
    {
        EntityLiving.func_189752_a(fixer, EntityWolfTFC.class);
    }

    private static int getRandomGrowth()
    {
        int lifeTimeDays = Constants.RNG.nextInt(DAYS_TO_ADULTHOOD * 4);
        return (int) (CalendarTFC.PLAYER_TIME.getTotalDays() - lifeTimeDays);
    }
    private float headRotationCourse;
    private float headRotationCourseOld;
    private boolean isWet;
    private boolean isShaking;
    private float timeWolfIsShaking;
    private float prevTimeWolfIsShaking;

    @SuppressWarnings("unused")
    public EntityWolfTFC(World worldIn)
    {
        this(worldIn, Gender.fromBool(Constants.RNG.nextBoolean()), getRandomGrowth());
    }

    public EntityWolfTFC(World worldIn, Gender gender, int birthDay)
    {
        super(worldIn, gender, birthDay);
        this.func_70105_a(0.6F, 0.85F);
        this.setTamed(false);
    }

    @Override
    public boolean isValidSpawnConditions(Biome biome, float temperature, float rainfall)
    {
        return temperature > -20 && temperature < 20 && rainfall > 75;
    }

    public void func_70636_d()
    {
        super.func_70636_d();
        if (!this.field_70170_p.field_72995_K && this.isWet && !this.isShaking && !this.func_70781_l() && this.field_70122_E)
        {
            this.isShaking = true;
            this.timeWolfIsShaking = 0.0F;
            this.prevTimeWolfIsShaking = 0.0F;
            this.field_70170_p.func_72960_a(this, (byte) 8);
        }
        if (!this.field_70170_p.field_72995_K && this.func_70638_az() == null && this.isAngry())
        {
            this.setAngry(false);
        }
    }

    @Override
    public void birthChildren()
    {
        int numberOfChilds = 1 + field_70146_Z.nextInt(1); //1-2
        for (int i = 0; i < numberOfChilds; i++)
        {
            EntityWolfTFC baby = new EntityWolfTFC(this.field_70170_p, Gender.fromBool(Constants.RNG.nextBoolean()), (int) CalendarTFC.PLAYER_TIME.getTotalDays());
            baby.func_70012_b(this.field_70165_t, this.field_70163_u, this.field_70161_v, 0.0F, 0.0F);
            UUID uuid = this.getOwnerId();
            if (uuid != null)
            {
                baby.setOwnerId(uuid);
                baby.setTamed(true);
            }
            this.field_70170_p.func_72838_d(baby);
        }
    }

    @Override
    public long gestationDays()
    {
        return DAYS_TO_FULL_GESTATION;
    }

    @SideOnly(Side.CLIENT)
    public boolean isWolfWet()
    {
        return this.isWet;
    }

    @SideOnly(Side.CLIENT)
    public float getShadingWhileWet(float p_70915_1_)
    {
        return 0.75F + (this.prevTimeWolfIsShaking + (this.timeWolfIsShaking - this.prevTimeWolfIsShaking) * p_70915_1_) / 2.0F * 0.25F;
    }

    @SideOnly(Side.CLIENT)
    public float getShakeAngle(float p_70923_1_, float p_70923_2_)
    {
        float f = (this.prevTimeWolfIsShaking + (this.timeWolfIsShaking - this.prevTimeWolfIsShaking) * p_70923_1_ + p_70923_2_) / 1.8F;
        if (f < 0.0F)
        {
            f = 0.0F;
        }
        else if (f > 1.0F)
        {
            f = 1.0F;
        }
        return MathHelper.func_76126_a(f * 3.1415927F) * MathHelper.func_76126_a(f * 3.1415927F * 11.0F) * 0.15F * 3.1415927F;
    }

    @SideOnly(Side.CLIENT)
    public float getInterestedAngle(float p_70917_1_)
    {
        return (this.headRotationCourseOld + (this.headRotationCourse - this.headRotationCourseOld) * p_70917_1_) * 0.15F * 3.1415927F;
    }

    public float func_70047_e()
    {
        return this.field_70131_O * 0.8F;
    }

    @SideOnly(Side.CLIENT)
    public float getTailRotation()
    {
        if (this.isAngry())
        {
            return 1.5393804F;
        }
        else
        {
            return this.isTamed() ? (0.55F - (this.func_110138_aP() - this.field_70180_af.func_187225_a(DATA_HEALTH_ID)) * 0.02F) * 3.1415927F : 0.62831855F;
        }
    }

    public boolean func_70877_b(ItemStack stack)
    {
        return stack.func_77973_b() instanceof ItemFood && ((ItemFood) stack.func_77973_b()).func_77845_h();
    }

    public boolean func_184645_a(EntityPlayer player, EnumHand hand)
    {
        ItemStack itemstack = player.func_184586_b(hand);
        if (this.isTamed())
        {
            if (!itemstack.func_190926_b())
            {
                if (itemstack.func_77973_b() instanceof ItemFoodTFC)
                {
                    ItemFoodTFC itemfood = (ItemFoodTFC) itemstack.func_77973_b();
                    if (itemfood.func_77845_h() && this.field_70180_af.func_187225_a(DATA_HEALTH_ID) < 20.0F)
                    {
                        if (!player.field_71075_bZ.field_75098_d)
                        {
                            itemstack.func_190918_g(1);
                        }
                        this.func_70691_i((float) itemfood.func_150905_g(itemstack));
                        return true;
                    }
                }
                else if (itemstack.func_77973_b() == Items.field_151100_aR)
                {
                    EnumDyeColor enumdyecolor = EnumDyeColor.func_176766_a(itemstack.func_77960_j());
                    if (enumdyecolor != this.getCollarColor())
                    {
                        this.setCollarColor(enumdyecolor);
                        if (!player.field_71075_bZ.field_75098_d)
                        {
                            itemstack.func_190918_g(1);
                        }
                        return true;
                    }
                }
            }
            if (this.isOwner(player) && !this.field_70170_p.field_72995_K && !this.func_70877_b(itemstack))
            {
                this.aiSit.setSitting(!this.isSitting());
                this.field_70703_bu = false;
                this.field_70699_by.func_75499_g();
                this.func_70624_b(null);
            }
        }
        else if (itemstack.func_77973_b() == Items.field_151103_aS && !this.isAngry())
        {
            if (!player.field_71075_bZ.field_75098_d)
            {
                itemstack.func_190918_g(1);
            }
            if (!this.field_70170_p.field_72995_K)
            {
                if (getFamiliarity() >= 0.3f && this.field_70146_Z.nextInt(3) == 0 && !ForgeEventFactory.onAnimalTame(this, player))
                {
                    this.setTamedBy(player);
                    this.field_70699_by.func_75499_g();
                    this.func_70624_b(null);
                    this.aiSit.setSitting(true);
                    this.func_70606_j(20.0F);
                    this.playTameEffect(true);
                    this.field_70170_p.func_72960_a(this, (byte) 7);
                }
                else
                {
                    this.playTameEffect(false);
                    this.field_70170_p.func_72960_a(this, (byte) 6);
                }
            }
            return true;
        }
        return super.func_184645_a(player, hand);
    }

    @Override
    public float getPercentToAdulthood()
    {
        if (this.getAge() != Age.CHILD) return 1;
        double value = (CalendarTFC.PLAYER_TIME.getTotalDays() - this.getBirthDay()) / (double) DAYS_TO_ADULTHOOD;
        if (value > 1f) value = 1f;
        if (value < 0f) value = 0;
        return (float) value;
    }

    @Override
    public Age getAge()
    {
        return CalendarTFC.PLAYER_TIME.getTotalDays() >= this.getBirthDay() + DAYS_TO_ADULTHOOD ? Age.ADULT : Age.CHILD;
    }

    public boolean isAngry()
    {
        return (this.field_70180_af.func_187225_a(TAMED) & 2) != 0;
    }

    public void setAngry(boolean angry)
    {
        byte b0 = this.field_70180_af.func_187225_a(TAMED);
        if (angry)
        {
            this.field_70180_af.func_187227_b(TAMED, (byte) (b0 | 2));
        }
        else
        {
            this.field_70180_af.func_187227_b(TAMED, (byte) (b0 & -3));
        }
    }

    public EnumDyeColor getCollarColor()
    {
        return EnumDyeColor.func_176766_a(this.field_70180_af.func_187225_a(COLLAR_COLOR) & 15);
    }

    public void setCollarColor(EnumDyeColor collarcolor)
    {
        this.field_70180_af.func_187227_b(COLLAR_COLOR, collarcolor.func_176767_b());
    }

    public boolean isBegging()
    {
        return this.field_70180_af.func_187225_a(BEGGING);
    }

    public void setBegging(boolean beg)
    {
        this.field_70180_af.func_187227_b(BEGGING, beg);
    }

    @SuppressWarnings("unchecked")
    protected void func_184651_r()
    {
        this.aiSit = new EntityAISitTFC(this);
        this.field_70714_bg.func_75776_a(1, new EntityAISwimming(this));
        this.field_70714_bg.func_75776_a(2, this.aiSit);
        this.field_70714_bg.func_75776_a(3, new EntityAIMate(this, 1.0D));
        this.field_70714_bg.func_75776_a(4, new EntityAILeapAtTarget(this, 0.4F));
        this.field_70714_bg.func_75776_a(5, new EntityAIAttackMelee(this, 1.0D, true));
        this.field_70714_bg.func_75776_a(6, new EntityAIFollowOwnerTFC(this, 1.0D, 10.0F, 2.0F));
        this.field_70714_bg.func_75776_a(8, new EntityAIWanderAvoidWater(this, 1.0D));
        this.field_70714_bg.func_75776_a(9, new EntityAIBegTFC(this, 8.0F));
        this.field_70714_bg.func_75776_a(10, new EntityAIWatchClosest(this, EntityPlayer.class, 8.0F));
        this.field_70714_bg.func_75776_a(10, new EntityAILookIdle(this));
        this.field_70715_bh.func_75776_a(1, new EntityAIOwnerHurtByTargetTFC(this));
        this.field_70715_bh.func_75776_a(2, new EntityAIOwnerHurtTargetTFC(this));
        this.field_70715_bh.func_75776_a(3, new EntityAIHurtByTarget(this, true));
        this.field_70715_bh.func_75776_a(4, new EntityAITargetNonTamedTFC(this, EntityAnimalTFC.class, false, new Predicate<Entity>()
        {
            public boolean apply(@Nullable Entity entity)
            {
                return entity instanceof EntitySheepTFC || entity instanceof EntityPigTFC; // TODO: add RabbitsTFC
            }
        }));
        this.field_70715_bh.func_75776_a(5, new EntityAINearestAttackableTarget(this, AbstractSkeleton.class, false));
    }

    protected void func_110147_ax()
    {
        super.func_110147_ax();
        this.func_110148_a(SharedMonsterAttributes.field_111263_d).func_111128_a(0.30000001192092896D);
        if (this.isTamed())
        {
            this.func_110148_a(SharedMonsterAttributes.field_111267_a).func_111128_a(20.0D);
        }
        else
        {
            this.func_110148_a(SharedMonsterAttributes.field_111267_a).func_111128_a(8.0D);
        }

        this.func_110140_aT().func_111150_b(SharedMonsterAttributes.field_111264_e).func_111128_a(2.0D);
    }

    public void func_70624_b(@Nullable EntityLivingBase entitylivingbaseIn)
    {
        super.func_70624_b(entitylivingbaseIn);
        if (entitylivingbaseIn == null)
        {
            this.setAngry(false);
        }
        else if (!this.isTamed())
        {
            this.setAngry(true);
        }

    }

    public void func_70071_h_()
    {
        super.func_70071_h_();
        this.headRotationCourseOld = this.headRotationCourse;
        if (this.isBegging())
        {
            this.headRotationCourse += (1.0F - this.headRotationCourse) * 0.4F;
        }
        else
        {
            this.headRotationCourse += (0.0F - this.headRotationCourse) * 0.4F;
        }

        if (this.func_70026_G())
        {
            this.isWet = true;
            this.isShaking = false;
            this.timeWolfIsShaking = 0.0F;
            this.prevTimeWolfIsShaking = 0.0F;
        }
        else if ((this.isWet || this.isShaking) && this.isShaking)
        {
            if (this.timeWolfIsShaking == 0.0F)
            {
                this.func_184185_a(SoundEvents.field_187867_gJ, this.func_70599_aP(), (this.field_70146_Z.nextFloat() - this.field_70146_Z.nextFloat()) * 0.2F + 1.0F);
            }

            this.prevTimeWolfIsShaking = this.timeWolfIsShaking;
            this.timeWolfIsShaking += 0.05F;
            if (this.prevTimeWolfIsShaking >= 2.0F)
            {
                this.isWet = false;
                this.isShaking = false;
                this.prevTimeWolfIsShaking = 0.0F;
                this.timeWolfIsShaking = 0.0F;
            }

            if (this.timeWolfIsShaking > 0.4F)
            {
                float f = (float) this.func_174813_aQ().field_72338_b;
                int i = (int) (MathHelper.func_76126_a((this.timeWolfIsShaking - 0.4F) * 3.1415927F) * 7.0F);
                for (int j = 0; j < i; ++j)
                {
                    float f1 = (this.field_70146_Z.nextFloat() * 2.0F - 1.0F) * this.field_70130_N * 0.5F;
                    float f2 = (this.field_70146_Z.nextFloat() * 2.0F - 1.0F) * this.field_70130_N * 0.5F;
                    this.field_70170_p.func_175688_a(EnumParticleTypes.WATER_SPLASH, this.field_70165_t + (double) f1, f + 0.8F, this.field_70161_v + (double) f2, this.field_70159_w, this.field_70181_x, this.field_70179_y);
                }
            }
        }

    }

    protected SoundEvent func_184639_G()
    {
        if (this.isAngry())
        {
            return SoundEvents.field_187861_gG;
        }
        else if (this.field_70146_Z.nextInt(3) != 0)
        {
            return SoundEvents.field_187857_gE;
        }
        else
        {
            return this.isTamed() && this.field_70180_af.func_187225_a(DATA_HEALTH_ID) < 10.0F ? SoundEvents.field_187871_gL : SoundEvents.field_187865_gI;
        }
    }

    @Nullable
    protected ResourceLocation func_184647_J()
    {
        return LootTablesTFC.ANIMALS_WOLF;
    }

    public int func_70646_bf()
    {
        return this.isSitting() ? 20 : super.func_70646_bf();
    }

    public int func_70641_bl()
    {
        return 8;
    }

    protected void func_70619_bc()
    {
        this.field_70180_af.func_187227_b(DATA_HEALTH_ID, this.func_110143_aJ());
    }

    public boolean func_70097_a(DamageSource source, float amount)
    {
        if (this.func_180431_b(source))
        {
            return false;
        }
        else
        {
            Entity entity = source.func_76346_g();
            if (this.aiSit != null)
            {
                this.aiSit.setSitting(false);
            }
            if (entity != null && !(entity instanceof EntityPlayer) && !(entity instanceof EntityArrow))
            {
                amount = (amount + 1.0F) / 2.0F;
            }
            return super.func_70097_a(source, amount);
        }
    }

    protected void func_70088_a()
    {
        super.func_70088_a();
        this.field_70180_af.func_187214_a(DATA_HEALTH_ID, this.func_110143_aJ());
        this.field_70180_af.func_187214_a(BEGGING, false);
        this.field_70180_af.func_187214_a(COLLAR_COLOR, EnumDyeColor.RED.func_176767_b());
    }

    public void func_70014_b(NBTTagCompound compound)
    {
        super.func_70014_b(compound);
        compound.func_74757_a("Angry", this.isAngry());
        compound.func_74774_a("CollarColor", (byte) this.getCollarColor().func_176767_b());
    }

    public void func_70037_a(NBTTagCompound compound)
    {
        super.func_70037_a(compound);
        this.setAngry(compound.func_74767_n("Angry"));
        if (compound.func_150297_b("CollarColor", 99))
        {
            this.setCollarColor(EnumDyeColor.func_176766_a(compound.func_74771_c("CollarColor")));
        }

    }

    public boolean func_184652_a(EntityPlayer player)
    {
        return !this.isAngry() && super.func_184652_a(player);
    }

    @SideOnly(Side.CLIENT)
    public void func_70103_a(byte id)
    {
        if (id == 8)
        {
            this.isShaking = true;
            this.timeWolfIsShaking = 0.0F;
            this.prevTimeWolfIsShaking = 0.0F;
        }
        else
        {
            super.func_70103_a(id);
        }
    }

    public void setTamed(boolean tamed)
    {
        super.setTamed(tamed);
        if (tamed)
        {
            this.func_110148_a(SharedMonsterAttributes.field_111267_a).func_111128_a(20.0D);
        }
        else
        {
            this.func_110148_a(SharedMonsterAttributes.field_111267_a).func_111128_a(8.0D);
        }
        this.func_110148_a(SharedMonsterAttributes.field_111264_e).func_111128_a(4.0D);
    }

    public boolean shouldAttackEntity(EntityLivingBase target, EntityLivingBase owner)
    {
        if (!(target instanceof EntityCreeper) && !(target instanceof EntityGhast))
        {
            if (target instanceof EntityWolfTFC)
            {
                EntityWolfTFC entitywolf = (EntityWolfTFC) target;
                if (entitywolf.isTamed() && entitywolf.getOwner() == owner)
                {
                    return false;
                }
            }
            if (target instanceof EntityPlayer && owner instanceof EntityPlayer && !((EntityPlayer) owner).func_96122_a((EntityPlayer) target))
            {
                return false;
            }
            else
            {
                return !(target instanceof AbstractHorse) || !((AbstractHorse) target).func_110248_bS();
            }
        }
        else
        {
            return false;
        }
    }

    protected void playStepSound(BlockPos pos, Block blockIn)
    {
        this.func_184185_a(SoundEvents.field_187869_gK, 0.15F, 1.0F);
    }

    protected SoundEvent func_184601_bQ(DamageSource damageSourceIn)
    {
        return SoundEvents.field_187863_gH;
    }

    protected SoundEvent func_184615_bR()
    {
        return SoundEvents.field_187859_gF;
    }

    protected float func_70599_aP()
    {
        return 0.4F;
    }

    public boolean func_70652_k(Entity entityIn)
    {
        boolean flag = entityIn.func_70097_a(DamageSource.func_76358_a(this), (float) ((int) this.func_110148_a(SharedMonsterAttributes.field_111264_e).func_111126_e()));
        if (flag)
        {
            this.func_174815_a(this, entityIn);
        }
        return flag;
    }
}
