/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.items.itemblock;

import java.util.List;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.client.resources.I18n;
import net.minecraft.client.util.ITooltipFlag;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.world.World;
import net.minecraftforge.fluids.FluidTank;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.items.ItemStackHandler;

import net.dries007.tfc.api.capability.size.Size;
import net.dries007.tfc.api.capability.size.Weight;
import net.dries007.tfc.api.util.TFCConstants;
import net.dries007.tfc.objects.te.TEBarrel;

@ParametersAreNonnullByDefault
public class ItemBlockBarrel extends ItemBlockTFC
{
    public ItemBlockBarrel(Block block)
    {
        super(block);
        this.func_77627_a(true);
    }

    @Override
    public int func_77647_b(int damage)
    {
        return damage;
    }

    @Override
    @Nonnull
    public String func_77667_c(@Nonnull ItemStack stack)
    {
        if (stack.func_77960_j() == 1)
        {
            return super.func_77658_a() + ".sealed";
        }

        return super.func_77658_a();
    }

    @SideOnly(Side.CLIENT)
    @Override
    public void func_77624_a(ItemStack stack, @Nullable World worldIn, List<String> tooltip, ITooltipFlag flagIn)
    {
        if (stack.func_77960_j() == 1)
        {
            NBTTagCompound compound = stack.func_77978_p();

            if (compound != null)
            {
                FluidTank tank = new FluidTank(0).readFromNBT(compound.func_74775_l("tank"));
                ItemStackHandler stackHandler = new ItemStackHandler();
                stackHandler.deserializeNBT(compound.func_74775_l("inventory"));
                ItemStack inventory = stackHandler.getStackInSlot(TEBarrel.SLOT_ITEM);

                if (tank.getFluid() == null || tank.getFluidAmount() == 0)
                {
                    if (inventory.func_190926_b())
                    {
                        tooltip.add(I18n.func_135052_a(TFCConstants.MOD_ID + ".tooltip.barrel_empty"));
                    }
                    else
                    {
                        tooltip.add(I18n.func_135052_a(TFCConstants.MOD_ID + ".tooltip.barrel_item", inventory.func_190916_E(), inventory.func_77973_b().func_77653_i(inventory)));
                    }
                }
                else
                {
                    tooltip.add(I18n.func_135052_a(TFCConstants.MOD_ID + ".tooltip.barrel_fluid", tank.getFluidAmount(), tank.getFluid().getLocalizedName()));

                    if (!inventory.func_190926_b())
                    {
                        tooltip.add(I18n.func_135052_a(TFCConstants.MOD_ID + ".tooltip.barrel_item_in_fluid", inventory.func_190916_E(), inventory.func_77973_b().func_77653_i(inventory)));
                    }
                }
            }
        }
    }

    @Nonnull
    @Override
    public Size getSize(@Nonnull ItemStack stack)
    {
        return Size.HUGE;
    }

    @Nonnull
    @Override
    public Weight getWeight(@Nonnull ItemStack stack)
    {
        return Weight.HEAVY;
    }

    @Override
    public boolean canStack(@Nonnull ItemStack stack)
    {
        return stack.func_77960_j() == 0;
    }
}
