/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.items.metal;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.stats.StatList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import net.dries007.tfc.api.types.Metal;

@ParametersAreNonnullByDefault
public class ItemMetalShears extends ItemMetalTool
{
    public ItemMetalShears(Metal metal, Metal.ItemType type)
    {
        super(metal, type);
    }

    @Override
    public boolean func_111207_a(ItemStack itemstack, net.minecraft.entity.player.EntityPlayer player, EntityLivingBase entity, net.minecraft.util.EnumHand hand)
    {
        if (entity.field_70170_p.field_72995_K)
        {
            return false;
        }
        if (entity instanceof net.minecraftforge.common.IShearable)
        {
            net.minecraftforge.common.IShearable target = (net.minecraftforge.common.IShearable) entity;
            BlockPos pos = new BlockPos(entity.field_70165_t, entity.field_70163_u, entity.field_70161_v);
            if (target.isShearable(itemstack, entity.field_70170_p, pos))
            {
                java.util.List<ItemStack> drops = target.onSheared(itemstack, entity.field_70170_p, pos,
                    net.minecraft.enchantment.EnchantmentHelper.func_77506_a(net.minecraft.init.Enchantments.field_185308_t, itemstack));

                java.util.Random rand = new java.util.Random();
                for (ItemStack stack : drops)
                {
                    EntityItem ent = entity.func_70099_a(stack, 1.0F);
                    if (ent != null)
                    {
                        ent.field_70181_x += rand.nextFloat() * 0.05F;
                        ent.field_70159_w += (rand.nextFloat() - rand.nextFloat()) * 0.1F;
                        ent.field_70179_y += (rand.nextFloat() - rand.nextFloat()) * 0.1F;
                    }
                }
                itemstack.func_77972_a(1, entity);
            }
            return true;
        }
        return false;
    }

    @Override
    public boolean onBlockStartBreak(ItemStack itemstack, BlockPos pos, net.minecraft.entity.player.EntityPlayer player)
    {
        if (player.field_70170_p.field_72995_K || player.field_71075_bZ.field_75098_d)
        {
            return false;
        }
        Block block = player.field_70170_p.func_180495_p(pos).func_177230_c();
        if (block instanceof net.minecraftforge.common.IShearable)
        {
            net.minecraftforge.common.IShearable target = (net.minecraftforge.common.IShearable) block;
            if (target.isShearable(itemstack, player.field_70170_p, pos))
            {
                java.util.List<ItemStack> drops = target.onSheared(itemstack, player.field_70170_p, pos,
                    net.minecraft.enchantment.EnchantmentHelper.func_77506_a(net.minecraft.init.Enchantments.field_185308_t, itemstack));
                java.util.Random rand = new java.util.Random();

                for (ItemStack stack : drops)
                {
                    float f = 0.7F;
                    double d = (double) (rand.nextFloat() * f) + (double) (1.0F - f) * 0.5D;
                    double d1 = (double) (rand.nextFloat() * f) + (double) (1.0F - f) * 0.5D;
                    double d2 = (double) (rand.nextFloat() * f) + (double) (1.0F - f) * 0.5D;
                    net.minecraft.entity.item.EntityItem entityitem = new net.minecraft.entity.item.EntityItem(player.field_70170_p, (double) pos.func_177958_n() + d, (double) pos.func_177956_o() + d1, (double) pos.func_177952_p() + d2, stack);
                    entityitem.func_174869_p();
                    player.field_70170_p.func_72838_d(entityitem);
                }

                itemstack.func_77972_a(1, player);
                //noinspection ConstantConditions
                player.func_71029_a(StatList.func_188055_a(block));
                player.field_70170_p.func_180501_a(pos, Blocks.field_150350_a.func_176223_P(), 11);
                return true;
            }
        }
        return false;
    }

    @Override
    public float func_150893_a(@Nonnull ItemStack stack, IBlockState state)
    {
        Block block = state.func_177230_c();

        if (block != Blocks.field_150321_G && state.func_185904_a() != Material.field_151584_j)
        {
            return block == Blocks.field_150325_L ? 5.0F : super.func_150893_a(stack, state);
        }
        else
        {
            return 15.0F;
        }
    }

    @Override
    public boolean func_179218_a(@Nonnull ItemStack stack, World worldIn, @Nonnull IBlockState state, @Nonnull BlockPos pos, @Nonnull EntityLivingBase entityLiving)
    {
        if (!worldIn.field_72995_K)
        {
            stack.func_77972_a(1, entityLiving);
        }

        Block block = state.func_177230_c();
        if (block instanceof net.minecraftforge.common.IShearable) return true;
        return state.func_185904_a() == Material.field_151584_j || block == Blocks.field_150321_G || block == Blocks.field_150329_H || block == Blocks.field_150395_bd || block == Blocks.field_150473_bD || block == Blocks.field_150325_L || super.func_179218_a(stack, worldIn, state, pos, entityLiving);
    }

    /**
     * Check whether this Item can harvest the given Block
     */
    public boolean func_150897_b(IBlockState blockIn)
    {
        Block block = blockIn.func_177230_c();
        return block == Blocks.field_150321_G || block == Blocks.field_150488_af || block == Blocks.field_150473_bD;
    }
}
