/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import java.util.ArrayList;
import java.util.List;
import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.EntitySelectors;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3i;
import net.minecraft.world.World;
import net.minecraftforge.common.util.Constants;

import net.dries007.tfc.ConfigTFC;
import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.api.util.IMetalObject;
import net.dries007.tfc.objects.blocks.BlockCharcoalPile;
import net.dries007.tfc.objects.blocks.BlockMolten;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.items.metal.ItemOreTFC;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.fuel.FuelManager;

import static net.dries007.tfc.objects.blocks.property.ILightableBlock.LIT;
import static net.minecraft.block.BlockHorizontal.FACING;

@ParametersAreNonnullByDefault
public class TEBloomery extends TEInventory implements ITickable
{
    //Gets the internal block, should be charcoal pile/bloom
    private static final Vec3i OFFSET_INTERNAL = new Vec3i(1, 0, 0);
    //Gets the external block, the front of the facing to dump contents in world.
    private static final Vec3i OFFSET_EXTERNAL = new Vec3i(-1, 0, 0);
    private List<ItemStack> oreStacks = new ArrayList<>();
    private List<ItemStack> fuelStacks = new ArrayList<>();

    private int maxFuel = 0, maxOre = 0, delayTimer = 0;
    private long burnTicksLeft;
    private EnumFacing direction = null;

    private BlockPos internalBlock = null, externalBlock = null;

    public TEBloomery()
    {
        super(0);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag)
    {
        oreStacks.clear();
        NBTTagList ores = tag.func_150295_c("ores", Constants.NBT.TAG_COMPOUND);
        for (int i = 0; i < ores.func_74745_c(); i++)
        {
            oreStacks.add(new ItemStack(ores.func_150305_b(i)));
        }

        fuelStacks.clear();
        NBTTagList fuels = tag.func_150295_c("fuels", Constants.NBT.TAG_COMPOUND);
        for (int i = 0; i < fuels.func_74745_c(); i++)
        {
            fuelStacks.add(new ItemStack(fuels.func_150305_b(i)));
        }
        burnTicksLeft = tag.func_74763_f("burnTicksLeft");
        super.func_145839_a(tag);
    }

    @Override
    @Nonnull
    public NBTTagCompound func_189515_b(NBTTagCompound tag)
    {
        NBTTagList ores = new NBTTagList();
        for (ItemStack stack : oreStacks)
        {
            ores.func_74742_a(stack.serializeNBT());
        }
        tag.func_74782_a("ores", ores);
        NBTTagList fuels = new NBTTagList();
        for (ItemStack stack : fuelStacks)
        {
            fuels.func_74742_a(stack.serializeNBT());
        }
        tag.func_74782_a("fuels", fuels);
        tag.func_74772_a("burnTicksLeft", burnTicksLeft);
        return super.func_189515_b(tag);
    }

    @Override
    public void onBreakBlock(World worldIn, BlockPos pos)
    {
        dumpItems();
        super.onBreakBlock(field_145850_b, pos);
    }

    public BlockPos getInternalBlock()
    {
        if (internalBlock == null)
        {
            internalBlock = field_174879_c.func_177981_b(OFFSET_INTERNAL.func_177956_o())
                .func_177967_a(direction, OFFSET_INTERNAL.func_177958_n())
                .func_177967_a(direction.func_176746_e(), OFFSET_INTERNAL.func_177952_p());
        }
        return internalBlock;
    }

    public BlockPos getExternalBlock()
    {
        if (externalBlock == null)
        {
            externalBlock = field_174879_c.func_177981_b(OFFSET_EXTERNAL.func_177956_o())
                .func_177967_a(direction, OFFSET_EXTERNAL.func_177958_n())
                .func_177967_a(direction.func_176746_e(), OFFSET_EXTERNAL.func_177952_p());
        }
        return externalBlock;
    }

    public boolean canIgnite()
    {
        if (field_145850_b.field_72995_K) return false;
        if (this.fuelStacks.size() < this.oreStacks.size() || this.oreStacks.isEmpty())
            return false;

        return isInternalBlockComplete();
    }

    public void onIgnite()
    {
        this.burnTicksLeft = ConfigTFC.GENERAL.bloomeryTime;
    }

    @Override
    public void func_73660_a()
    {
        if (field_145850_b.field_72995_K) return;
        if (--delayTimer <= 0)
        {
            delayTimer = 20;
            // Update multiblock status
            if (direction == null)
            {
                direction = field_145850_b.func_180495_p(field_174879_c).func_177229_b(field_185512_D);
            }

            int newMaxItems = BlocksTFC.BLOOMERY.getChimneyLevels(field_145850_b, getInternalBlock()) * 8;
            if (!BlocksTFC.BLOOMERY.isFormed(field_145850_b, getInternalBlock(), field_145850_b.func_180495_p(field_174879_c).func_177229_b(field_185512_D)))
            {
                newMaxItems = 0;
            }

            maxFuel = newMaxItems;
            maxOre = newMaxItems;
            while (maxOre < oreStacks.size())
            {
                //Structure lost one or more chimney levels
                InventoryHelper.func_180173_a(field_145850_b, getExternalBlock().func_177958_n(), getExternalBlock().func_177956_o(), getExternalBlock().func_177952_p(), oreStacks.get(0));
                oreStacks.remove(0);
            }
            while (maxFuel < fuelStacks.size())
            {
                InventoryHelper.func_180173_a(field_145850_b, getExternalBlock().func_177958_n(), getExternalBlock().func_177956_o(), getExternalBlock().func_177952_p(), fuelStacks.get(0));
                fuelStacks.remove(0);
            }
            if (maxOre <= 0)
            {
                //Structure became compromised
                field_145850_b.func_175655_b(field_174879_c, true);
                return;
            }
            if (!isInternalBlockComplete() && !fuelStacks.isEmpty())
            {
                dumpItems();
            }

            if (isInternalBlockComplete())
            {
                addItemsFromWorld();
            }

            updateSlagBlock(this.burnTicksLeft > 0);
        }
        IBlockState state = field_145850_b.func_180495_p(field_174879_c);
        if (state.func_177229_b(LIT))
        {
            if (--burnTicksLeft <= 0)
            {
                burnTicksLeft = 0;
                int totalOutput = 0;
                for (ItemStack stack : oreStacks)
                {
                    IMetalObject metal = (IMetalObject) stack.func_77973_b();
                    totalOutput += metal.getSmeltAmount(stack);
                }

                oreStacks.clear();
                fuelStacks.clear();

                field_145850_b.func_175656_a(getInternalBlock(), BlocksTFC.BLOOM.func_176223_P());

                TEBloom te = Helpers.getTE(field_145850_b, getInternalBlock(), TEBloom.class);
                if (te != null)
                {
                    te.setMetalAmount(totalOutput);
                }

                updateSlagBlock(false);
                field_145850_b.func_175656_a(field_174879_c, state.func_177226_a(LIT, false));
                func_70296_d();
            }
        }
    }

    private void dumpItems()
    {
        //Dump everything in world
        for (int i = 1; i < 4; i++)
        {
            if (field_145850_b.func_180495_p(getInternalBlock().func_177981_b(i)).func_177230_c() == BlocksTFC.MOLTEN)
            {
                field_145850_b.func_175698_g(getInternalBlock().func_177981_b(i));
            }
        }
        oreStacks.forEach(i -> InventoryHelper.func_180173_a(field_145850_b, getExternalBlock().func_177958_n(), getExternalBlock().func_177956_o(), getExternalBlock().func_177952_p(), i));
        fuelStacks.forEach(i -> InventoryHelper.func_180173_a(field_145850_b, getExternalBlock().func_177958_n(), getExternalBlock().func_177956_o(), getExternalBlock().func_177952_p(), i));
    }

    private boolean isInternalBlockComplete()
    {
        IBlockState inside = field_145850_b.func_180495_p(getInternalBlock());
        return inside.func_177230_c() == BlocksTFC.CHARCOAL_PILE && inside.func_177229_b(BlockCharcoalPile.LAYERS) >= 8;
    }

    private void addItemsFromWorld()
    {
        for (EntityItem entityItem : field_145850_b.func_175647_a(EntityItem.class, new AxisAlignedBB(getInternalBlock().func_177984_a(), getInternalBlock().func_177982_a(1, 4, 1)), EntitySelectors.field_94557_a))
        {
            ItemStack stack = entityItem.func_92059_d();
            if (FuelManager.isItemFuel(stack))
            {
                // Add fuel
                while (fuelStacks.size() < maxFuel)
                {
                    this.func_70296_d();
                    fuelStacks.add(stack.func_77979_a(1));
                    if (stack.func_190916_E() <= 0)
                    {
                        entityItem.func_70106_y();
                        break;
                    }
                }
            }
            else if (stack.func_77973_b() instanceof ItemOreTFC)
            {
                ItemOreTFC metal = (ItemOreTFC) stack.func_77973_b();
                if (metal.getMetal(stack) == Metal.WROUGHT_IRON || metal.getMetal(stack) == Metal.PIG_IRON)
                {
                    while (oreStacks.size() < maxOre)
                    {
                        this.func_70296_d();
                        oreStacks.add(stack.func_77979_a(1));
                        if (stack.func_190916_E() <= 0)
                        {
                            entityItem.func_70106_y();
                            break;
                        }
                    }
                }
            }
        }
    }

    private void updateSlagBlock(boolean cooking)
    {
        int slag = fuelStacks.size() + oreStacks.size();
        //If there's at least one item, show one layer so player knows that it is holding stacks
        int slagLayers = slag > 0 && slag < 4 ? 1 : slag / 4;
        for (int i = 1; i < 4; i++)
        {
            if (slagLayers > 0)
            {
                if (slagLayers >= 4)
                {
                    slagLayers -= 4;
                    field_145850_b.func_175656_a(getInternalBlock().func_177981_b(i), BlocksTFC.MOLTEN.func_176223_P().func_177226_a(LIT, cooking).func_177226_a(BlockMolten.LAYERS, 4));
                }
                else
                {
                    field_145850_b.func_175656_a(getInternalBlock().func_177981_b(i), BlocksTFC.MOLTEN.func_176223_P().func_177226_a(LIT, cooking).func_177226_a(BlockMolten.LAYERS, slagLayers));
                    slagLayers = 0;
                }
            }
            else
            {
                //Remove any surplus slag(ie: after cooking/structure became compromised)
                if (field_145850_b.func_180495_p(getInternalBlock().func_177981_b(i)).func_177230_c() == BlocksTFC.MOLTEN)
                {
                    field_145850_b.func_175698_g(getInternalBlock().func_177981_b(i));
                }
            }
        }
    }

}
