/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.ConfigTFC;
import net.dries007.tfc.Constants;
import net.dries007.tfc.objects.blocks.BlockCharcoalPile;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.blocks.wood.BlockLogPile;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.OreDictionaryHelper;

import static net.dries007.tfc.objects.blocks.BlockCharcoalPile.LAYERS;
import static net.dries007.tfc.objects.blocks.property.ILightableBlock.LIT;

@ParametersAreNonnullByDefault
@MethodsReturnNonnullByDefault
public class TELogPile extends TEInventory implements ITickable
{
    private static final int NUM_SLOTS = 4;

    private boolean burning;
    private int burnTicks;
    private boolean isContainerOpen;

    public TELogPile()
    {
        super(NUM_SLOTS);

        burnTicks = 0;
        burning = false;
    }

    public void setContainerOpen(boolean containerOpen)
    {
        isContainerOpen = containerOpen;
        setAndUpdateSlots(-1);
    }

    @Override
    public void setAndUpdateSlots(int slot)
    {
        if (!field_145850_b.field_72995_K)
        {
            for (int i = 0; i < 4; i++)
            {
                if (!inventory.getStackInSlot(i).func_190926_b())
                {
                    super.setAndUpdateSlots(slot);
                    return;
                }
            }
            if (!isContainerOpen)
            {
                field_145850_b.func_175698_g(field_174879_c);
            }
        }
        super.setAndUpdateSlots(slot);
    }

    @Override
    public void func_145839_a(NBTTagCompound nbt)
    {
        burnTicks = nbt.func_74762_e("burn_ticks");
        burning = nbt.func_74767_n("burning");
        super.func_145839_a(nbt);
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound nbt)
    {
        nbt.func_74768_a("burn_ticks", burnTicks);
        nbt.func_74757_a("burning", burning);
        return super.func_189515_b(nbt);
    }

    @Override
    public boolean canInteractWith(EntityPlayer player)
    {
        return !burning && field_145850_b.func_175625_s(field_174879_c) == this;
    }

    @Override
    public void func_73660_a()
    {
        if (!field_145850_b.field_72995_K)
        {
            if (burning)
            {
                if (burnTicks < ConfigTFC.GENERAL.pitKilnTime)
                {
                    burnTicks++;
                }
                else
                {
                    // Attempt to turn this log pile into charcoal
                    createCharcoal();
                }
            }
            else
            {
                if (field_145850_b.func_180495_p(field_174879_c.func_177984_a()).func_177230_c() == Blocks.field_150480_ab)
                {
                    burning = true;
                }
            }
        }
    }

    @Override
    public int getSlotLimit(int slot)
    {
        return 4;
    }

    @Override
    public boolean isItemValid(int slot, ItemStack stack)
    {
        return OreDictionaryHelper.doesStackMatchOre(stack, "logWood");
    }

    public boolean insertLog(ItemStack stack)
    {
        stack.func_190920_e(1);
        for (int i = 0; i < inventory.getSlots(); i++)
        {
            if (inventory.insertItem(i, stack, false).func_190926_b())
            {
                return true;
            }
        }
        return false;
    }

    public ItemStack getLog()
    {
        for (int i = 0; i < inventory.getSlots(); i++)
        {
            if (!inventory.getStackInSlot(i).func_190926_b())
            {
                return inventory.getStackInSlot(i);
            }
        }
        return ItemStack.field_190927_a;
    }

    public void light()
    {
        this.burning = true;
        tryLightNearby();
    }

    public int countLogs()
    {
        int logs = 0;
        for (int i = 0; i < inventory.getSlots(); i++)
        {
            logs += inventory.getStackInSlot(i).func_190916_E();
        }
        return logs;
    }

    public boolean isBurning()
    {
        return burning;
    }

    private void tryLightNearby()
    {
        for (EnumFacing side : EnumFacing.values())
        {
            IBlockState state = field_145850_b.func_180495_p(field_174879_c.func_177972_a(side));
            if (state.func_177230_c() instanceof BlockLogPile)
            {
                if (state.func_177229_b(LIT)) continue;
                field_145850_b.func_175656_a(field_174879_c.func_177972_a(side), state.func_177226_a(LIT, true));
                TELogPile tile = Helpers.getTE(field_145850_b, field_174879_c.func_177972_a(side), TELogPile.class);
                if (tile != null)
                {
                    tile.light();
                }
            }
        }
    }

    // This function does some magic **** to not create floating charcoal. Don't touch unless broken
    // - AlcatrazEscapee
    private void createCharcoal()
    {
        int j = 0;
        Block block;
        do
        {
            j++;
            block = field_145850_b.func_180495_p(field_174879_c.func_177979_c(j)).func_177230_c();
            // This is here so that the charcoal pile will collapse Bottom > Top
            // Because the pile scans Top > Bottom this is necessary to avoid floating blocks
            if (block instanceof BlockLogPile)
            {
                return;
            }
        } while (block == Blocks.field_150350_a || block instanceof BlockCharcoalPile);

        double logs = (double) countLogs();
        double log2 = 0.008d * logs * (logs + 42.5d) - 0.75d + 1.5d * Constants.RNG.nextFloat();
        int charcoal = (int) Math.min(8, Math.max(0, Math.round(log2)));
        if (charcoal == 0)
        {
            field_145850_b.func_175656_a(field_174879_c, Blocks.field_150350_a.func_176223_P());
            return;
        }
        if (j == 1)
        {
            // This log pile is at the bottom of the charcoal pit
            field_145850_b.func_175656_a(field_174879_c, BlocksTFC.CHARCOAL_PILE.func_176223_P().func_177226_a(LAYERS, charcoal));
            return;
        }
        for (int k = j - 1; k >= 0; k--)
        {
            // Climb back up from the bottom
            IBlockState state = field_145850_b.func_180495_p(field_174879_c.func_177979_c(k));
            if (state.func_177230_c() == Blocks.field_150350_a)
            {
                // If it hits air, place the remaining pile in that block
                field_145850_b.func_175656_a(field_174879_c.func_177979_c(k), BlocksTFC.CHARCOAL_PILE.func_176223_P().func_177226_a(LAYERS, charcoal));
                field_145850_b.func_175656_a(field_174879_c, Blocks.field_150350_a.func_176223_P());
                return;
            }

            if (state.func_177230_c() instanceof BlockCharcoalPile)
            {
                // Place what it can in the existing charcoal pit, then continue climbing
                charcoal += state.func_177229_b(LAYERS);
                int toCreate = charcoal > 8 ? 8 : charcoal;
                field_145850_b.func_175656_a(field_174879_c.func_177979_c(k), BlocksTFC.CHARCOAL_PILE.func_176223_P().func_177226_a(LAYERS, toCreate));
                charcoal -= toCreate;
            }

            if (charcoal <= 0)
            {
                field_145850_b.func_175656_a(field_174879_c, Blocks.field_150350_a.func_176223_P());
                return;
            }
        }
        // If you exit the loop, its arrived back at the original position OR needs to rest the original position, and needs to replace that block
        field_145850_b.func_175656_a(field_174879_c, BlocksTFC.CHARCOAL_PILE.func_176223_P().func_177226_a(LAYERS, charcoal));
    }
}
