/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumHand;
import net.minecraft.util.ITickable;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.recipes.LoomRecipe;
import net.dries007.tfc.api.registries.TFCRegistries;
import net.dries007.tfc.api.types.Tree;
import net.dries007.tfc.network.PacketLoomUpdate;
import net.dries007.tfc.objects.blocks.wood.BlockLoom;
import net.dries007.tfc.objects.inventory.ingredient.IIngredient;

@ParametersAreNonnullByDefault
public class TELoom extends TEInventory implements ITickable
{
    private Tree cachedWood;

    private int progress = 0;

    private LoomRecipe recipe = null;
    private long lastPushed = 0L;
    private boolean needsUpdate = false;

    public TELoom()
    {
        super(2);
    }

    @Nullable
    public Tree getWood()
    {
        if (cachedWood == null)
        {
            if (field_145850_b != null)
            {
                cachedWood = ((BlockLoom) field_145850_b.func_180495_p(field_174879_c).func_177230_c()).wood;
            }
        }
        return cachedWood;
    }

    @Override
    public void func_145839_a(NBTTagCompound nbt)
    {
        super.func_145839_a(nbt);
        progress = nbt.func_74762_e("progress");
        recipe = nbt.func_74764_b("recipe") ? TFCRegistries.LOOM.getValue(new ResourceLocation(nbt.func_74779_i("recipe"))) : null;
    }

    @Override
    @Nonnull
    public NBTTagCompound func_189515_b(NBTTagCompound nbt)
    {
        super.func_189515_b(nbt);
        nbt.func_74768_a("progress", progress);
        if (recipe != null)
        {
            //noinspection ConstantConditions
            nbt.func_74778_a("recipe", recipe.getRegistryName().toString());
        }
        return nbt;
    }

    public void onReceivePacket(long lastPushed)
    {
        this.lastPushed = lastPushed;
    }

    @SideOnly(Side.CLIENT)
    public double getAnimPos()
    {
        int time = (int) (field_145850_b.func_82737_E() - lastPushed);
        if (time < 10)
        {
            return Math.sin((Math.PI / 20) * time) * 0.23125;
        }
        else if (time < 20)
        {
            return Math.sin((Math.PI / 20) * (20 - time)) * 0.23125;
        }
        return 0;
    }

    @SideOnly(Side.CLIENT)
    public String getAnimElement()
    {
        return (progress % 2 == 0) ? "u" : "l";
    }

    public boolean onRightClick(EntityPlayer player)
    {
        if (player.func_70093_af())
        {
            if (!inventory.getStackInSlot(0).func_190926_b() && progress == 0)
            {
                ItemStack heldItem = player.func_184586_b(EnumHand.MAIN_HAND);

                if (heldItem.func_190926_b())
                {
                    ItemStack temp = inventory.getStackInSlot(0).func_77946_l();
                    temp.func_190920_e(1);
                    player.func_191521_c(temp);
                    inventory.getStackInSlot(0).func_190918_g(1);

                    if (inventory.getStackInSlot(0).func_190926_b())
                    {
                        recipe = null;
                    }
                    updateBlock();
                    return true;
                }
            }
        }
        else
        {
            ItemStack heldItem = player.func_184586_b(EnumHand.MAIN_HAND);

            if (inventory.getStackInSlot(0).func_190926_b() && inventory.getStackInSlot(1).func_190926_b() && LoomRecipe.get(heldItem) != null)
            {
                inventory.setStackInSlot(0, heldItem.func_77946_l());
                inventory.getStackInSlot(0).func_190920_e(1);
                heldItem.func_190918_g(1);
                recipe = LoomRecipe.get(inventory.getStackInSlot(0));

                updateBlock();
                return true;
            }
            else if (!inventory.getStackInSlot(0).func_190926_b())
            {
                if (IIngredient.of(inventory.getStackInSlot(0)).testIgnoreCount(heldItem) && recipe.getInputCount() > inventory.getStackInSlot(0).func_190916_E())
                {
                    heldItem.func_190918_g(1);
                    inventory.getStackInSlot(0).func_190917_f(1);

                    updateBlock();
                    return true;
                }
            }

            if (recipe != null && heldItem.func_190926_b())
            {
                if (recipe.getInputCount() == inventory.getStackInSlot(0).func_190916_E() && progress < recipe.getStepCount() && !needsUpdate)
                {
                    if (!field_145850_b.field_72995_K)
                    {
                        long time = field_145850_b.func_82737_E() - lastPushed;
                        if (time < 20)
                            return true;
                        lastPushed = field_145850_b.func_82737_E();
                        needsUpdate = true;

                        TerraFirmaCraft.getNetwork().sendToDimension(new PacketLoomUpdate(this, lastPushed), field_145850_b.field_73011_w.getDimension());
                    }
                    return true;
                }
            }

            if (!inventory.getStackInSlot(1).func_190926_b())
            {
                if (heldItem.func_190926_b())
                {
                    player.func_191521_c(inventory.getStackInSlot(1).func_77946_l());
                    inventory.setStackInSlot(1, ItemStack.field_190927_a);
                    progress = 0;
                    recipe = null;
                    updateBlock();
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public void func_73660_a()
    {
        if (recipe != null)
        {
            if (needsUpdate)
            {
                if (field_145850_b.func_82737_E() - lastPushed >= 20)
                {
                    needsUpdate = false;
                    progress++;

                    if (progress == recipe.getStepCount())
                    {
                        inventory.setStackInSlot(0, ItemStack.field_190927_a);
                        inventory.setStackInSlot(1, recipe.getOutputItem());
                    }
                    updateBlock();
                }
            }
        }
    }

    public int getMaxInputCount()
    {
        return (recipe == null) ? 1 : recipe.getInputCount();
    }

    public int getCount()
    {
        return inventory.getStackInSlot(0).func_190916_E();
    }

    public int getMaxProgress()
    {
        return (recipe == null) ? 1 : recipe.getStepCount();
    }

    public int getProgress()
    {
        return progress;
    }

    public boolean hasRecipe()
    {
        return recipe != null;
    }

    public ResourceLocation getInProgressTexture()
    {
        return recipe.getInProgressTexture();
    }

    private void updateBlock()
    {
        IBlockState state = field_145850_b.func_180495_p(field_174879_c);
        field_145850_b.func_184138_a(field_174879_c, state, state, 3);
        func_70296_d();
    }
}
